@@ -2,12 +2,10 @@
  * botcmd.c -- handles:
  *   commands that comes across the botnet
  *   userfile transfer and update commands from sharebots
- *
- * $Id: botcmd.c,v 1.53 2011/02/13 14:19:33 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,6 +30,11 @@
 
 extern char botnetnick[], ver[], admin[], network[], motdfile[];
 extern int dcc_total, remote_boots, noshare;
+#ifdef TLS
+extern int tls_vfybots;
+extern int tls_vfyclients;
+extern SSL_CTX *ssl_ctx;
+#endif
 extern struct dcc_t *dcc;
 extern struct chanset_t *chanset;
 extern struct userrec *userlist;
@@ -120,7 +123,7 @@ static void bot_chan2(int idx, char *msg)
     *p = 0;
   p = strchr(from, '@');
   if (p) {
-    snprintf(TBUF, sizeof(TBUF), "<%s> %s", from, msg);
+    egg_snprintf(TBUF, sizeof(TBUF), "<%s> %s", from, msg);
     *p = 0;
     if (!partyidle(p + 1, from)) {
       *p = '@';
@@ -168,7 +171,7 @@ static void bot_chat(int idx, char *par)
     fake_alert(idx, "direction", from);
     return;
   }
-  chatout("*** (%s) %s\n", from, par);
+  putlog(LOG_BOTMSG, "*", "(%s) %s", from, par);
   botnet_send_chat(idx, from, par);
   check_tcl_bcst(from, -1, par);
 }
@@ -296,8 +299,7 @@ static void bot_bye(int idx, char *par)
                  BOT_DISCONNECTED, dcc[idx].nick, par[0] ?
                  par : "No reason", bots, (bots != 1) ?
                  "s" : "", users, (users != 1) ? "s" : "");
-  putlog(LOG_BOTS, "*", "%s", s);
-  chatout("*** %s\n", s);
+  putlog(LOG_BOTS, "*", "%s.", s);
   botnet_send_unlinked(idx, dcc[idx].nick, s);
   dprintf(idx, "*bye\n");
   killsock(dcc[idx].sock);
@@ -307,8 +309,11 @@ static void bot_bye(int idx, char *par)
 static void remote_tell_who(int idx, char *nick, int chan)
 {
   int i = 10, k, l, ok = 0;
-  char s[1024], *realnick;
+  /* botnet_send_priv truncates at 450 */
+  char s[450] = "Channels: ", *realnick;
   struct chanset_t *c;
+  /* usable size of channelslist */
+  size_t ssize = sizeof(s) - 1;
 
   realnick = strchr(nick, ':');
   if (realnick)
@@ -316,24 +321,46 @@ static void remote_tell_who(int idx, char *nick, int chan)
   else
     realnick = nick;
   putlog(LOG_BOTS, "*", "#%s# who", realnick);
-  strcpy(s, "Channels: ");
-  for (c = chanset; c; c = c->next)
+  for (c = chanset; c; c = c->next) {
     if (!channel_secret(c) && !channel_inactive(c)) {
       l = strlen(c->dname);
-      if (i + l < 1021) {
-        if (i > 10)
-          sprintf(s, "%s, %s", s, c->dname);
-        else {
-          strcpy(s, c->dname);
-          i += (l + 2);
+      /* for 2nd and more chans we need to prepend ','; i is > 10 */
+      if (i > 10) {
+        /* check if ", #chan" fits or if there is a next chan, if ", #chan," fits */
+	if ((c->next && i + l + 3 <= ssize) || (!c->next && i + l + 2 <= ssize)) {
+          strcat(s, ", ");
+          i += 2;
+        } else {
+          /* output and prepare for more, there should always be place for ',' */
+          strcat(s, ",");
+          botnet_send_priv(idx, botnetnick, nick, NULL, "%s", s);
+          strcpy(s, "          ");
+          i = 10;
         }
       }
+
+      i += l;
+      strncat(s, c->dname, ssize);
+
+      /* check if we need to trunc, normally only for first chans on the line */
+      if (i > ssize) {
+        unsigned int trunc = 4;
+        if (c->next) {
+          /* more to come, leave place for ',' */
+          ++trunc;
+        }
+        strcpy(s + ssize - trunc, " ...");
+        s[ssize - trunc + 4] = '\0';
+        i = ssize - trunc + 4;
+      }
     }
+  }
   if (i > 10) {
-    botnet_send_priv(idx, botnetnick, nick, NULL, "%s (%s)", s, ver);
+    botnet_send_priv(idx, botnetnick, nick, NULL, "%s", s);
   } else
-    botnet_send_priv(idx, botnetnick, nick, NULL, "%s (%s)", BOT_NOCHANNELS,
-                     ver);
+    botnet_send_priv(idx, botnetnick, nick, NULL, "%s", BOT_NOCHANNELS);
+
+  botnet_send_priv(idx, botnetnick, nick, NULL, "Version: %s", ver);
   if (admin[0])
     botnet_send_priv(idx, botnetnick, nick, NULL, "Admin: %s", admin);
   if (chan == 0)
@@ -669,7 +696,7 @@ static void bot_nlinked(int idx, char *par)
     dprintf(idx, "error %s\n", BOT_YOUREALEAF);
   }
   if (s[0]) {
-    chatout("*** %s\n", s);
+    putlog(LOG_BOTS, "*", "%s.", s);
     botnet_send_unlinked(idx, dcc[idx].nick, s);
     dprintf(idx, "bye %s\n", BOT_ILLEGALLINK);
     killsock(dcc[idx].sock);
@@ -689,7 +716,7 @@ static void bot_nlinked(int idx, char *par)
     i = base64_to_int(par);
   botnet_send_nlinked(idx, newbot, next, x, i);
   if (x == '!') {
-    chatout("*** (%s) %s %s.\n", next, NET_LINKEDTO, newbot);
+    putlog(LOG_BOTMSG, "*", "(%s) %s %s.", next, NET_LINKEDTO, newbot);
     x = '-';
   }
   addbot(newbot, dcc[idx].nick, next, x, i);
@@ -714,7 +741,7 @@ static void bot_linked(int idx, char *par)
   simple_sprintf(s, "%s %s (%s) (lost %d bot%s and %d user%s",
                  MISC_DISCONNECTED, dcc[idx].nick, MISC_OUTDATED,
                  bots, (bots != 1) ? "s" : "", users, (users != 1) ? "s" : "");
-  chatout("*** %s\n", s);
+  putlog(LOG_BOTS, "*", "%s.", s);
   botnet_send_unlinked(idx, dcc[idx].nick, s);
   killsock(dcc[idx].sock);
   lostdcc(idx);
@@ -733,7 +760,7 @@ static void bot_unlinked(int idx, char *par)
     fake_alert(idx, "direction", bot);
   else if (i >= 0) {            /* Valid bot downstream of idx */
     if (par[0])
-      chatout("*** (%s) %s\n", lastbot(bot), par);
+      putlog(LOG_BOTMSG, "*", "(%s) %s", lastbot(bot), par);
     botnet_send_unlinked(idx, bot, par);
     unvia(idx, findbot(bot));
     rembot(bot);
@@ -848,12 +875,11 @@ static void bot_reject(int idx, char *par)
       char s[1024];
 
       /* I'm the connection to the rejected bot */
-      putlog(LOG_BOTS, "*", "%s %s %s", from, MISC_REJECTED, dcc[i].nick);
       dprintf(i, "bye %s\n", par[0] ? par : MISC_REJECTED);
       simple_sprintf(s, "%s %s (%s: %s)",
                      MISC_DISCONNECTED, dcc[i].nick, from,
                      par[0] ? par : MISC_REJECTED);
-      chatout("*** %s\n", s);
+      putlog(LOG_BOTS, "*", "%s.", s);
       botnet_send_unlinked(i, dcc[i].nick, s);
       killsock(dcc[i].sock);
       lostdcc(i);
@@ -913,7 +939,7 @@ static void bot_thisbot(int idx, char *par)
     dprintf(idx, "bye %s\n", MISC_IMPOSTER);
     simple_sprintf(s, "%s %s (%s)", MISC_DISCONNECTED, dcc[idx].nick,
                    MISC_IMPOSTER);
-    chatout("*** %s\n", s);
+    putlog(LOG_BOTS, "*", "%s.", s);
     botnet_send_unlinked(idx, dcc[idx].nick, s);
     unvia(idx, findbot(dcc[idx].nick));
     killsock(dcc[idx].sock);
@@ -926,7 +952,7 @@ static void bot_thisbot(int idx, char *par)
   noshare = 1;
   change_handle(dcc[idx].user, par);
   noshare = 0;
-  strcpy(dcc[idx].nick, par);
+  strncpyz(dcc[idx].nick, par, sizeof dcc[idx].nick);
 }
 
 static void bot_handshake(int idx, char *par)
@@ -1020,17 +1046,19 @@ static void bot_motd(int idx, char *par)
     if (vv != NULL) {
       botnet_send_priv(idx, botnetnick, who, NULL, "--- %s\n", MISC_MOTDFILE);
       help_subst(NULL, NULL, 0, irc, NULL);
-      while (!feof(vv)) {
-        fgets(s, 120, vv);
-        if (!feof(vv)) {
-          if (s[strlen(s) - 1] == '\n')
-            s[strlen(s) - 1] = 0;
-          if (!s[0])
-            strcpy(s, " ");
-          help_subst(s, who, &fr, HELP_DCC, dcc[idx].nick);
-          if (s[0])
-            botnet_send_priv(idx, botnetnick, who, NULL, "%s", s);
-        }
+      /* don't check for feof after fgets, skips last line if it has no \n (ie on windows) */
+      while (!feof(vv) && fgets(s, sizeof TBUF, vv) != NULL) {
+        if (s[strlen(s) - 1] == '\n')
+          s[strlen(s) - 1] = 0;
+        if (!s[0])
+          strcpy(s, " ");
+        help_subst(s, who, &fr, HELP_DCC, dcc[idx].nick);
+        if (s[0])
+          botnet_send_priv(idx, botnetnick, who, NULL, "%s", s);
+      }
+      /* fgets == NULL means error or empty file, so check for error */
+      if (ferror(vv)) {
+        putlog(LOG_MISC, "*", "Error reading MOTD file");
       }
       fclose(vv);
     } else
@@ -1446,6 +1474,32 @@ static void bot_versions(int sock, char *par)
   }
 }
 
+#ifdef TLS
+/* Negotiate an encrypted session over the existing link
+ * starttls <c/s>
+ */
+static void bot_starttls(int idx, char *par)
+{
+  char *con;
+  /* We're already using ssl, ignore the request */
+  if (dcc[idx].ssl)
+    return;
+
+  con = newsplit(&par);
+  /* check who's going to play the server */
+  if (!egg_strcasecmp(con, "s")) { /* we're server */
+    putlog(LOG_BOTS, "*", "Got STARTTLS from %s. Replying...", dcc[idx].nick);
+    dprintf(idx, "starttls c\n");
+    ssl_handshake(dcc[idx].sock, TLS_LISTEN, tls_vfybots, LOG_BOTS,
+                  dcc[idx].host, NULL);
+  } else { /* we're client, don't reply or we'll be in the loop forever */
+    ssl_handshake(dcc[idx].sock, TLS_CONNECT, tls_vfybots, LOG_BOTS,
+                  dcc[idx].host, NULL);
+  }
+  dcc[idx].ssl = 1;
+}
+#endif
+
 /* BOT COMMANDS
  *
  * function call should be:
@@ -1531,6 +1585,9 @@ botcmd_t C_bot[] =
   {"reject",     (IntFunc) bot_reject},
 #endif
   {"s",          (IntFunc) bot_share},
+#ifdef TLS
+  {"starttls",   (IntFunc) bot_starttls},
+#endif
   {"t",          (IntFunc) bot_trace},
   {"tb",         (IntFunc) bot_thisbot},
   {"td",         (IntFunc) bot_traced},
