@@ -6,12 +6,10 @@
  *   rejecting a bot
  *   linking, unlinking, and relaying to another bot
  *   pinging the bots periodically and checking leaf status
- *
- * $Id: botnet.c,v 1.66 2011/02/13 14:19:33 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -44,7 +42,9 @@ int tands = 0;                     /* Number of bots on the botnet */
 int parties = 0;                   /* Number of people on the botnet */
 char botnetnick[HANDLEN + 1] = ""; /* Botnet nickname */
 int share_unlinks = 0;             /* Allow remote unlinks of my sharebots? */
-
+#ifdef TLS
+int tls_vfybots = 0;               /* Verify SSL certificates from bots? */
+#endif
 
 int expmem_botnet()
 {
@@ -136,6 +136,11 @@ int partysock(char *bot, char *nick)
   return 0;
 }
 
+/* Set the botnetnick and truncate as necessary */
+void set_botnetnick(const char *newnick) {
+  strncpyz(botnetnick, newnick, sizeof botnetnick);
+}
+
 /* New botnet member
  */
 int addparty(char *bot, char *nick, int chan, char flag, int sock,
@@ -751,7 +756,7 @@ void dump_links(int z)
 #endif
       l = simple_sprintf(x, "n %s %s %c%D\n", bot->bot, p,
                          bot->share, bot->ver);
-    tputs(dcc[z].sock, x, l);
+    dprint(z, x, l);
   }
   if (!(bot_flags(dcc[z].user) & BOT_ISOLATE)) {
     /* Dump party line members */
@@ -761,7 +766,7 @@ void dump_links(int z)
             (dcc[i].u.chat->channel < GLOBAL_CHANS)) {
 #ifndef NO_OLD_BOTNET
           if (b_numver(z) < NEAT_BOTNET)
-            l = simple_sprintf(x, "join %s %s %d %c%d %s\n",
+             l =simple_sprintf(x, "join %s %s %d %c%d %s\n",
                                botnetnick, dcc[i].nick,
                                dcc[i].u.chat->channel, geticon(i),
                                dcc[i].sock, dcc[i].host);
@@ -771,13 +776,13 @@ void dump_links(int z)
                                botnetnick, dcc[i].nick,
                                dcc[i].u.chat->channel, geticon(i),
                                dcc[i].sock, dcc[i].host);
-          tputs(dcc[z].sock, x, l);
+          dprint(z, x, l);
 #ifndef NO_OLD_BOTNET
           if (b_numver(z) < NEAT_BOTNET) {
             if (dcc[i].u.chat->away) {
               l = simple_sprintf(x, "away %s %d %s\n", botnetnick,
                                  dcc[i].sock, dcc[i].u.chat->away);
-              tputs(dcc[z].sock, x, l);
+              dprint(z, x, l);
             }
             l = simple_sprintf(x, "idle %s %d %d\n", botnetnick,
                                dcc[i].sock, now - dcc[i].timeval);
@@ -786,7 +791,7 @@ void dump_links(int z)
             l = simple_sprintf(x, "i %s %D %D %s\n", botnetnick,
                                dcc[i].sock, now - dcc[i].timeval,
                                dcc[i].u.chat->away ? dcc[i].u.chat->away : "");
-          tputs(dcc[z].sock, x, l);
+          dprint(z, x, l);
         }
       }
     }
@@ -803,14 +808,14 @@ void dump_links(int z)
                            party[i].bot, party[i].nick,
                            party[i].chan, party[i].flag,
                            party[i].sock, party[i].from);
-      tputs(dcc[z].sock, x, l);
+      dprint(z, x, l);
       if ((party[i].status & PLSTAT_AWAY) || (party[i].timer != 0)) {
 #ifndef NO_OLD_BOTNET
         if (b_numver(z) < NEAT_BOTNET) {
           if (party[i].status & PLSTAT_AWAY) {
             l = simple_sprintf(x, "away %s %d %s\n", party[i].bot,
                                party[i].sock, party[i].away);
-            tputs(dcc[z].sock, x, l);
+            dprint(z, x, l);
           }
           l = simple_sprintf(x, "idle %s %d %d\n", party[i].bot,
                              party[i].sock, now - party[i].timer);
@@ -819,7 +824,7 @@ void dump_links(int z)
           l = simple_sprintf(x, "i %s %D %D %s\n", party[i].bot,
                              party[i].sock, now - party[i].timer,
                              party[i].away ? party[i].away : "");
-        tputs(dcc[z].sock, x, l);
+        dprint(z, x, l);
       }
     }
   }
@@ -842,7 +847,7 @@ int bots_in_subtree(tand_t *bot)
   if (!bot)
     return 0;
   for (b = tandbot; b; b = b->next) {
-    if (b->bot && (b->uplink == bot)) {
+    if (b->bot[0] && (b->uplink == bot)) {
       nr += bots_in_subtree(b);
     }
   }
@@ -861,7 +866,7 @@ int users_in_subtree(tand_t *bot)
     if (!egg_strcasecmp(party[i].bot, bot->bot))
       nr++;
   for (b = tandbot; b; b = b->next)
-    if (b->bot && (b->uplink == bot))
+    if (b->bot[0] && (b->uplink == bot))
       nr += users_in_subtree(b);
   return nr;
 }
@@ -921,7 +926,8 @@ int botunlink(int idx, char *nick, char *reason, char *from)
                          (users != 1) ? "s" : "");
           dprintf(i, "bye No reason\n");
         }
-        chatout("*** %s\n", s);
+        putlog(LOG_BOTS, "*", "%s.", s);
+        dprintf(idx, "%s.\n", s);
         botnet_send_unlinked(i, dcc[i].nick, s);
         killsock(dcc[i].sock);
         lostdcc(i);
@@ -991,6 +997,10 @@ int botlink(char *linker, int idx, char *nick)
   } else if (in_chain(nick) && (idx != -3)) {
     if (idx >= 0)
       dprintf(idx, "%s\n", BOT_ALREADYLINKED);
+  } else if (bot_flags(u) & BOT_REJECT) {
+    if (idx >= 0) {
+      dprintf(idx, "%s %s\n", BOT_REJECTING, nick);
+    }
   } else {
     for (i = 0; i < dcc_total; i++)
       if ((dcc[i].user == u) &&
@@ -1015,12 +1025,25 @@ int botlink(char *linker, int idx, char *nick)
       correct_handle(nick);
 
       if (idx > -2)
+#ifdef IPV6
+      {
+        if (strchr(bi->address, ':'))
+          putlog(LOG_BOTS, "*", "%s %s at [%s]:%d ...", BOT_LINKING, nick,
+                 bi->address, bi->telnet_port);
+        else
+          putlog(LOG_BOTS, "*", "%s %s at %s:%d ...", BOT_LINKING, nick,
+                 bi->address, bi->telnet_port);
+      }
+#else
         putlog(LOG_BOTS, "*", "%s %s at %s:%d ...", BOT_LINKING, nick,
                bi->address, bi->telnet_port);
-
+#endif
       i = new_dcc(&DCC_DNSWAIT, sizeof(struct dns_info));
       dcc[i].timeval = now;
       dcc[i].port = bi->telnet_port;
+#ifdef TLS
+      dcc[i].ssl = (bi->ssl & TLS_BOT);
+#endif
       dcc[i].user = u;
       strcpy(dcc[i].nick, nick);
       strcpy(dcc[i].host, bi->address);
@@ -1053,22 +1076,29 @@ static void botlink_resolve_failure(int i)
 
 static void botlink_resolve_success(int i)
 {
+  int ret;
   int idx = dcc[i].u.dns->ibuf;
   char *linker = dcc[i].u.dns->cptr;
 
-  dcc[i].addr = dcc[i].u.dns->ip;
   changeover_dcc(i, &DCC_FORK_BOT, sizeof(struct bot_info));
   dcc[i].timeval = now;
-  strcpy(dcc[i].u.bot->linker, linker);
+  strncpyz(dcc[i].u.bot->linker, linker, sizeof dcc[i].u.bot->linker);
   strcpy(dcc[i].u.bot->version, "(primitive bot)");
   dcc[i].u.bot->numver = idx;
   dcc[i].u.bot->port = dcc[i].port;     /* Remember where i started */
-  dcc[i].sock = getsock(SOCK_STRONGCONN);
   nfree(linker);
-  if (dcc[i].sock < 0 ||
-      open_telnet_raw(dcc[i].sock, iptostr(htonl(dcc[i].addr)),
-                      dcc[i].port) < 0)
+  setsnport(dcc[i].sockname, dcc[i].port);
+  dcc[i].sock = getsock(dcc[i].sockname.family, SOCK_STRONGCONN);
+  if (dcc[i].sock < 0)
     failed_link(i);
+  ret = open_telnet_raw(dcc[i].sock, &dcc[i].sockname);
+  if (ret < 0)
+    failed_link(i);
+#ifdef TLS
+  else if (dcc[i].ssl && ssl_handshake(dcc[i].sock, TLS_CONNECT,
+           tls_vfybots, LOG_BOTS, dcc[i].host, NULL))
+    failed_link(i);
+#endif
 }
 
 static void failed_tandem_relay(int idx)
@@ -1099,15 +1129,20 @@ static void failed_tandem_relay(int idx)
     return;
   }
   killsock(dcc[idx].sock);
-  dcc[idx].sock = getsock(SOCK_STRONGCONN);
+  if (!dcc[idx].sockname.addrlen)
+    (void) setsockname(&dcc[idx].sockname, dcc[idx].host, dcc[idx].port, 0);
+  dcc[idx].sock = getsock(dcc[idx].sockname.family, SOCK_STRONGCONN);
   dcc[uidx].u.relay->sock = dcc[idx].sock;
   dcc[idx].port++;
   dcc[idx].timeval = now;
   if (dcc[idx].sock < 0 ||
-      open_telnet_raw(dcc[idx].sock, dcc[idx].addr ?
-                      iptostr(htonl(dcc[idx].addr)) :
-                      dcc[idx].host, dcc[idx].port) < 0)
+      open_telnet_raw(dcc[idx].sock, &dcc[idx].sockname) < 0)
     failed_tandem_relay(idx);
+#ifdef TLS
+  else if (dcc[idx].ssl && ssl_handshake(dcc[idx].sock, TLS_CONNECT,
+           tls_vfybots, LOG_BOTS, dcc[idx].host, NULL))
+    failed_tandem_relay(idx);
+#endif
 }
 
 
@@ -1145,7 +1180,7 @@ void tandem_relay(int idx, char *nick, register int i)
     return;
   }
 
-  dcc[i].sock = getsock(SOCK_STRONGCONN | SOCK_VIRTUAL);
+  dcc[i].sock = getsock(AF_INET, SOCK_STRONGCONN | SOCK_VIRTUAL);
   if (dcc[i].sock < 0) {
     lostdcc(i);
     dprintf(idx, "%s\n", MISC_NOFREESOCK);
@@ -1153,10 +1188,19 @@ void tandem_relay(int idx, char *nick, register int i)
   }
 
   dcc[i].port = bi->relay_port;
+#ifdef TLS
+  dcc[i].ssl = (bi->ssl & TLS_RELAY);
+#endif
   dcc[i].addr = 0L;
   strcpy(dcc[i].nick, nick);
   dcc[i].user = u;
   strcpy(dcc[i].host, bi->address);
+#ifdef IPV6
+  if (strchr(bi->address, ':'))
+    dprintf(idx, "%s %s @ [%s]:%d ...\n", BOT_CONNECTINGTO, nick,
+            bi->address, bi->relay_port);
+  else
+#endif
   dprintf(idx, "%s %s @ %s:%d ...\n", BOT_CONNECTINGTO, nick,
           bi->address, bi->relay_port);
   dprintf(idx, "%s\n", BOT_BYEINFO1);
@@ -1165,8 +1209,8 @@ void tandem_relay(int idx, char *nick, register int i)
   dcc[idx].u.relay = get_data_ptr(sizeof(struct relay_info));
   dcc[idx].u.relay->chat = ci;
   dcc[idx].u.relay->old_status = dcc[idx].status;
-  dcc[idx].u.relay->sock = dcc[i].sock;
   dcc[i].timeval = now;
+  dcc[idx].u.relay->sock = dcc[i].sock;
   dcc[i].u.dns->ibuf = dcc[idx].sock;
   dcc[i].u.dns->host = get_data_ptr(strlen(bi->address) + 1);
   strcpy(dcc[i].u.dns->host, bi->address);
@@ -1209,7 +1253,6 @@ static void tandem_relay_resolve_success(int i)
 {
   int sock = dcc[i].u.dns->ibuf;
 
-  dcc[i].addr = dcc[i].u.dns->ip;
   changeover_dcc(i, &DCC_FORK_RELAY, sizeof(struct relay_info));
   dcc[i].u.relay->chat = get_data_ptr(sizeof(struct chat_info));
 
@@ -1223,9 +1266,22 @@ static void tandem_relay_resolve_success(int i)
   dcc[i].u.relay->chat->line_count = 0;
   dcc[i].u.relay->chat->current_lines = 0;
   dcc[i].timeval = now;
-  if (open_telnet_raw(dcc[i].sock, iptostr(htonl(dcc[i].addr)),
-                      dcc[i].port) < 0)
+#ifdef IPV6
+  if (dcc[i].sockname.family == AF_INET6) {
+    killsock(dcc[i].sock);
+    dcc[i].sock = getsock(AF_INET6, SOCK_STRONGCONN | SOCK_VIRTUAL);
+    dcc[i].sockname.addr.s6.sin6_port = htons(dcc[i].port);
+    dcc[i].u.relay->sock = dcc[i].sock;
+  } else
+#endif
+    dcc[i].sockname.addr.s4.sin_port = htons(dcc[i].port);
+  if (open_telnet_raw(dcc[i].sock, &dcc[i].sockname) < 0)
     failed_tandem_relay(i);
+#ifdef TLS
+  else if (dcc[i].ssl && ssl_handshake(dcc[i].sock, TLS_CONNECT,
+           tls_vfybots, LOG_BOTS, dcc[i].host, NULL))
+    failed_tandem_relay(i);
+#endif
 }
 
 /* Input from user before connect is ready
@@ -1440,7 +1496,7 @@ static void dcc_relay(int idx, char *buf, int j)
         for (e = p + 2; *e != 'm' && *e; e++);
         strcpy((char *) p, (char *) (e + 1));
       } else if (*p == '\r')
-        strcpy((char *) p, (char *) (p + 1));
+        memmove(p, p + 1, strlen((char *)p + 1) + 1);
     }
     if (!buf[0])
       dprintf(-dcc[idx].u.relay->sock, " \n");
@@ -1605,14 +1661,13 @@ void check_botnet_pings()
       if (dcc[i].status & STAT_PINGED) {
         char s[1024];
 
-        putlog(LOG_BOTS, "*", "%s: %s", BOT_PINGTIMEOUT, dcc[i].nick);
         bot = findbot(dcc[i].nick);
         bots = bots_in_subtree(bot);
         users = users_in_subtree(bot);
         simple_sprintf(s, "%s: %s (lost %d bot%s and %d user%s)",
                        BOT_PINGTIMEOUT, dcc[i].nick, bots,
                        (bots != 1) ? "s" : "", users, (users != 1) ? "s" : "");
-        chatout("*** %s\n", s);
+        putlog(LOG_BOTS, "*", "%s.", s);
         botnet_send_unlinked(i, dcc[i].nick, s);
         killsock(dcc[i].sock);
         lostdcc(i);
@@ -1632,8 +1687,6 @@ void check_botnet_pings()
           if (dcc[i].status & STAT_WARNED) {
             char s[1024];
 
-            putlog(LOG_BOTS, "*", "%s %s (%s).", BOT_DISCONNECTED,
-                   dcc[i].nick, BOT_BOTNOTLEAFLIKE);
             dprintf(i, "bye %s\n", BOT_BOTNOTLEAFLIKE);
             bot = findbot(dcc[i].nick);
             bots = bots_in_subtree(bot);
@@ -1642,7 +1695,7 @@ void check_botnet_pings()
                            BOT_DISCONNECTED, dcc[i].nick, BOT_BOTNOTLEAFLIKE,
                            bots, (bots != 1) ? "s" : "", users, (users != 1) ?
                            "s" : "");
-            chatout("*** %s\n", s);
+            putlog(LOG_BOTS, "*", "%s.", s);
             botnet_send_unlinked(i, dcc[i].nick, s);
             killsock(dcc[i].sock);
             lostdcc(i);
@@ -1668,7 +1721,7 @@ void zapfbot(int idx)
   simple_sprintf(s, "%s: %s (lost %d bot%s and %d user%s)", BOT_BOTDROPPED,
                  dcc[idx].nick, bots, (bots != 1) ? "s" : "", users,
                  (users != 1) ? "s" : "");
-  chatout("*** %s\n", s);
+  putlog(LOG_BOTS, "*", "%s.", s);
   botnet_send_unlinked(idx, dcc[idx].nick, s);
   killsock(dcc[idx].sock);
   lostdcc(idx);
