@@ -6,12 +6,10 @@
  *   timers, utimers
  *   telling the current programmed settings
  *   initializing a lot of stuff and loading the tcl scripts
- *
- * $Id: chanprog.c,v 1.69 2011/09/23 20:31:32 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -43,15 +41,19 @@
 
 #include "modules.h"
 
+extern struct dcc_t *dcc;
 extern struct userrec *userlist;
 extern log_t *logs;
 extern Tcl_Interp *interp;
 extern char ver[], botnetnick[], firewall[], motdfile[], userfile[], helpdir[],
-            tempdir[], moddir[], notify_new[], owner[], configfile[];
+            moddir[], notify_new[], configfile[];
 extern time_t now, online_since;
 extern int backgrd, term_z, con_chan, cache_hit, cache_miss, firewallport,
            default_flags, max_logs, conmask, protect_readonly, make_userfile,
            noshare, ignore_time, max_socks;
+#ifdef TLS
+extern SSL_CTX *ssl_ctx;
+#endif
 
 tcl_timer_t *timer = NULL;         /* Minutely timer               */
 tcl_timer_t *utimer = NULL;        /* Secondly timer               */
@@ -61,6 +63,7 @@ struct chanset_t *chanset = NULL;  /* Channel list                 */
 char admin[121] = "";              /* Admin info                   */
 char origbotname[NICKLEN + 1];
 char botname[NICKLEN + 1];         /* Primary botname              */
+char owner[121] = "";              /* Permanent botowner(s)        */
 
 
 /* Remove leading and trailing whitespaces.
@@ -225,6 +228,25 @@ int expmem_chanprog()
   return tot;
 }
 
+float getcputime()
+{
+#ifdef HAVE_GETRUSAGE
+  float stime, utime;
+  struct rusage ru;
+
+  getrusage(RUSAGE_SELF, &ru);
+  utime = ru.ru_utime.tv_sec + (ru.ru_utime.tv_usec / 1000000.00);
+  stime = ru.ru_stime.tv_sec + (ru.ru_stime.tv_usec / 1000000.00);
+  return (utime + stime);
+#else
+#  ifdef HAVE_CLOCK
+  return (clock() / (CLOCKS_PER_SEC * 1.00));
+#  else
+  return -1.00;
+#  endif
+#endif
+}
+
 /* Dump uptime info out to dcc (guppy 9Jan99)
  */
 void tell_verbose_uptime(int idx)
@@ -268,17 +290,11 @@ void tell_verbose_status(int idx)
   char *vers_t, *uni_t;
   int i;
   time_t now2 = now - online_since, hr, min;
-#ifdef HAVE_GETRUSAGE
-  struct rusage ru;
-#else
-#  ifdef HAVE_CLOCK
-  clock_t cl;
-#  endif
-#endif
+  float cputime;
 #ifdef HAVE_UNAME
   struct utsname un;
 
-  if (!uname(&un) < 0) {
+  if (uname(&un) < 0) {
 #endif
     vers_t = " ";
     uni_t  = "*unknown*";
@@ -309,30 +325,23 @@ void tell_verbose_status(int idx)
   sprintf(&s[strlen(s)], "%02d:%02d", (int) hr, (int) min);
   s1[0] = 0;
   if (backgrd)
-    strcpy(s1, MISC_BACKGROUND);
+    strncpyz(s1, MISC_BACKGROUND, sizeof s1);
   else {
     if (term_z)
-      strcpy(s1, MISC_TERMMODE);
+      strncpyz(s1, MISC_TERMMODE, sizeof s1);
     else if (con_chan)
-      strcpy(s1, MISC_STATMODE);
+      strncpyz(s1, MISC_STATMODE, sizeof s1);
     else
-      strcpy(s1, MISC_LOGMODE);
+      strncpyz(s1, MISC_LOGMODE, sizeof s1);
+  }
+  cputime = getcputime();
+  if (cputime < 0)
+    sprintf(s2, "CPU: unknown");
+  else {
+    hr = cputime / 60;
+    cputime -= hr * 60;
+    sprintf(s2, "CPU: %02d:%05.2f", (int) hr, cputime); /* Actally min/sec */
   }
-#ifdef HAVE_GETRUSAGE
-  getrusage(RUSAGE_SELF, &ru);
-  hr = (int) ((ru.ru_utime.tv_sec + ru.ru_stime.tv_sec) / 60);
-  min = (int) ((ru.ru_utime.tv_sec + ru.ru_stime.tv_sec) - (hr * 60));
-  sprintf(s2, "CPU: %02d:%02d", (int) hr, (int) min);    /* Actally min/sec */
-#else
-#  ifdef HAVE_CLOCK
-  cl = (clock() / CLOCKS_PER_SEC);
-  hr = (int) (cl / 60);
-  min = (int) (cl - (hr * 60));
-  sprintf(s2, "CPU: %02d:%02d", (int) hr, (int) min);    /* Actually min/sec */
-#  else
-  sprintf(s2, "CPU: unknown");
-#  endif
-#endif
   dprintf(idx, "%s %s (%s) - %s - %s: %4.1f%%\n", MISC_ONLINEFOR,
           s, s1, s2, MISC_CACHEHIT,
           100.0 * ((float) cache_hit) / ((float) (cache_hit + cache_miss)));
@@ -343,6 +352,7 @@ void tell_verbose_status(int idx)
 
   dprintf(idx, "Config file: %s\n", configfile);
   dprintf(idx, "OS: %s %s\n", uni_t, vers_t);
+  dprintf(idx, "Process ID: %d (parent %d)\n", getpid(), getppid());
 
   /* info library */
   dprintf(idx, "%s %s\n", MISC_TCLLIBRARY,
@@ -353,11 +363,21 @@ void tell_verbose_status(int idx)
   dprintf(idx, "%s %s (%s %s)\n", MISC_TCLVERSION,
           ((interp) && (Tcl_Eval(interp, "info patchlevel") == TCL_OK)) ?
           tcl_resultstring() : (Tcl_Eval(interp, "info tclversion") == TCL_OK) ?
-          tcl_resultstring() : "*unknown*", MISC_TCLHVERSION,
-          TCL_PATCH_LEVEL ? TCL_PATCH_LEVEL : "*unknown*");
+          tcl_resultstring() : "*unknown*", MISC_TCLHVERSION, TCL_PATCH_LEVEL);
 
   if (tcl_threaded())
     dprintf(idx, "Tcl is threaded.\n");
+#ifdef TLS
+  dprintf(idx, "TLS support is enabled.\n");
+  dprintf(idx, "TLS library: %s\n", SSLeay_version(SSLEAY_VERSION));
+#else
+  dprintf(idx, "TLS support is not available.\n");
+#endif
+#ifdef IPV6
+  dprintf(idx, "IPv6 support is enabled.\n");
+#else
+  dprintf(idx, "IPv6 support is not available.\n");
+#endif
   dprintf(idx, "Socket table: %d/%d\n", threaddata()->MAXSOCKS, max_socks);
 }
 
@@ -377,11 +397,9 @@ void tell_settings(int idx)
   dprintf(idx, "Directories:\n");
 #ifndef STATIC
   dprintf(idx, "  Help   : %s\n", helpdir);
-  dprintf(idx, "  Temp   : %s\n", tempdir);
   dprintf(idx, "  Modules: %s\n", moddir);
 #else
   dprintf(idx, "  Help: %s\n", helpdir);
-  dprintf(idx, "  Temp: %s\n", tempdir);
 #endif
   fr.global = default_flags;
 
@@ -429,13 +447,11 @@ void reaffirm_owners()
 void chanprog()
 {
   int i;
-  FILE *f;
-  char s[161], rands[8];
 
   admin[0]   = 0;
   helpdir[0] = 0;
-  tempdir[0] = 0;
-  conmask    = 0;
+  /* default mkcoblxs */
+  conmask = LOG_MSGS|LOG_MODES|LOG_CMDS|LOG_MISC|LOG_BOTS|LOG_BOTMSG|LOG_FILES|LOG_SERV;
 
   for (i = 0; i < max_logs; i++)
     logs[i].flags |= LF_EXPIRING;
@@ -471,7 +487,7 @@ void chanprog()
   protect_readonly = 1;
 
   if (!botnetnick[0])
-    strncpyz(botnetnick, origbotname, HANDLEN + 1);
+    set_botnetnick(origbotname);
 
   if (!botnetnick[0])
     fatal("I don't have a botnet nick!!\n", 0);
@@ -499,25 +515,6 @@ void chanprog()
     if (helpdir[strlen(helpdir) - 1] != '/')
       strcat(helpdir, "/");
 
-  if (tempdir[0])
-    if (tempdir[strlen(tempdir) - 1] != '/')
-      strcat(tempdir, "/");
-
-  /* Test tempdir: it's vital. */
-
-  /* Possible file race condition solved by using a random string
-   * and the process id in the filename.
-   * FIXME: This race is only partitially fixed. We could still be
-   *        overwriting an existing file / following a malicious
-   *        link.
-   */
-  make_rand_str(rands, 7); /* create random string */
-  sprintf(s, "%s.test-%u-%s", tempdir, getpid(), rands);
-  f = fopen(s, "w");
-  if (f == NULL)
-    fatal(MISC_CANTWRITETEMP, 0);
-  fclose(f);
-  unlink(s);
   reaffirm_owners();
   check_tcl_event("userfile-loaded");
 }
@@ -538,6 +535,7 @@ void reload()
   if (!readuserfile(userfile, &userlist))
     fatal(MISC_MISSINGUSERF, 0);
   reaffirm_owners();
+  add_hq_user();
   check_tcl_event("userfile-loaded");
   call_hook(HOOK_READ_USERFILE);
 }
@@ -550,6 +548,7 @@ void rehash()
   noshare = 0;
   userlist = NULL;
   chanprog();
+  add_hq_user();
 }
 
 /*
@@ -558,14 +557,15 @@ void rehash()
 
 /* Add a timer
  */
-unsigned long add_timer(tcl_timer_t ** stack, int elapse, char *cmd,
-                        unsigned long prev_id)
+unsigned long add_timer(tcl_timer_t ** stack, int elapse, int count,
+                        char *cmd, unsigned long prev_id)
 {
   tcl_timer_t *old = (*stack);
 
   *stack = nmalloc(sizeof **stack);
   (*stack)->next = old;
-  (*stack)->mins = elapse;
+  (*stack)->mins = (*stack)->interval = elapse;
+  (*stack)->count = count;
   (*stack)->cmd = nmalloc(strlen(cmd) + 1);
   strcpy((*stack)->cmd, cmd);
   /* If it's just being added back and already had an id,
@@ -618,12 +618,18 @@ void do_check_timers(tcl_timer_t ** stack)
     if (!old->mins) {
       egg_snprintf(x, sizeof x, "timer%lu", old->id);
       do_tcl(x, old->cmd);
-      nfree(old->cmd);
-      nfree(old);
-    } else {
-      old->next = *stack;
-      *stack = old;
+      if (old->count == 1) {
+        nfree(old->cmd);
+        nfree(old);
+        continue;
+      } else {
+        old->mins = old->interval;
+        if (old->count > 1)
+          old->count--;
+      }
     }
+    old->next = *stack;
+    *stack = old;
   }
 }
 
@@ -646,17 +652,19 @@ void wipe_timers(Tcl_Interp *irp, tcl_timer_t **stack)
  */
 void list_timers(Tcl_Interp *irp, tcl_timer_t *stack)
 {
-  char mins[10], id[16], *x;
-  EGG_CONST char *argv[3];
+  char mins[10], count[10], id[16], *x;
+  EGG_CONST char *argv[4];
   tcl_timer_t *mark;
 
   for (mark = stack; mark; mark = mark->next) {
     egg_snprintf(mins, sizeof mins, "%u", mark->mins);
     egg_snprintf(id, sizeof id, "timer%lu", mark->id);
+    egg_snprintf(count, sizeof count, "%u", mark->count);
     argv[0] = mins;
     argv[1] = mark->cmd;
     argv[2] = id;
-    x = Tcl_Merge(3, argv);
+    argv[3] = count;
+    x = Tcl_Merge(sizeof(argv)/sizeof(*argv), argv);
     Tcl_AppendElement(irp, x);
     Tcl_Free((char *) x);
   }
@@ -691,3 +699,30 @@ int isowner(char *name)
 
   return 0;
 }
+
+/*
+ * Adds the -HQ user to the userlist and takes care of needed permissions
+ */
+void add_hq_user()
+{
+  if (!backgrd && term_z > 0 && userlist) {
+    /* HACK: Workaround using dcc[].nick not to pass literal "-HQ" as a non-const arg */
+    dcc[term_z].user = get_user_by_handle(userlist, dcc[term_z].nick);
+    /* Make sure there's an innocuous -HQ user if needed */
+    if (!dcc[term_z].user) {
+      userlist = adduser(userlist, dcc[term_z].nick, "none", "-", USER_PARTY);
+      dcc[term_z].user = get_user_by_handle(userlist, dcc[term_z].nick);
+    }
+    /* Give all useful flags: efjlmnoptuvx */
+    dcc[term_z].user->flags = USER_EXEMPT | USER_FRIEND | USER_JANITOR |
+                              USER_HALFOP | USER_MASTER | USER_OWNER | USER_OP |
+                              USER_PARTY | USER_BOTMAST | USER_UNSHARED |
+                              USER_VOICE | USER_XFER;
+    /* Add to permowner list if there's place */
+    if (strlen(owner) + sizeof EGG_BG_HANDLE < sizeof owner)
+      strcat(owner, " " EGG_BG_HANDLE);
+
+    /* Update laston info, gets cleared at rehash/reload */
+    touch_laston(dcc[term_z].user, "partyline", now);
+  }
+}
