@@ -2,12 +2,10 @@
  * cmds.c -- handles:
  *   commands from a user via dcc
  *   (split in 2, this portion contains no-irc commands)
- *
- * $Id: cmds.c,v 1.129 2011/02/13 14:19:33 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -50,6 +48,10 @@ extern module_entry *module_list;
 
 static char *btos(unsigned long);
 
+/* Define some characters not allowed in address/port string
+ */
+#define BADADDRCHARS "+/"
+
 
 /* Add hostmask to a bot's record if possible.
  */
@@ -643,33 +645,29 @@ static void cmd_boot(struct userrec *u, int idx, char *par)
     dprintf(idx, "Who?  No such person on the party line.\n");
 }
 
-static void cmd_console(struct userrec *u, int idx, char *par)
+/* Make changes to user console settings */
+static void do_console(struct userrec *u, int idx, char *par, int reset)
 {
   char *nick, s[2], s1[512];
   int dest = 0, i, ok = 0, pls;
   struct flag_record fr = { FR_GLOBAL | FR_CHAN, 0, 0, 0, 0, 0 };
   module_entry *me;
 
-  if (!par[0]) {
-    dprintf(idx, "Your console is %s: %s (%s).\n",
-            dcc[idx].u.chat->con_chan,
-            masktype(dcc[idx].u.chat->con_flags),
-            maskname(dcc[idx].u.chat->con_flags));
-    return;
-  }
   get_user_flagrec(u, &fr, dcc[idx].u.chat->con_chan);
-  strcpy(s1, par);
+  strncpyz(s1, par, sizeof s1);
   nick = newsplit(&par);
-  /* Don't remove '+' as someone couldn't have '+' in CHANMETA cause
+  /* Check if the parameter is a handle.
+   * Don't remove '+' as someone couldn't have '+' in CHANMETA cause
    * he doesn't use IRCnet ++rtc.
    */
   if (nick[0] && !strchr(CHANMETA "+-*", nick[0]) && glob_master(fr)) {
-    for (i = 0; i < dcc_total; i++)
+    for (i = 0; i < dcc_total; i++) {
       if (!egg_strcasecmp(nick, dcc[i].nick) &&
           (dcc[i].type == &DCC_CHAT) && (!ok)) {
         ok = 1;
         dest = i;
       }
+    }
     if (!ok) {
       dprintf(idx, "No such user on the party line!\n");
       return;
@@ -679,9 +677,11 @@ static void cmd_console(struct userrec *u, int idx, char *par)
     dest = idx;
   if (!nick[0])
     nick = newsplit(&par);
-  /* Consider modeless channels, starting with '+' */
-  if ((nick[0] == '+' && findchan_by_dname(nick)) ||
-      (nick[0] != '+' && strchr(CHANMETA "*", nick[0]))) {
+  /* Check if the parameter is a channel.
+   * Consider modeless channels, starting with '+'
+   */
+  if (nick[0] && !reset && ((nick[0] == '+' && findchan_by_dname(nick)) ||
+      (nick[0] != '+' && strchr(CHANMETA "*", nick[0])))) {
     if (strcmp(nick, "*") && !findchan_by_dname(nick)) {
       dprintf(idx, "Invalid console channel: %s.\n", nick);
       return;
@@ -692,7 +692,8 @@ static void cmd_console(struct userrec *u, int idx, char *par)
               nick);
       return;
     }
-    strncpyz(dcc[dest].u.chat->con_chan, nick, 81);
+    strncpyz(dcc[dest].u.chat->con_chan, nick,
+        sizeof dcc[dest].u.chat->con_chan);
     nick[0] = 0;
     if (dest != idx)
       get_user_flagrec(dcc[dest].user, &fr, dcc[dest].u.chat->con_chan);
@@ -700,7 +701,7 @@ static void cmd_console(struct userrec *u, int idx, char *par)
   if (!nick[0])
     nick = newsplit(&par);
   pls = 1;
-  if (nick[0]) {
+  if (!reset && nick[0]) {
     if ((nick[0] != '+') && (nick[0] != '-'))
       dcc[dest].u.chat->con_flags = 0;
     for (; *nick; nick++) {
@@ -717,10 +718,12 @@ static void cmd_console(struct userrec *u, int idx, char *par)
           dcc[dest].u.chat->con_flags &= ~logmodes(s);
       }
     }
+  } else if (reset) {
+    dcc[dest].u.chat->con_flags = (u->flags & USER_MASTER) ? conmask : 0;
   }
   dcc[dest].u.chat->con_flags = check_conflags(&fr,
                                                dcc[dest].u.chat->con_flags);
-  putlog(LOG_CMDS, "*", "#%s# console %s", dcc[idx].nick, s1);
+  putlog(LOG_CMDS, "*", "#%s# %sconsole %s", dcc[idx].nick, reset ? "reset" : "", s1);
   if (dest == idx) {
     dprintf(idx, "Set your console to %s: %s (%s).\n",
             dcc[idx].u.chat->con_chan,
@@ -744,20 +747,64 @@ static void cmd_console(struct userrec *u, int idx, char *par)
   }
 }
 
+static void cmd_console(struct userrec *u, int idx, char *par)
+{
+  if (!par[0]) {
+    dprintf(idx, "Your console is %s: %s (%s).\n",
+            dcc[idx].u.chat->con_chan,
+            masktype(dcc[idx].u.chat->con_flags),
+            maskname(dcc[idx].u.chat->con_flags));
+    return;
+  }
+  do_console(u, idx, par, 0);
+}
+
+/* Reset console flags to config defaults */
+static void cmd_resetconsole(struct userrec *u, int idx, char *par)
+{
+  do_console(u, idx, par, 1);
+}
+
+/* Check if a string is a valid integer and lies non-inclusive
+ * between two given integers. Returns 1 if true, 0 if not.
+ */
+int check_int_range(char *value, int min, int max) {
+  char *endptr = NULL;
+  long intvalue;
+
+  if (value && value[0]) {
+    intvalue = strtol(value, &endptr, 10);
+    if ((intvalue < max) && (intvalue > min) && (*endptr == '\0')) {
+      return 1;
+    }
+  }
+  return 0;
+}
+
 static void cmd_pls_bot(struct userrec *u, int idx, char *par)
 {
-  char *handle, *addr, *p, *q, *host;
+  char *handle, *addr, *port, *port2, *relay, *host;
   struct userrec *u1;
   struct bot_addr *bi;
+  int i, found = 0;
 
   if (!par[0]) {
-    dprintf(idx, "Usage: +bot <handle> [address[:telnet-port[/relay-port]]] "
+    dprintf(idx, "Usage: +bot <handle> [address [telnet-port[/relay-port]]] "
             "[host]\n");
     return;
   }
 
   handle = newsplit(&par);
   addr = newsplit(&par);
+  port2 = newsplit(&par);
+  port = strtok(port2, "/");
+  relay = strtok(NULL, "/");
+
+  if (strtok(NULL, "/")) {
+    dprintf(idx, "You've supplied more than 2 ports, make up your mind.\n");
+    return;
+  }
+
   host = newsplit(&par);
 
   if (strlen(handle) > HANDLEN)
@@ -773,41 +820,117 @@ static void cmd_pls_bot(struct userrec *u, int idx, char *par)
     return;
   }
 
+  if (addr[0]) {
+#ifndef IPV6
+ /* Reject IPv6 addresses */
+    for (i=0; addr[i]; i++) {
+      if (addr[i] == ':') {
+        dprintf(idx, "Invalid IP address format (this Eggdrop "
+          "was compiled without IPv6 support).\n");
+        return;
+      }
+    }
+#endif
+ /* Check if user forgot address field by checking if argument is completely
+  * numerical, implying a port was provided as the next argument instead.
+  */
+    for (i=0; addr[i]; i++) {
+      if (strchr(BADADDRCHARS, addr[i])) {
+        dprintf(idx, "Bot address may not contain a '%c'. ", addr[i]);
+        break;
+      }
+      if (!isdigit((unsigned char) addr[i])) {
+        found=1;
+        break;
+      }
+    }
+    if (!found) {
+      dprintf(idx, "Invalid host address.\n");
+      dprintf(idx, "Usage: +bot <handle> [address [telnet-port[/relay-port]]] "
+              "[host]\n");
+      return;
+    }
+  }
+
+#ifndef TLS
+  if ((port && *port == '+') || (relay && relay[0] == '+')) {
+    dprintf(idx, "Ports prefixed with '+' are not enabled "
+      "(this Eggdrop was compiled without TLS support).\n");
+    return;
+  }
+#endif
+  if (port) {
+    if (!check_int_range(port, 0, 65536)) {
+      dprintf(idx, "Ports must be integers between 1 and 65535.\n");
+      return;
+    }
+  }
+  if (relay) {
+    if (!check_int_range(relay, 0, 65536)) {
+      dprintf(idx, "Ports must be integers between 1 and 65535.\n");
+      return;
+    }
+  }
+
   if (strlen(addr) > 60)
     addr[60] = 0;
 
-  putlog(LOG_CMDS, "*", "#%s# +bot %s%s%s%s%s", dcc[idx].nick, handle,
-         addr[0] ? " " : "", addr, host[0] ? " " : "", host);
+/* Trim IPv6 []s out if present */
+  if (addr[0] == '[') {
+    addr[strlen(addr)-1] = 0;
+    memmove(addr, addr + 1, strlen(addr));
+  }
   userlist = adduser(userlist, handle, "none", "-", USER_BOT);
   u1 = get_user_by_handle(userlist, handle);
   bi = user_malloc(sizeof(struct bot_addr));
+#ifdef TLS
+  bi->ssl = 0;
+#endif
+  bi->address = user_malloc(strlen(addr) + 1);
+  strcpy(bi->address, addr);
 
-  q = strchr(addr, ':');
-  if (!q) {
-    bi->address = user_malloc(strlen(addr) + 1);
-    strcpy(bi->address, addr);
+  if (!port) {
     bi->telnet_port = 3333;
     bi->relay_port = 3333;
   } else {
-    bi->address = user_malloc(q - addr + 1);
-    strncpy(bi->address, addr, q - addr);
-    bi->address[q - addr] = 0;
-    p = q + 1;
-    bi->telnet_port = atoi(p);
-    q = strchr(p, '/');
-    if (!q) {
+#ifdef TLS
+    if (*port == '+')
+      bi->ssl |= TLS_BOT;
+#endif
+    bi->telnet_port = atoi(port);
+    if (!relay) {
       bi->relay_port = bi->telnet_port;
+#ifdef TLS
+      bi->ssl *= TLS_BOT + TLS_RELAY;
+#endif
     } else  {
-      bi->relay_port = atoi(q + 1);
+#ifdef TLS
+      if (relay[0] == '+')
+        bi->ssl |= TLS_RELAY;
+#endif
+      bi->relay_port = atoi(relay);
     }
   }
 
   set_user(&USERENTRY_BOTADDR, u1, bi);
   if (addr[0]) {
-    dprintf(idx, "Added bot '%s' with address '%s' and %s%s%s.\n", handle,
-            addr, host[0] ? "hostmask '" : "no hostmask", host[0] ? host : "",
+    putlog(LOG_CMDS, "*", "#%s# +bot %s %s%s%s%s%s %s%s", dcc[idx].nick, handle,
+           addr, port ? " " : "", port ? port : "", relay ? " " : "",
+           relay ? relay : "", host[0] ? " " : "", host);
+#ifdef TLS
+    dprintf(idx, "Added bot '%s' with address [%s]:%s%d/%s%d and %s%s%s.\n",
+            handle, addr, (bi->ssl & TLS_BOT) ? "+" : "", bi->telnet_port,
+            (bi->ssl & TLS_RELAY) ? "+" : "", bi->relay_port, host[0] ?
+            "hostmask '" : "no hostmask", host[0] ? host : "",
             host[0] ? "'" : "");
-  } else{
+#else
+    dprintf(idx, "Added bot '%s' with address [%s]:%d/%d and %s%s%s.\n", handle,
+            addr, bi->telnet_port, bi->relay_port, host[0] ? "hostmask '" :
+            "no hostmask", host[0] ? host : "", host[0] ? "'" : "");
+#endif
+  } else {
+    putlog(LOG_CMDS, "*", "#%s# +bot %s %s%s", dcc[idx].nick, handle,
+           host[0] ? " " : "", host);
     dprintf(idx, "Added bot '%s' with no address and %s%s%s.\n", handle,
             host[0] ? "hostmask '" : "no hostmask", host[0] ? host : "",
             host[0] ? "'" : "");
@@ -847,6 +970,8 @@ static void cmd_chhandle(struct userrec *u, int idx, char *par)
     atr2 = u2 ? u2->flags : 0;
     if ((atr & USER_BOTMAST) && !(atr & USER_MASTER) && !(atr2 & USER_BOT))
       dprintf(idx, "You can't change handles for non-bots.\n");
+    else if (!egg_strcasecmp(hand, EGG_BG_HANDLE))
+      dprintf(idx, "You can't change the handle of a temporary user.\n");
     else if ((bot_flags(u2) & BOT_SHARE) && !(atr & USER_OWNER))
       dprintf(idx, "You can't change share bot's nick.\n");
     else if ((atr2 & USER_OWNER) && !(atr & USER_OWNER) &&
@@ -884,6 +1009,8 @@ static void cmd_handle(struct userrec *u, int idx, char *par)
     dprintf(idx,
             "Bizarre quantum forces prevent handle from starting with '%c'.\n",
             newhandle[0]);
+  else if (!egg_strcasecmp(u->handle, EGG_BG_HANDLE))
+    dprintf(idx, "You can't change the handle of this temporary user.\n");
   else if (get_user_by_handle(userlist, newhandle) &&
            egg_strcasecmp(dcc[idx].nick, newhandle))
     dprintf(idx, "Somebody is already using %s.\n", newhandle);
@@ -941,20 +1068,150 @@ static void cmd_chpass(struct userrec *u, int idx, char *par)
   }
 }
 
+#ifdef TLS
+static void cmd_fprint(struct userrec *u, int idx, char *par)
+{
+  char *new;
+
+  if (!par[0]) {
+    dprintf(idx, "Usage: fprint <newfingerprint|+>\n");
+    return;
+  }
+  new = newsplit(&par);
+  if (!strcmp(new, "+")) {
+    if (!dcc[idx].ssl) {
+      dprintf(idx, "You aren't connected with SSL. "
+              "Please set your fingerprint manually.\n");
+      return;
+    } else if (!(new = ssl_getfp(dcc[idx].sock))) {
+      dprintf(idx, "Can't get your current fingerprint. "
+              "Set up you client to send a certificate!\n");
+      return;
+    }
+  }
+  if (set_user(&USERENTRY_FPRINT, u, new)) {
+    putlog(LOG_CMDS, "*", "#%s# fprint...", dcc[idx].nick);
+    dprintf(idx, "Changed fingerprint to '%s'.\n", new);
+  } else
+    dprintf(idx, "Invalid fingerprint. Must be a hexadecimal string.\n");
+}
+
+static void cmd_chfinger(struct userrec *u, int idx, char *par)
+{
+  char *handle, *new;
+  int atr = u ? u->flags : 0;
+
+  if (!par[0])
+    dprintf(idx, "Usage: chfinger <handle> [fingerprint]\n");
+  else {
+    handle = newsplit(&par);
+    u = get_user_by_handle(userlist, handle);
+    if (!u)
+      dprintf(idx, "No such user.\n");
+    else if ((atr & USER_BOTMAST) && !(atr & USER_MASTER) &&
+             !(u->flags & USER_BOT))
+      dprintf(idx, "You can't change fingerprints for non-bots.\n");
+    else if ((bot_flags(u) & BOT_SHARE) && !(atr & USER_OWNER))
+      dprintf(idx, "You can't change a share bot's fingerprint.\n");
+    else if ((u->flags & USER_OWNER) && !(atr & USER_OWNER) &&
+             egg_strcasecmp(handle, dcc[idx].nick))
+      dprintf(idx, "You can't change a bot owner's fingerprint.\n");
+    else if (isowner(handle) && egg_strcasecmp(dcc[idx].nick, handle))
+      dprintf(idx, "You can't change a permanent bot owner's fingerprint.\n");
+    else if (!par[0]) {
+      putlog(LOG_CMDS, "*", "#%s# chfinger %s [nothing]", dcc[idx].nick, handle);
+      set_user(&USERENTRY_FPRINT, u, NULL);
+      dprintf(idx, "Removed fingerprint.\n");
+    } else {
+      new = newsplit(&par);
+      if (set_user(&USERENTRY_FPRINT, u, new)) {
+        putlog(LOG_CMDS, "*", "#%s# chfinger %s %s", dcc[idx].nick,
+               handle, new);
+        dprintf(idx, "Changed fingerprint.\n");
+      } else
+        dprintf(idx, "Invalid fingerprint. Must be a hexadecimal string.\n");
+    }
+  }
+}
+#endif
+
 static void cmd_chaddr(struct userrec *u, int idx, char *par)
 {
-  int telnet_port = 3333, relay_port = 3333;
-  char *handle, *addr, *p, *q;
+#ifdef TLS
+  int use_ssl = 0;
+#endif
+  int i, found = 0, telnet_port = 3333, relay_port = 3333;
+  char *handle, *addr, *port, *port2, *relay;
   struct bot_addr *bi;
   struct userrec *u1;
 
   handle = newsplit(&par);
   if (!par[0]) {
-    dprintf(idx, "Usage: chaddr <botname> "
-            "<address[:telnet-port[/relay-port]]>\n");
+    dprintf(idx, "Usage: chaddr <botname> <address> "
+            "[telnet-port[/relay-port]]>\n");
     return;
   }
   addr = newsplit(&par);
+  port2 = newsplit(&par);
+  port = strtok(port2, "/");
+  relay = strtok(NULL, "/");
+
+  if (strtok(NULL, "/")) {
+    dprintf(idx, "You've supplied more than 2 ports, make up your mind.\n");
+    return;
+  }
+
+  if (addr[0]) {
+#ifndef IPV6
+    for (i=0; addr[i]; i++) {
+      if (addr[i] == ':') {
+        dprintf(idx, "Invalid IP address format (this Eggdrop "
+          "was compiled without IPv6 support).\n");
+        return;
+      }
+    }
+#endif
+ /* Check if user forgot address field by checking if argument is completely
+  * numerical, implying a port was provided as the next argument instead.
+  */
+    for (i=0; addr[i]; i++) {
+      if (strchr(BADADDRCHARS, addr[i])) {
+        dprintf(idx, "Bot address may not contain a '%c'. ", addr[i]);
+        break;
+      }
+      if (!isdigit((unsigned char) addr[i])) {
+        found=1;
+        break;
+      }
+    }
+    if (!found) {
+      dprintf(idx, "Invalid host address.\n");
+      dprintf(idx, "Usage: chaddr <botname> <address> "
+              "[telnet-port[/relay-port]]>\n");
+      return;
+    }
+  }
+
+#ifndef TLS
+  if ((port && *port == '+') || (relay && relay[0] == '+')) {
+    dprintf(idx, "Ports prefixed with '+' are not enabled "
+      "(this Eggdrop was compiled without TLS support).\n");
+    return;
+  }
+#endif
+  if (port && port[0]) {
+    if (!check_int_range(port, 0, 65536)) {
+      dprintf(idx, "Ports must be integers between 1 and 65535.\n");
+      return;
+    }
+  }
+  if (relay) {
+    if (!check_int_range(relay, 0, 65536)) {
+      dprintf(idx, "Ports must be integers between 1 and 65535.\n");
+      return;
+    }
+  }
+
   if (strlen(addr) > UHOSTMAX)
     addr[UHOSTMAX] = 0;
   u1 = get_user_by_handle(userlist, handle);
@@ -966,36 +1223,56 @@ static void cmd_chaddr(struct userrec *u, int idx, char *par)
     dprintf(idx, "You can't change a share bot's address.\n");
     return;
   }
-  putlog(LOG_CMDS, "*", "#%s# chaddr %s %s", dcc[idx].nick, handle, addr);
-  dprintf(idx, "Changed bot's address.\n");
 
   bi = (struct bot_addr *) get_user(&USERENTRY_BOTADDR, u1);
   if (bi) {
     telnet_port = bi->telnet_port;
     relay_port = bi->relay_port;
+#ifdef TLS
+    use_ssl = bi->ssl;
+#endif
   }
 
+/* Trim IPv6 []s out if present */
+  if (addr[0] == '[') {
+    addr[strlen(addr)-1] = 0;
+    memmove(addr, addr + 1, strlen(addr));
+  }
   bi = user_malloc(sizeof(struct bot_addr));
+  bi->address = user_malloc(strlen(addr) + 1);
+  strcpy(bi->address, addr);
 
-  q = strchr(addr, ':');
-  if (!q) {
-    bi->address = user_malloc(strlen(addr) + 1);
-    strcpy(bi->address, addr);
+  if (!port) {
     bi->telnet_port = telnet_port;
     bi->relay_port = relay_port;
+#ifdef TLS
+    bi->ssl = use_ssl;
   } else {
-    bi->address = user_malloc(q - addr + 1);
-    strncpyz(bi->address, addr, q - addr + 1);
-    p = q + 1;
-    bi->telnet_port = atoi(p);
-    q = strchr(p, '/');
-    if (!q) {
+    bi->ssl = 0;
+    if (*port == '+')
+      bi->ssl |= TLS_BOT;
+    bi->telnet_port = atoi(port);
+    if (!relay) {
+      bi->relay_port = bi->telnet_port;
+      bi->ssl *= TLS_BOT + TLS_RELAY;
+    } else {
+      if (*relay == '+') {
+        bi->ssl |= TLS_RELAY;
+      }
+#else
+  } else {
+    bi->telnet_port = atoi(port);
+    if (!relay) {
       bi->relay_port = bi->telnet_port;
     } else {
-      bi->relay_port = atoi(q + 1);
+#endif
+     bi->relay_port = atoi(relay);
     }
   }
   set_user(&USERENTRY_BOTADDR, u1, bi);
+  putlog(LOG_CMDS, "*", "#%s# chaddr %s %s%s%s%s%s", dcc[idx].nick, handle,
+         addr, port ? " " : "", port ? port : "", relay ? "/" : "", relay ? relay : "");
+  dprintf(idx, "Changed bot's address.\n");
 }
 
 static void cmd_comment(struct userrec *u, int idx, char *par)
@@ -1207,7 +1484,7 @@ static void cmd_backup(struct userrec *u, int idx, char *par)
 static void cmd_trace(struct userrec *u, int idx, char *par)
 {
   int i;
-  char x[NOTENAMELEN + 11], y[11];
+  char x[NOTENAMELEN + 11], y[12];
 
   if (!par[0]) {
     dprintf(idx, "Usage: trace <botname>\n");
@@ -1333,8 +1610,7 @@ int check_dcc_attrs(struct userrec *u, int oatr)
           dcc[i].status &= (~STAT_CHAT);
           dcc[i].type = &DCC_CHAT;
           if (dcc[i].u.chat->channel >= 0) {
-            chanout_but(-1, dcc[i].u.chat->channel,
-                        "*** %s has returned.\n", dcc[i].nick);
+            chanout_but(-1, dcc[i].u.chat->channel, DCC_RETURN, dcc[i].nick);
             if (dcc[i].u.chat->channel < GLOBAL_CHANS)
               botnet_send_join_idx(i, -1);
           }
@@ -1560,7 +1836,7 @@ static void cmd_chattr(struct userrec *u, int idx, char *par)
   }
   if (chan)
     putlog(LOG_CMDS, "*", "#%s# (%s) chattr %s %s",
-           dcc[idx].nick, chan ? chan->dname : "*", hand, chg ? chg : "");
+           dcc[idx].nick, chan->dname, hand, chg ? chg : "");
   else
     putlog(LOG_CMDS, "*", "#%s# chattr %s %s", dcc[idx].nick, hand,
            chg ? chg : "");
@@ -1689,9 +1965,6 @@ static void cmd_botattr(struct userrec *u, int idx, char *par)
     pls.match = user.match;
     break_down_flags(chg, &pls, &mns);
     /* No-one can change these flags on-the-fly */
-    pls.global &=~BOT_BOT;
-    mns.global &=~BOT_BOT;
-
     if (chan && glob_owner(user)) {
       pls.chan &= BOT_SHARE;
       mns.chan &= BOT_SHARE;
@@ -1747,7 +2020,9 @@ static void cmd_botattr(struct userrec *u, int idx, char *par)
 static void cmd_chat(struct userrec *u, int idx, char *par)
 {
   char *arg;
-  int newchan, oldchan;
+  int localchan = 0;
+  int newchan = 0;
+  int oldchan;
   module_entry *me;
 
   arg = newsplit(&par);
@@ -1797,8 +2072,12 @@ static void cmd_chat(struct userrec *u, int idx, char *par)
         else {
           Tcl_SetVar(interp, "_chan", arg, 0);
           if ((Tcl_VarEval(interp, "assoc ", "$_chan", NULL) == TCL_OK) &&
-              !tcl_resultempty())
+              !tcl_resultempty()) {
             newchan = tcl_resultint();
+            if ((newchan >= GLOBAL_CHANS) && (newchan <= 199999)) {
+              localchan = 1;
+            }
+          }
           else
             newchan = -1;
         }
@@ -1808,7 +2087,8 @@ static void cmd_chat(struct userrec *u, int idx, char *par)
         }
       } else
         newchan = atoi(arg);
-      if ((newchan < 0) || (newchan >= GLOBAL_CHANS)) {
+      if ((newchan < 0) || ((newchan >= GLOBAL_CHANS) && (!localchan)) ||
+          (newchan >= 199999)) {
         dprintf(idx, "Channel number out of range: must be between 0 and %d."
                 "\n", GLOBAL_CHANS);
         return;
@@ -1893,17 +2173,17 @@ int stripmodes(char *s)
 
   for (; *s; s++)
     switch (tolower((unsigned) *s)) {
-    case 'b':
-      res |= STRIP_BOLD;
-      break;
     case 'c':
       res |= STRIP_COLOR;
       break;
+    case 'b':
+      res |= STRIP_BOLD;
+      break;
     case 'r':
-      res |= STRIP_REV;
+      res |= STRIP_REVERSE;
       break;
     case 'u':
-      res |= STRIP_UNDER;
+      res |= STRIP_UNDERLINE;
       break;
     case 'a':
       res |= STRIP_ANSI;
@@ -1911,6 +2191,12 @@ int stripmodes(char *s)
     case 'g':
       res |= STRIP_BELLS;
       break;
+    case 'o':
+      res |= STRIP_ORDINARY;
+      break;
+    case 'i':
+      res |= STRIP_ITALICS;
+      break;
     case '*':
       res |= STRIP_ALL;
       break;
@@ -1923,18 +2209,22 @@ char *stripmasktype(int x)
   static char s[20];
   char *p = s;
 
-  if (x & STRIP_BOLD)
-    *p++ = 'b';
   if (x & STRIP_COLOR)
     *p++ = 'c';
-  if (x & STRIP_REV)
+  if (x & STRIP_BOLD)
+    *p++ = 'b';
+  if (x & STRIP_REVERSE)
     *p++ = 'r';
-  if (x & STRIP_UNDER)
+  if (x & STRIP_UNDERLINE)
     *p++ = 'u';
   if (x & STRIP_ANSI)
     *p++ = 'a';
   if (x & STRIP_BELLS)
     *p++ = 'g';
+  if (x & STRIP_ORDINARY)
+    *p++ = 'o';
+  if (x & STRIP_ITALICS)
+    *p++ = 'i';
   if (p == s)
     *p++ = '-';
   *p = 0;
@@ -1947,18 +2237,22 @@ static char *stripmaskname(int x)
   int i = 0;
 
   s[i] = 0;
-  if (x & STRIP_BOLD)
-    i += my_strcpy(s + i, "bold, ");
   if (x & STRIP_COLOR)
     i += my_strcpy(s + i, "color, ");
-  if (x & STRIP_REV)
+  if (x & STRIP_BOLD)
+    i += my_strcpy(s + i, "bold, ");
+  if (x & STRIP_REVERSE)
     i += my_strcpy(s + i, "reverse, ");
-  if (x & STRIP_UNDER)
+  if (x & STRIP_UNDERLINE)
     i += my_strcpy(s + i, "underline, ");
   if (x & STRIP_ANSI)
     i += my_strcpy(s + i, "ansi, ");
   if (x & STRIP_BELLS)
     i += my_strcpy(s + i, "bells, ");
+  if (x & STRIP_ORDINARY)
+    i += my_strcpy(s + i, "ordinary, ");
+  if (x & STRIP_ITALICS)
+    i += my_strcpy(s + i, "italics, ");
   if (!i)
     strcpy(s, "none");
   else
@@ -2110,7 +2404,7 @@ static void cmd_su(struct userrec *u, int idx, char *par)
         dcc[idx].u.chat->su_nick = get_data_ptr(strlen(dcc[idx].nick) + 1);
         strcpy(dcc[idx].u.chat->su_nick, dcc[idx].nick);
         dcc[idx].user = u;
-        strcpy(dcc[idx].nick, par);
+        strncpyz(dcc[idx].nick, par, sizeof dcc[idx].nick);
         dcc_chatter(idx);
       }
     }
@@ -2179,9 +2473,7 @@ static void cmd_tcl(struct userrec *u, int idx, char *msg)
 {
   int code;
   char *result;
-#ifdef USE_TCL_ENCODING
   Tcl_DString dstr;
-#endif
 
   if (!(isowner(dcc[idx].nick)) && (must_be_owner)) {
     dprintf(idx, MISC_NOSUCHCMD);
@@ -2190,24 +2482,17 @@ static void cmd_tcl(struct userrec *u, int idx, char *msg)
   debug1("tcl: evaluate (.tcl): %s", msg);
   code = Tcl_GlobalEval(interp, msg);
 
-#ifdef USE_TCL_ENCODING
   /* properly convert string to system encoding. */
   Tcl_DStringInit(&dstr);
   Tcl_UtfToExternalDString(NULL, tcl_resultstring(), -1, &dstr);
   result = Tcl_DStringValue(&dstr);
-#else
-  /* use old pre-Tcl 8.1 way. */
-  result = tcl_resultstring();
-#endif
 
   if (code == TCL_OK)
     dumplots(idx, "Tcl: ", result);
   else
     dumplots(idx, "Tcl error: ", result);
 
-#ifdef USE_TCL_ENCODING
   Tcl_DStringFree(&dstr);
-#endif
 }
 
 /* Perform a 'set' command
@@ -2216,9 +2501,7 @@ static void cmd_set(struct userrec *u, int idx, char *msg)
 {
   int code;
   char s[512], *result;
-#ifdef USE_TCL_ENCODING
   Tcl_DString dstr;
-#endif
 
   if (!(isowner(dcc[idx].nick)) && (must_be_owner)) {
     dprintf(idx, MISC_NOSUCHCMD);
@@ -2232,18 +2515,13 @@ static void cmd_set(struct userrec *u, int idx, char *msg)
     dumplots(idx, "Global vars: ", tcl_resultstring());
     return;
   }
-  strcpy(s + 4, msg);
+  strncpyz(s + 4, msg, sizeof s - 4);
   code = Tcl_Eval(interp, s);
 
-#ifdef USE_TCL_ENCODING
   /* properly convert string to system encoding. */
   Tcl_DStringInit(&dstr);
   Tcl_UtfToExternalDString(NULL, tcl_resultstring(), -1, &dstr);
   result = Tcl_DStringValue(&dstr);
-#else
-  /* use old pre-Tcl 8.1 way. */
-  result = tcl_resultstring();
-#endif
 
   if (code == TCL_OK) {
     if (!strchr(msg, ' '))
@@ -2253,9 +2531,7 @@ static void cmd_set(struct userrec *u, int idx, char *msg)
   } else
     dprintf(idx, "Error: %s\n", result);
 
-#ifdef USE_TCL_ENCODING
   Tcl_DStringFree(&dstr);
-#endif
 }
 
 static void cmd_module(struct userrec *u, int idx, char *par)
@@ -2448,10 +2724,14 @@ static void cmd_mns_user(struct userrec *u, int idx, char *par)
     dprintf(idx, "You can't remove a permanent bot owner!\n");
     return;
   }
-  if ((u2->flags & USER_OWNER) && !(u->flags & USER_OWNER)) {
+  if ((u2->flags & USER_OWNER) && !isowner(u->handle)) {
     dprintf(idx, "You can't remove a bot owner!\n");
     return;
   }
+  if ((u2->flags & USER_MASTER) && !(u->flags & USER_OWNER)) {
+    dprintf(idx, "Only owners can remove a master!\n");
+    return;
+  }
   if (u2->flags & USER_BOT) {
     if ((bot_flags(u2) & BOT_SHARE) && !(u->flags & USER_OWNER)) {
       dprintf(idx, "You can't remove share bots.\n");
@@ -2518,7 +2798,7 @@ static void cmd_pls_host(struct userrec *u, int idx, char *par)
       dprintf(idx, "You can't add hostmasks to non-bots.\n");
       return;
     }
-    if (!glob_owner(fr) && glob_bot(fr2) && (bot_flags(u2) & BOT_SHARE)) {
+    if (!(glob_owner(fr) || glob_botmast(fr)) && glob_bot(fr2) && (bot_flags(u2) & BOT_SHARE)) {
       dprintf(idx, "You can't add hostmasks to share bots.\n");
       return;
     }
@@ -2780,15 +3060,22 @@ cmd_t C_dcc[] = {
   {"chaddr",    "t",    (IntFunc) cmd_chaddr,     NULL},
   {"chat",      "",     (IntFunc) cmd_chat,       NULL},
   {"chattr",    "m|m",  (IntFunc) cmd_chattr,     NULL},
+#ifdef TLS
+  {"chfinger",  "t",    (IntFunc) cmd_chfinger,   NULL},
+#endif
   {"chhandle",  "t",    (IntFunc) cmd_chhandle,   NULL},
   {"chnick",    "t",    (IntFunc) cmd_chhandle,   NULL},
   {"chpass",    "t",    (IntFunc) cmd_chpass,     NULL},
   {"comment",   "m",    (IntFunc) cmd_comment,    NULL},
   {"console",   "to|o", (IntFunc) cmd_console,    NULL},
+  {"resetconsole", "to|o", (IntFunc) cmd_resetconsole, NULL},
   {"dccstat",   "t",    (IntFunc) cmd_dccstat,    NULL},
   {"debug",     "m",    (IntFunc) cmd_debug,      NULL},
   {"die",       "n",    (IntFunc) cmd_die,        NULL},
   {"echo",      "",     (IntFunc) cmd_echo,       NULL},
+#ifdef TLS
+  {"fprint",    "",     (IntFunc) cmd_fprint,     NULL},
+#endif
   {"fixcodes",  "",     (IntFunc) cmd_fixcodes,   NULL},
   {"help",      "",     (IntFunc) cmd_help,       NULL},
   {"ignores",   "m",    (IntFunc) cmd_ignores,    NULL},
