@@ -1,11 +1,9 @@
 /*
  * compat.h
  *   wrap-around header for all compability functions.
- *
- * $Id: compat.h,v 1.16 2011/07/09 15:07:48 thommey Exp $
  */
 /*
- * Copyright (C) 2000 - 2011 Eggheads Development Team
+ * Copyright (C) 2000 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,7 +28,9 @@
 #include "memset.h"
 #include "memcpy.h"
 #include "strcasecmp.h"
-#include "strdup.h"
 #include "strftime.h"
+#include "inet_ntop.h"
+#include "inet_pton.h"
+#include "gethostbyname2.h"
 
 #endif /* !__EGG_COMPAT_COMPAT_H */
