@@ -3,12 +3,10 @@
  *   activity on a dcc socket
  *   disconnect on a dcc socket
  *   ...and that's it!  (but it's a LOT)
- *
- * $Id: dcc.c,v 1.95 2011/10/02 18:21:35 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -43,26 +41,32 @@ extern int egg_numver, connect_timeout, conmask, backgrd, max_dcc,
            make_userfile, default_flags, raw_log, ignore_time,
            par_telnet_flood;
 
-struct dcc_t *dcc = NULL;       /* DCC list                                */
-int dcc_total = 0;              /* Total dcc's                             */
-char tempdir[121] = "";         /* Temporary directory
-                                 * (default: current directory)            */
+#ifdef TLS
+extern int tls_vfybots, ssl_files_loaded;
+
+int tls_vfyclients = 0;         /* Certificate validation mode for clients    */
+int tls_vfydcc = 0;             /* Verify DCC chat/send user certificates     */
+int tls_auth = 0;               /* Allow certificate authentication           */
+#endif
+
+struct dcc_t *dcc = NULL;       /* DCC list                                   */
+int dcc_total = 0;              /* Total dcc's                                */
 int require_p = 0;              /* Require 'p' access to get on the
-                                 * party line?                             */
+                                 * party line?                                */
 int allow_new_telnets = 0;      /* Allow people to introduce themselves
-                                 * via telnet                              */
-int stealth_telnets = 0;        /* Be paranoid? <cybah>                    */
-int use_telnet_banner = 0;      /* Display telnet banner?                  */
-char network[41] = "unknown-net";       /* Name of the IRC network you're on  */
-int password_timeout = 180;     /* Time to wait for a password from a user */
-int bot_timeout = 60;           /* Bot timeout value                       */
-int identtimeout = 5;           /* Timeout value for ident lookups         */
-int dupwait_timeout = 5;        /* Timeout for rejecting duplicate entries */
-int protect_telnet = 1;         /* Even bother with ident lookups :)       */
+                                 * via telnet                                 */
+int stealth_telnets = 0;        /* Be paranoid? <cybah>                       */
+int use_telnet_banner = 0;      /* Display telnet banner?                     */
+int password_timeout = 180;     /* Time to wait for a password from a user    */
+int bot_timeout = 60;           /* Bot timeout value                          */
+int identtimeout = 5;           /* Timeout value for ident lookups            */
+int dupwait_timeout = 5;        /* Timeout for rejecting duplicate entries    */
+int protect_telnet = 1;         /* Even bother with ident lookups :)          */
 int flood_telnet_thr = 5;       /* Number of telnet connections to be
-                                 * considered a flood                      */
-int flood_telnet_time = 60;     /* In how many seconds?                    */
-char bannerfile[121] = "text/banner";   /* File displayed on telnet login */
+                                 * considered a flood                         */
+int flood_telnet_time = 60;     /* In how many seconds?                       */
+char network[41] = "unknown-net";      /* Name of the IRC network you're on   */
+char bannerfile[121] = "text/banner";  /* File displayed on telnet login      */
 
 static void dcc_telnet_hostresolved(int);
 static void dcc_telnet_got_ident(int, char *);
@@ -123,20 +127,20 @@ static void strip_telnet(int sock, char *buf, int *len)
         /* WILL X -> response: DONT X */
         /* except WILL ECHO which we just smile and ignore */
         if (*(p + 1) != TLN_ECHO) {
-          write(sock, TLN_IAC_C TLN_DONT_C, 2);
-          write(sock, p + 1, 1);
+          safe_write(sock, TLN_IAC_C TLN_DONT_C, 2);
+          safe_write(sock, p + 1, 1);
         }
       } else if (*p == TLN_DO) {
         /* DO X -> response: WONT X */
         /* except DO ECHO which we just smile and ignore */
         if (*(p + 1) != TLN_ECHO) {
-          write(sock, TLN_IAC_C TLN_WONT_C, 2);
-          write(sock, p + 1, 1);
+          safe_write(sock, TLN_IAC_C TLN_WONT_C, 2);
+          safe_write(sock, p + 1, 1);
         }
       } else if (*p == TLN_AYT) {
         /* "Are You There?" */
         /* response is: "Hell, yes!" */
-        write(sock, "\r\nHell, yes!\r\n", 14);
+        safe_write(sock, "\r\nHell, yes!\r\n", 14);
       } else if (*p == TLN_IAC) {
         /* IAC character in data, escaped with another IAC */
         *o++ = *p++;
@@ -234,7 +238,6 @@ static void bot_version(int idx, char *par)
 #endif
   strncpyz(dcc[idx].u.bot->version, par, 120);
   putlog(LOG_BOTS, "*", DCC_LINKED, dcc[idx].nick);
-  chatout("*** Linked to %s\n", dcc[idx].nick);
   botnet_send_nlinked(idx, dcc[idx].nick, botnetnick, '!',
                       dcc[idx].u.bot->numver);
   touch_laston(dcc[idx].user, "linked", now);
@@ -266,17 +269,11 @@ void failed_link(int idx)
     return;
   }
 
-  /* Try next port */
+  /* Try next port, if it makes sense (no AF_UNSPEC, ...) */
   killsock(dcc[idx].sock);
-  dcc[idx].sock = getsock(SOCK_STRONGCONN);
-  dcc[idx].port++;
   dcc[idx].timeval = now;
-  if (dcc[idx].sock < 0 ||
-      open_telnet_raw(dcc[idx].sock, dcc[idx].addr ?
-                      iptostr(htonl(dcc[idx].addr)) : dcc[idx].host,
-                      dcc[idx].port) < 0) {
+  if (open_telnet(idx, dcc[idx].host, dcc[idx].port + 1) < 0)
     failed_link(idx);
-  }
 }
 
 static void cont_link(int idx, char *buf, int i)
@@ -305,7 +302,7 @@ static void cont_link(int idx, char *buf, int i)
                      "Unlinked %s (restructure) (lost %d bot%s and %d user%s)",
                      dcc[i].nick, bots, (bots != 1) ? "s" : "",
                      users, (users != 1) ? "s" : "");
-        chatout("*** %s\n", x);
+        putlog(LOG_BOTS, "*", "%s.", x);
         botnet_send_unlinked(i, dcc[i].nick, x);
         dprintf(i, "bye %s\n", "restructure");
         killsock(dcc[i].sock);
@@ -351,6 +348,13 @@ static void dcc_bot_new(int idx, char *buf, int x)
   struct userrec *u = get_user_by_handle(userlist, dcc[idx].nick);
   char *code;
 
+  if (raw_log) {
+    if (!strncmp(buf, "s ", 2))
+      putlog(LOG_BOTSHRIN, "*", "{m<-%s} %s", dcc[idx].nick, buf + 2);
+    else
+      putlog(LOG_BOTNETIN, "*", "[m<-%s] %s", dcc[idx].nick, buf);
+  }
+
   code = newsplit(&buf);
   if (!egg_strcasecmp(code, "*hello!"))
     greet_new_bot(idx);
@@ -362,6 +366,22 @@ static void dcc_bot_new(int idx, char *buf, int x)
   else if (!egg_strcasecmp(code, "passreq")) {
     char *pass = get_user(&USERENTRY_PASS, u);
 
+#ifdef TLS
+    /* We got a STARTTLS request earlier. Switch to ssl NOW. Doing this
+     * in two steps is necessary in order to synchronize the handshake.
+     */
+    if (dcc[idx].status & STAT_STARTTLS) {
+      dcc[idx].ssl = 1;
+      if (ssl_handshake(dcc[idx].sock, TLS_CONNECT, tls_vfybots, LOG_BOTS,
+                    dcc[idx].host, NULL)) {
+        putlog(LOG_BOTS, "*", "STARTTLS failed while linking to %s",
+               dcc[idx].nick);
+        if (!ssl_files_loaded)
+          putlog(LOG_BOTS, "*", "SSL cert and/or key file not loaded");
+      }
+      dcc[idx].status &= ~STAT_STARTTLS;
+    }
+#endif
     if (!pass || !strcmp(pass, "-")) {
       putlog(LOG_BOTS, "*", DCC_PASSREQ, dcc[idx].nick);
       dprintf(idx, "-\n");
@@ -374,6 +394,18 @@ static void dcc_bot_new(int idx, char *buf, int x)
       else
         dprintf(idx, "%s\n", pass);
     }
+#ifdef TLS
+  } else if (!egg_strcasecmp(code, "starttls") && !dcc[idx].ssl) {
+    /* Mark the connection for secure communication, but don't switch yet.
+     * The hub has to send a plaintext passreq right after the starttls command
+     * and if we switch now, we'll break the handshake. Instead, we'll only
+     * send a confirmation to the peer and wait for the passreq.
+     */
+    putlog(LOG_BOTS, "*", "Got STARTTLS from %s. Replying...", dcc[idx].nick);
+    dcc[idx].status |= STAT_STARTTLS;
+    /* needs to have space to be distinguished from a plaintext password */
+    dprintf(idx, "starttls -\n");
+#endif
   } else if (!egg_strcasecmp(code, "error"))
     putlog(LOG_BOTS, "*", DCC_LINKERROR, dcc[idx].nick, buf);
   /* Ignore otherwise */
@@ -416,6 +448,36 @@ static void free_dcc_bot_(int n, void *x)
   nfree(x);
 }
 
+static void out_dcc_bot(int idx, char *buf, void *x)
+{
+  size_t len = strlen(buf);
+  /* We don't really use x here, so "use it" for the compiler */
+  (void)x;
+
+  if (raw_log) {
+    /* strip \n from end as putlog appends this */
+    char *p = buf, *fnd = NULL;
+
+    if (len && buf[len - 1] == '\n') {
+      /* Make a copy as buf could be const */
+      fnd = nmalloc(len);
+      strncpyz(fnd, buf, len);
+      p = fnd;
+    }
+
+    if (!strncmp(p, "s ", 2))
+      putlog(LOG_BOTSHROUT, "*", "{m->%s} %s", dcc[idx].nick, p + 2);
+    else
+      putlog(LOG_BOTNETOUT, "*", "[m->%s] %s", dcc[idx].nick, p);
+
+    if (fnd)
+      nfree(fnd);
+
+  }
+
+  tputs(dcc[idx].sock, buf, len);
+}
+
 struct dcc_table DCC_BOT_NEW = {
   "BOT_NEW",
   0,
@@ -426,7 +488,7 @@ struct dcc_table DCC_BOT_NEW = {
   display_dcc_bot_new,
   expmem_dcc_bot_,
   free_dcc_bot_,
-  NULL
+  out_dcc_bot
 };
 
 /* Hash function for tandem bot commands */
@@ -438,10 +500,10 @@ static void dcc_bot(int idx, char *code, int i)
   int f;
 
   if (raw_log) {
-    if (code[0] == 's')
-      putlog(LOG_BOTSHARE, "*", "{%s} %s", dcc[idx].nick, code + 2);
+    if (!strncmp(code, "s ", 2))
+      putlog(LOG_BOTSHRIN, "*", "{m<-%s} %s", dcc[idx].nick, code + 2);
     else
-      putlog(LOG_BOTNET, "*", "[%s] %s", dcc[idx].nick, code);
+      putlog(LOG_BOTNETIN, "*", "[m<-%s] %s", dcc[idx].nick, code);
   }
   msg = strchr(code, ' ');
   if (msg) {
@@ -473,7 +535,6 @@ static void eof_dcc_bot(int idx)
                dcc[idx].nick, bots, (bots != 1) ? "s" : "", users,
                (users != 1) ? "s" : "");
   putlog(LOG_BOTS, "*", "%s.", x);
-  chatout("*** %s\n", x);
   botnet_send_unlinked(idx, dcc[idx].nick, x);
   killsock(dcc[idx].sock);
   lostdcc(idx);
@@ -511,7 +572,7 @@ struct dcc_table DCC_BOT = {
   display_dcc_bot,
   expmem_dcc_bot_,
   free_dcc_bot_,
-  NULL
+  out_dcc_bot
 };
 
 struct dcc_table DCC_FORK_BOT = {
@@ -524,7 +585,7 @@ struct dcc_table DCC_FORK_BOT = {
   display_dcc_fork_bot,
   expmem_dcc_bot_,
   free_dcc_bot_,
-  NULL
+  out_dcc_bot
 };
 
 /* This function generates a digest by combining a challenge consisting
@@ -578,6 +639,43 @@ static void dcc_chat_pass(int idx, char *buf, int atr)
     buf += 3; /* 'IAC','DO(DONT)','STATUS' */
   atr = dcc[idx].user ? dcc[idx].user->flags : 0;
 
+  if (atr & USER_BOT) {
+    if (raw_log) {
+      if (!strncmp(buf, "s ", 2))
+        putlog(LOG_BOTSHRIN, "*", "{m<-%s} %s", dcc[idx].nick, buf + 2);
+      else
+        putlog(LOG_BOTNETIN, "*", "[m<-%s] %s", dcc[idx].nick, buf);
+    }
+#ifdef TLS
+    if (!egg_strncasecmp(buf, "starttls ", 9)) {
+      dcc[idx].ssl = 1;
+      if (ssl_handshake(dcc[idx].sock, TLS_LISTEN, tls_vfybots, LOG_BOTS,
+                        dcc[idx].host, NULL)) {
+        killsock(dcc[idx].sock);
+        lostdcc(idx);
+      }
+      return;
+    }
+#endif
+    /* No password set? */
+    if (u_pass_match(dcc[idx].user, "-")) {
+      char ps[20];
+
+      makepass(ps);
+      set_user(&USERENTRY_PASS, dcc[idx].user, ps);
+      changeover_dcc(idx, &DCC_BOT_NEW, sizeof(struct bot_info));
+
+      dcc[idx].status = STAT_CALLED;
+      dprintf(idx, "*hello!\n");
+      greet_new_bot(idx);
+#ifdef NO_OLD_BOTNET
+      dprintf(idx, "h %s\n", ps);
+#else
+      dprintf(idx, "handshake %s\n", ps);
+#endif
+      return;
+    }
+  }
   /* Check for MD5 digest from remote _bot_. <cybah> */
   if ((atr & USER_BOT) && !egg_strncasecmp(buf, "digest ", 7)) {
     if (dcc_bot_check_digest(idx, buf + 7)) {
@@ -590,7 +688,12 @@ static void dcc_chat_pass(int idx, char *buf, int atr)
       return;
     } else {
       /* Invalid password/digest */
+      /* change here temp to use bot output */
+      struct dcc_table *old = dcc[idx].type;
+      dcc[idx].type = &DCC_BOT_NEW;
       dprintf(idx, "badpass\n");
+      /* change back to original for general kill */
+      dcc[idx].type = old;
       putlog(LOG_MISC, "*", DCC_BADLOGIN, dcc[idx].nick, dcc[idx].host,
              dcc[idx].port);
       killsock(dcc[idx].sock);
@@ -599,7 +702,14 @@ static void dcc_chat_pass(int idx, char *buf, int atr)
     }
   }
 
+#ifdef TLS
+  /* Skip checking the password if the user is already identified by
+   * fingerprint.
+   */
+  if (dcc[idx].status & STAT_FPRINT || u_pass_match(dcc[idx].user, buf)) {
+#else
   if (u_pass_match(dcc[idx].user, buf)) {
+#endif
     if (atr & USER_BOT) {
       nfree(dcc[idx].u.chat);
       dcc[idx].type = &DCC_BOT_NEW;
@@ -626,9 +736,13 @@ static void dcc_chat_pass(int idx, char *buf, int atr)
       dcc_chatter(idx);
     }
   } else {
-    if (atr & USER_BOT)
+    if (atr & USER_BOT) {
+      /* change here temp to use bot output */
+      struct dcc_table *old = dcc[idx].type;
+      dcc[idx].type = &DCC_BOT_NEW;
       dprintf(idx, "badpass\n");
-    else
+      dcc[idx].type = old;
+    } else
       dprintf(idx, DCC_HOUSTON);
     putlog(LOG_MISC, "*", DCC_BADLOGIN, dcc[idx].nick,
            dcc[idx].host, dcc[idx].port);
@@ -637,7 +751,7 @@ static void dcc_chat_pass(int idx, char *buf, int atr)
       if (dcc[idx].status & STAT_TELNET)
         tputs(dcc[idx].sock, TLN_IAC_C TLN_WONT_C TLN_ECHO_C "\n", 4);
       dcc[idx].user = get_user_by_handle(userlist, dcc[idx].u.chat->away);
-      strcpy(dcc[idx].nick, dcc[idx].u.chat->away);
+      strncpyz(dcc[idx].nick, dcc[idx].u.chat->away, sizeof dcc[idx].nick);
       nfree(dcc[idx].u.chat->away);
       nfree(dcc[idx].u.chat->su_nick);
       dcc[idx].u.chat->away = NULL;
@@ -731,12 +845,6 @@ void strip_mirc_codes(int flags, char *text)
 
   while (*text) {
     switch (*text) {
-    case 2:                    /* Bold text */
-      if (flags & STRIP_BOLD) {
-        text++;
-        continue;
-      }
-      break;
     case 3:                    /* mIRC colors? */
       if (flags & STRIP_COLOR) {
         if (egg_isdigit(text[1])) { /* Is the first char a number? */
@@ -754,20 +862,20 @@ void strip_mirc_codes(int flags, char *text)
         continue;
       }
       break;
-    case 7:
-      if (flags & STRIP_BELLS) {
+    case 2:                    /* Bold text */
+      if (flags & STRIP_BOLD) {
         text++;
         continue;
       }
       break;
     case 0x16:                 /* Reverse video */
-      if (flags & STRIP_REV) {
+      if (flags & STRIP_REVERSE) {
         text++;
         continue;
       }
       break;
     case 0x1f:                 /* Underlined text */
-      if (flags & STRIP_UNDER) {
+      if (flags & STRIP_UNDERLINE) {
         text++;
         continue;
       }
@@ -785,6 +893,24 @@ void strip_mirc_codes(int flags, char *text)
         continue;
       }
       break;
+    case 7:
+      if (flags & STRIP_BELLS) {
+        text++;
+        continue;
+      }
+      break;
+    case 017:                  /* Ordinary (mIRC ctrl+o) */
+      if (flags & STRIP_ORDINARY) {
+        text++;
+        continue;
+      }
+      break;
+    case 29:                   /* Italics */
+      if (flags & STRIP_ITALICS) {
+        text++;
+        continue;
+      }
+      break;
     }
     *dd++ = *text++;            /* Move on to the next char */
   }
@@ -833,7 +959,6 @@ static void append_line(int idx, char *line)
   }
 }
 
-
 static void out_dcc_general(int idx, char *buf, void *x)
 {
   register struct chat_info *p = (struct chat_info *) x;
@@ -978,7 +1103,7 @@ static void dcc_chat(int idx, char *buf, int i)
           if (dcc[idx].u.chat->su_nick) {
             dcc[idx].user = get_user_by_handle(userlist,
                                                dcc[idx].u.chat->su_nick);
-            strcpy(dcc[idx].nick, dcc[idx].u.chat->su_nick);
+            strncpyz(dcc[idx].nick, dcc[idx].u.chat->su_nick, sizeof dcc[idx].nick);
             dcc[idx].type = &DCC_CHAT;
             dprintf(idx, "Returning to real nick %s!\n",
                     dcc[idx].u.chat->su_nick);
@@ -1096,7 +1221,7 @@ static int detect_telnet_flood(char *floodhost)
   if (!flood_telnet_thr || (glob_friend(fr) && !par_telnet_flood))
     return 0;                   /* No flood protection */
   if (egg_strcasecmp(lasttelnethost, floodhost)) {      /* New... */
-    strcpy(lasttelnethost, floodhost);
+    strncpyz(lasttelnethost, floodhost, sizeof lasttelnethost);
     lasttelnettime = now;
     lasttelnets = 0;
     return 0;
@@ -1123,25 +1248,24 @@ static int detect_telnet_flood(char *floodhost)
 
 static void dcc_telnet(int idx, char *buf, int i)
 {
-  unsigned long ip;
   unsigned short port;
   int j = 0, sock;
-  char s[UHOSTLEN + 1];
 
   if (dcc_total + 1 > max_dcc && increase_socks_max()) {
-    j = answer(dcc[idx].sock, s, &ip, &port, 0);
+    sockname_t name;
+    j = answer(dcc[idx].sock, &name, &port, 0);
     if (j != -1) {
       dprintf(-j, "Sorry, too many connections already.\r\n");
       killsock(j);
     }
     return;
   }
-  sock = answer(dcc[idx].sock, s, &ip, &port, 0);
+  i = new_dcc(&DCC_DNSWAIT, sizeof(struct dns_info));
+  sock = answer(dcc[idx].sock, &dcc[i].sockname, &port, 0);
   while ((sock == -1) && (errno == EAGAIN))
-    sock = answer(sock, s, &ip, &port, 0);
+    sock = answer(dcc[idx].sock, &dcc[i].sockname, &port, 0);
   if (sock < 0) {
-    neterror(s);
-    putlog(LOG_MISC, "*", DCC_FAILED, s);
+    putlog(LOG_MISC, "*", DCC_FAILED, strerror(errno));
     return;
   }
   /* Buffer data received on this socket.  */
@@ -1152,31 +1276,48 @@ static void dcc_telnet(int idx, char *buf, int i)
 #else
   if (port < 1024 || port > 65535) {
 #endif
-    putlog(LOG_BOTS, "*", DCC_BADSRC, s, port);
+    putlog(LOG_BOTS, "*", DCC_BADSRC, iptostr(&dcc[i].sockname.addr.sa), port);
     killsock(sock);
+    lostdcc(i);
     return;
   }
 
-  i = new_dcc(&DCC_DNSWAIT, sizeof(struct dns_info));
+  dcc[i].u.dns->ip = &dcc[i].sockname;
   dcc[i].sock = sock;
-  dcc[i].addr = ip;
   dcc[i].port = port;
+#ifdef TLS
+  dcc[i].ssl = dcc[idx].ssl;
+  if (dcc[i].ssl) {
+    /* In order to support certificate CN validation we need to wait for the dns
+     * lookup to complete. Additional we need to mark the socket virtual so we
+     * won't read SSL client-hello before ssl_handshake has happened.
+     * NOTE: TLS_VERIFYCN configurations value is inverted
+     */
+    if (!(tls_vfyclients & TLS_VERIFYCN))
+      threaddata()->socklist[findsock(sock)].flags |= SOCK_VIRTUAL;
+    else if (ssl_handshake(dcc[i].sock, TLS_LISTEN, tls_vfyclients,
+        LOG_MISC, NULL, NULL)) {
+      killsock(dcc[i].sock);
+      lostdcc(i);
+      return;
+    }
+  }
+#endif
   dcc[i].timeval = now;
   strcpy(dcc[i].nick, "*");
-  dcc[i].u.dns->ip = ip;
   dcc[i].u.dns->dns_success = dcc_telnet_hostresolved;
   dcc[i].u.dns->dns_failure = dcc_telnet_hostresolved;
   dcc[i].u.dns->dns_type = RES_HOSTBYIP;
   dcc[i].u.dns->ibuf = dcc[idx].sock;
   dcc[i].u.dns->type = &DCC_IDENTWAIT;
-  dcc_dnshostbyip(ip);
+  dcc_dnshostbyip(&dcc[i].sockname);
 }
 
 static void dcc_telnet_hostresolved(int i)
 {
   int idx;
   int j = 0, sock;
-  char s[UHOSTLEN], s2[UHOSTLEN + 20];
+  char s[UHOSTLEN + 20], *userhost;
 
   strncpyz(dcc[i].host, dcc[i].u.dns->host, UHOSTLEN);
 
@@ -1201,35 +1342,76 @@ static void dcc_telnet_hostresolved(int i)
       return;
     }
   }
-  sprintf(s2, "-telnet!telnet@%s", dcc[i].host);
-  if (match_ignore(s2) || detect_telnet_flood(s2)) {
+  sprintf(s, "-telnet!telnet@%s", dcc[i].host);
+  userhost = s + strlen("-telnet!");
+  if (match_ignore(s) || detect_telnet_flood(s)) {
     killsock(dcc[i].sock);
     lostdcc(i);
     return;
   }
 
+  putlog(LOG_MISC, "*", DCC_TELCONN, dcc[i].host, dcc[i].port);
+
+#ifdef TLS
+  /* Remove SOCK_VIRTUAL and call ssl_handshake
+   * Check out dcc_telnet for more details
+   */
+  if (dcc[i].ssl && !(tls_vfyclients & TLS_VERIFYCN)) {
+    threaddata()->socklist[findsock(dcc[i].sock)].flags &= ~SOCK_VIRTUAL;
+    if (ssl_handshake(dcc[i].sock, TLS_LISTEN, tls_vfyclients,
+        LOG_MISC, dcc[i].host, NULL)) {
+      killsock(dcc[i].sock);
+      lostdcc(i);
+      return;
+    }
+  }
+#endif
+
+  /* Skip ident lookup for public script listeners */
+  if ((dcc[idx].status & LSTN_PUBLIC) && !strcmp(dcc[idx].nick, "(script)")) {
+    changeover_dcc(i, &DCC_SOCKET, 0);
+    dcc[i].u.other = NULL;
+    strcpy(dcc[i].nick, "*");
+    strncpyz(dcc[i].host, userhost, UHOSTLEN);
+    check_tcl_listen(dcc[idx].host, dcc[i].sock);
+    return;
+  }
+
+  /* Skip ident lookup if disabled */
+  if (identtimeout <= 0) {
+    dcc[i].u.ident_sock = dcc[idx].sock;
+    dcc_telnet_got_ident(i, userhost);
+    return;
+  }
+
   changeover_dcc(i, &DCC_IDENTWAIT, 0);
   dcc[i].timeval = now;
   dcc[i].u.ident_sock = dcc[idx].sock;
-  sock = open_telnet(iptostr(htonl(dcc[i].addr)), 113);
-  putlog(LOG_MISC, "*", DCC_TELCONN, dcc[i].host, dcc[i].port);
-  s[0] = 0;
-  if (sock < 0) {
-    if (sock == -2)
-      strcpy(s, "DNS lookup failed for ident");
-    else
-      neterror(s);
-  } else {
-    j = new_dcc(&DCC_IDENT, 0);
-    if (j < 0) {
-      killsock(sock);
-      strcpy(s, "No Free DCC's");
+  sock = -1;
+  j = new_dcc(&DCC_IDENT, 0);
+  if (j < 0)
+    putlog(LOG_MISC, "*", DCC_IDENTFAIL, dcc[i].host, strerror(errno));
+  else {
+    egg_memcpy(&dcc[j].sockname, &dcc[i].sockname, sizeof(sockname_t));
+    dcc[j].sock = getsock(dcc[j].sockname.family, 0);
+    if (dcc[j].sock >= 0) {
+      sockname_t name;
+      name.addrlen = sizeof(name.addr);
+      getsockname(dcc[i].sock, &name.addr.sa, &name.addrlen);
+      bind(dcc[j].sock, &name.addr.sa, name.addrlen);
+      setsnport(dcc[j].sockname, 113);
+      if (connect(dcc[j].sock, &dcc[j].sockname.addr.sa,
+          dcc[j].sockname.addrlen) < 0 && (errno != EINPROGRESS)) {
+        killsock(dcc[j].sock);
+        lostdcc(j);
+        putlog(LOG_MISC, "*", DCC_IDENTFAIL, dcc[i].host, strerror(errno));
+        j = 0;
+      }
+      sock = dcc[j].sock;
     }
   }
-  if (s[0]) {
-    putlog(LOG_MISC, "*", DCC_IDENTFAIL, dcc[i].host, s);
-    sprintf(s, "telnet@%s", dcc[i].host);
-    dcc_telnet_got_ident(i, s);
+  if (j < 0) {
+    dcc_telnet_got_ident(i, userhost);
     return;
   }
   dcc[j].sock = sock;
@@ -1337,7 +1519,7 @@ struct dcc_table DCC_DUPWAIT = {
   display_dupwait,
   expmem_dupwait,
   kill_dupwait,
-  NULL
+  out_dcc_bot
 };
 
 /* This function is called if a bot gets removed from the list. It checks
@@ -1361,6 +1543,7 @@ static void dcc_telnet_id(int idx, char *buf, int atr)
 {
   int ok = 0;
   struct flag_record fr = { FR_GLOBAL | FR_CHAN | FR_ANYWH, 0, 0, 0, 0, 0 };
+  struct dcc_table *old = dcc[idx].type;
 
   if (detect_telnet((unsigned char *) buf)) {
     dcc[idx].status |= STAT_TELNET;
@@ -1378,6 +1561,31 @@ static void dcc_telnet_id(int idx, char *buf, int atr)
   }
   dcc[idx].user = get_user_by_handle(userlist, buf);
   get_user_flagrec(dcc[idx].user, &fr, NULL);
+
+  if (glob_bot(fr) && raw_log) {
+    if (!strncmp(buf, "s ", 2))
+      putlog(LOG_BOTSHRIN, "*", "{m<-%s} %s", dcc[idx].user->handle, buf + 2);
+    else
+      putlog(LOG_BOTNETIN, "*", "[m<-%s] %s", dcc[idx].user->handle, buf);
+  }
+#ifdef TLS
+  if (dcc[idx].ssl && (tls_auth == 2)) {
+    const char *uid = ssl_getuid(dcc[idx].sock);
+
+    if (!uid || strcasecmp(uid, buf)) {
+      if (glob_bot(fr)) {
+        /* change here temp to use bot output */
+        dcc[idx].type = &DCC_BOT_NEW;
+        dprintf(idx, "error Certificate UID doesn't match handle\n");
+        dcc[idx].type = old;
+      } else
+        dprintf(idx, "Your certificate UID doesn't match your handle.\n");
+      killsock(dcc[idx].sock);
+      lostdcc(idx);
+      return;
+    }
+  }
+#endif
   /* Make sure users-only/bots-only connects are honored */
   if ((dcc[idx].status & STAT_BOTONLY) && !glob_bot(fr)) {
     dprintf(idx, "This telnet port is for bots only.\n");
@@ -1387,7 +1595,10 @@ static void dcc_telnet_id(int idx, char *buf, int atr)
     return;
   }
   if ((dcc[idx].status & STAT_USRONLY) && glob_bot(fr)) {
+    /* change here temp to use bot output */
+    dcc[idx].type = &DCC_BOT_NEW;
     dprintf(idx, "error Only users may connect at this port.\n");
+    dcc[idx].type = old;
     putlog(LOG_BOTS, "*", DCC_NONUSER, dcc[idx].host);
     killsock(dcc[idx].sock);
     lostdcc(idx);
@@ -1412,16 +1623,19 @@ static void dcc_telnet_id(int idx, char *buf, int atr)
 
   if (!ok) {
     dprintf(idx, "You don't have access.\n");
-    putlog(LOG_BOTS, "*", DCC_INVHANDLE, dcc[idx].host, buf);
+    putlog(LOG_MISC, "*", DCC_INVHANDLE, dcc[idx].host, buf);
     killsock(dcc[idx].sock);
     lostdcc(idx);
     return;
   }
   correct_handle(buf);
-  strcpy(dcc[idx].nick, buf);
+  strncpyz(dcc[idx].nick, buf, sizeof dcc[idx].nick);
   if (glob_bot(fr)) {
     if (!egg_strcasecmp(botnetnick, dcc[idx].nick)) {
+      /* change here temp to use bot output */
+      dcc[idx].type = &DCC_BOT_NEW;
       dprintf(idx, "error You cannot link using my botnetnick.\n");
+      dcc[idx].type = old;
       putlog(LOG_BOTS, "*", DCC_MYBOTNETNICK, dcc[idx].host);
       killsock(dcc[idx].sock);
       lostdcc(idx);
@@ -1440,31 +1654,86 @@ static void dcc_telnet_id(int idx, char *buf, int atr)
   dcc_telnet_pass(idx, atr);
 }
 
+#ifdef TLS
+int dcc_fingerprint(int idx)
+{
+  char *cf, *uf;
+  struct flag_record fr = { FR_GLOBAL | FR_CHAN | FR_ANYWH, 0, 0, 0, 0, 0 };
+
+  get_user_flagrec(dcc[idx].user, &fr, NULL);
+  /* Check if fingerprint authentication is allowed or required. */
+  if (dcc[idx].ssl && tls_auth) {
+    /* Get the fingerprint of the current certificate */
+    cf = ssl_getfp(dcc[idx].sock);
+    /* Get the fingerprint of the user, if set */
+    uf = get_user(&USERENTRY_FPRINT, dcc[idx].user);
+    if (cf && uf && !strcasecmp(cf, uf)) {
+      if (!glob_bot(fr))
+        dprintf(idx, "Used your fingerprint for automatic authentication.\n");
+      dcc[idx].status |= STAT_FPRINT;
+      dcc_chat_pass(idx, "+", 1);
+    /* Required? */
+    } else if (tls_auth == 2) {
+      if (glob_bot(fr)) {
+        /* change here temp to use bot output */
+        struct dcc_table *old = dcc[idx].type;
+        dcc[idx].type = &DCC_BOT_NEW;
+        dprintf(idx, "error fingerprint required\n");
+        dcc[idx].type = old;
+      } else
+        dprintf(idx, "Certificate authentication required. "
+                "You need to set your fingerprint.\n");
+      killsock(dcc[idx].sock);
+      lostdcc(idx);
+    }
+    return 0;
+  }
+  return 1;
+}
+#endif
+
 static void dcc_telnet_pass(int idx, int atr)
 {
   int ok = 0;
   struct flag_record fr = { FR_GLOBAL | FR_CHAN | FR_ANYWH, 0, 0, 0, 0, 0 };
 
   get_user_flagrec(dcc[idx].user, &fr, NULL);
-  /* No password set? */
-  if (u_pass_match(dcc[idx].user, "-")) {
-    if (glob_bot(fr)) {
-      char ps[20];
+#ifdef TLS
+  /* Check if fingerprint authentication is allowed or required. */
+  if (dcc[idx].ssl && tls_auth) {
+    char *cf, *uf;
 
-      makepass(ps);
-      set_user(&USERENTRY_PASS, dcc[idx].user, ps);
-      changeover_dcc(idx, &DCC_BOT_NEW, sizeof(struct bot_info));
+    /* Get the fingerprint of the current certificate */
+    cf = ssl_getfp(dcc[idx].sock);
+    /* Get the fingerprint of the user, if set */
+    uf = get_user(&USERENTRY_FPRINT, dcc[idx].user);
+    if (cf && uf && !strcasecmp(cf, uf)) {
+      char fakepass[2] = "+";
 
-      dcc[idx].status = STAT_CALLED;
-      dprintf(idx, "*hello!\n");
-      greet_new_bot(idx);
-#ifdef NO_OLD_BOTNET
-      dprintf(idx, "h %s\n", ps);
-#else
-      dprintf(idx, "handshake %s\n", ps);
-#endif
+      if (!glob_bot(fr))
+        dprintf(idx, "Used your fingerprint for automatic authentication.\n");
+      dcc[idx].status |= STAT_FPRINT;
+      dcc_chat_pass(idx, fakepass, 1);
+      return;
+    /* Required? */
+    } else if (tls_auth == 2) {
+      if (glob_bot(fr)) {
+        /* change here temp to use bot output */
+        struct dcc_table *old = dcc[idx].type;
+        dcc[idx].type = &DCC_BOT_NEW;
+        dprintf(idx, "error fingerprint required\n");
+        dcc[idx].type = old;
+      } else
+        dprintf(idx, "Certificate authentication required. "
+                "You need to set your fingerprint.\n");
+      killsock(dcc[idx].sock);
+      lostdcc(idx);
       return;
     }
+  }
+#endif
+  /* No password set? */
+  if (!glob_bot(fr) && u_pass_match(dcc[idx].user, "-")) {
     dprintf(idx, "Can't telnet until you have a password set.\n");
     putlog(LOG_MISC, "*", DCC_NOPASS, dcc[idx].nick, dcc[idx].host);
     killsock(dcc[idx].sock);
@@ -1503,6 +1772,19 @@ static void dcc_telnet_pass(int idx, int atr)
   }
 
   if (glob_bot(fr)) {
+    /* change here temp to use bot output */
+    struct dcc_table *old = dcc[idx].type;
+    dcc[idx].type = &DCC_BOT_NEW;
+#ifdef TLS
+  /* Ask the peer to switch to ssl communication. We'll continue using plain
+   * text, until it replies with starttls itself. Bots which don't support ssl
+   * will simply ignore the request and everything will go on as usual.
+   */
+    if (!dcc[idx].ssl) {
+      dprintf(idx, "starttls\n");
+      putlog(LOG_BOTS, "*", "Sent STARTTLS to %s...", dcc[idx].nick);
+    }
+#endif
     /* Must generate a string consisting of our process ID and the current
      * time. The bot will add it's password to the end and use it to generate
      * an MD5 checksum (always 128bit). The checksum is sent back and this
@@ -1515,6 +1797,7 @@ static void dcc_telnet_pass(int idx, int atr)
      */
     putlog(LOG_BOTS, "*", "Challenging %s...", dcc[idx].nick);
     dprintf(idx, "passreq <%x%x@%s>\n", getpid(), dcc[idx].timeval, botnetnick);
+    dcc[idx].type = old;
   } else {
     /* NOTE: The MD5 digest used above to prevent cleartext passwords being
      *       sent across the net will _only_ work when we have the cleartext
@@ -1527,7 +1810,7 @@ static void dcc_telnet_pass(int idx, int atr)
     /* Turn off remote telnet echo (send IAC WILL ECHO). */
     if (dcc[idx].status & STAT_TELNET) {
       char buf[1030];
-      snprintf(buf, sizeof buf, "\n%s%s\r\n", escape_telnet(DCC_ENTERPASS),
+      egg_snprintf(buf, sizeof buf, "\n%s%s\r\n", escape_telnet(DCC_ENTERPASS),
                TLN_IAC_C TLN_WILL_C TLN_ECHO_C);
       tputs(dcc[idx].sock, buf, strlen(buf));
     } else
@@ -1769,6 +2052,7 @@ static int call_tcl_func(char *name, int idx, char *args)
   Tcl_SetVar(interp, "_a", args, 0);
   if (Tcl_VarEval(interp, name, " $_n $_a", NULL) == TCL_ERROR) {
     putlog(LOG_MISC, "*", DCC_TCLERROR, name, tcl_resultstring());
+    Tcl_BackgroundError(interp);
     return -1;
   }
   return tcl_resultint();
@@ -1805,13 +2089,12 @@ static void dcc_script(int idx, char *buf, int len)
     }
     if (dcc[idx].type == &DCC_CHAT) {
       if (dcc[idx].u.chat->channel >= 0) {
-        chanout_but(-1, dcc[idx].u.chat->channel, DCC_JOIN, dcc[idx].nick);
+        chanout_but(-1, dcc[idx].u.chat->channel, DCC_RETURN, dcc[idx].nick);
         if (dcc[idx].u.chat->channel < 10000)
           botnet_send_join_idx(idx, -1);
         check_tcl_chjn(botnetnick, dcc[idx].nick, dcc[idx].u.chat->channel,
                        geticon(idx), dcc[idx].sock, dcc[idx].host);
       }
-      check_tcl_chon(dcc[idx].nick, dcc[idx].sock);
     }
   }
 }
@@ -2087,6 +2370,7 @@ static void dcc_telnet_got_ident(int i, char *host)
     lostdcc(i);
     return;
   }
+
   /* Script? */
   if (!strcmp(dcc[idx].nick, "(script)")) {
     dcc[i].type = &DCC_SOCKET;
