@@ -3,12 +3,10 @@
  *   Eggdrop compile-time settings
  *
  *   IF YOU ALTER THIS FILE, YOU NEED TO RECOMPILE THE BOT.
- *
- * $Id: eggdrop.h,v 1.81 2011/07/09 15:07:48 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -49,7 +47,7 @@
  *       You should leave this at 32 characters and modify nick-len in the
  *       configuration file instead.
  */
-#define HANDLEN 9   /* valid values 9->NICKMAX  */
+#define HANDLEN 32   /* valid values 9->NICKMAX  */
 #define NICKMAX 32  /* valid values HANDLEN->32 */
 
 
@@ -274,11 +272,14 @@
 typedef int socklen_t;
 #endif
 
+#ifdef TLS
+#  include <openssl/ssl.h>
+#endif
+
 /*
  *    Handy aliases for memory tracking and core dumps
  */
 
-#define nstrdup(x)    n_strdup((x),__FILE__,__LINE__)
 #define nmalloc(x)    n_malloc((x),__FILE__,__LINE__)
 #define nrealloc(x,y) n_realloc((x),(y),__FILE__,__LINE__)
 #define nfree(x)      n_free((x),__FILE__,__LINE__)
@@ -307,19 +308,9 @@ typedef int socklen_t;
 #  define free(x)   dont_use_old_free(x)
 #endif /* !COMPILING_MEM */
 
-/* 32 bit type */
-#if (SIZEOF_INT == 4)
-typedef unsigned int u_32bit_t;
-#else
-#  if (SIZEOF_LONG == 4)
-typedef unsigned long u_32bit_t;
-#  else
-#    include "Error: Can't find 32bit type."
-#  endif
-#endif
-
-typedef unsigned short int u_16bit_t;
-typedef unsigned char u_8bit_t;
+typedef uint8_t u_8bit_t;
+typedef uint16_t u_16bit_t;
+typedef uint32_t u_32bit_t;
 
 /* IP type */
 typedef u_32bit_t IP;
@@ -344,6 +335,25 @@ typedef u_32bit_t IP;
 typedef intptr_t (*Function) ();
 typedef int (*IntFunc) ();
 
+#ifdef IPV6
+#include "compat/in6.h"
+#endif
+
+#include <sys/socket.h>
+#include <netinet/in.h>
+
+typedef struct {
+  int family;
+  socklen_t addrlen;
+  union {
+    struct sockaddr sa;
+    struct sockaddr_in s4;
+#ifdef IPV6
+    struct sockaddr_in6 s6;
+#endif
+  } addr;
+} sockname_t;
+
 /* Public structure for the listening port map */
 struct portmap {
   int realport;
@@ -371,7 +381,11 @@ struct userrec;
 struct dcc_t {
   long sock;                    /* This should be a long to keep 64-bit machines sane. */
   IP addr;                      /* IP address in host network byte order. */
+  sockname_t sockname;          /* IPv4/IPv6 sockaddr placeholder */
   unsigned int port;
+#ifdef TLS
+  int ssl;
+#endif
   struct userrec *user;
   char nick[NICKLEN];
   char host[UHOSTLEN];
@@ -480,7 +494,7 @@ struct dns_info {
   char *cbuf;                   /* temporary buffer. Memory will be free'd
                                  * as soon as dns_info is free'd           */
   char *cptr;                   /* temporary pointer                       */
-  IP ip;                        /* IP address                              */
+  sockname_t *ip;               /* pointer to sockname with ipv4/6 address */
   int ibuf;                     /* temporary buffer for one integer        */
   char dns_type;                /* lookup type, e.g. RES_HOSTBYIP          */
   struct dcc_table *type;       /* type of the dcc table we are making the
@@ -517,8 +531,8 @@ struct dupwait_info {
 
 /* For dcc chat & files. */
 #define STAT_ECHO    0x00001    /* echo commands back?                  */
-#define STAT_DENY    0x00002    /* bad username (ignore password & deny
-                                 * access)                              */
+#define STAT_FPRINT  0x00002    /* fingerprint auth (ignore password &
+                                 * allow access)                        */
 #define STAT_CHAT    0x00004    /* in file-system but may return        */
 #define STAT_TELNET  0x00008    /* connected via telnet                 */
 #define STAT_PARTY   0x00010    /* only on party line via 'p' flag      */
@@ -527,13 +541,15 @@ struct dupwait_info {
 #define STAT_PAGE    0x00080    /* page output to the user              */
 
 /* For stripping out mIRC codes. */
-#define STRIP_COLOR  0x00001    /* remove mIRC color codes              */
-#define STRIP_BOLD   0x00002    /* remove bold codes                    */
-#define STRIP_REV    0x00004    /* remove reverse video codes           */
-#define STRIP_UNDER  0x00008    /* remove underline codes               */
-#define STRIP_ANSI   0x00010    /* remove ALL ANSI codes                */
-#define STRIP_BELLS  0x00020    /* remove ctrl-g's                      */
-#define STRIP_ALL    0x0007F    /* remove every damn thing!             */
+#define STRIP_COLOR     0x00001    /* remove mIRC color codes            */
+#define STRIP_BOLD      0x00002    /* remove bold codes                  */
+#define STRIP_REVERSE   0x00004    /* remove reverse video codes         */
+#define STRIP_UNDERLINE 0x00008    /* remove underline codes             */
+#define STRIP_ANSI      0x00010    /* remove ALL ANSI codes              */
+#define STRIP_BELLS     0x00020    /* remove ctrl-g's                    */
+#define STRIP_ORDINARY  0x00040    /* remove ordinary codes              */
+#define STRIP_ITALICS   0x00080    /* remove italics codes               */
+#define STRIP_ALL       0x000FF    /* remove every damn thing!           */
 
 /* For dcc bot links. */
 #define STAT_PINGED  0x00001    /* waiting for ping to return            */
@@ -547,6 +563,9 @@ struct dupwait_info {
 #define STAT_LINKING 0x00100    /* the bot is currently going through
                                  * the linking stage                     */
 #define STAT_AGGRESSIVE 0x00200 /* aggressively sharing with this bot    */
+#ifdef TLS
+#define STAT_STARTTLS   0x00400 /* have we sent a starttls request?      */
+#endif
 
 /* Flags for listening sockets */
 #define LSTN_PUBLIC  0x000001   /* No access restrictions               */
@@ -603,9 +622,12 @@ typedef struct {
 #define LOG_DEBUG    0x040000   /* d   debug                            */
 #define LOG_WALL     0x080000   /* w   wallops                          */
 #define LOG_SRVOUT   0x100000   /* v   server output                    */
-#define LOG_BOTNET   0x200000   /* t   botnet traffic                   */
-#define LOG_BOTSHARE 0x400000   /* h   share traffic                    */
-#define LOG_ALL      0x7fffff   /* (dump to all logfiles)               */
+#define LOG_BOTNETIN 0x200000   /* t   incoming botnet traffic          */
+#define LOG_BOTSHRIN 0x400000   /* h   incoming share traffic           */
+#define LOG_BOTNETOUT 0x800000  /* u   outgoing botnet traffic          */
+#define LOG_BOTSHROUT 0x1000000 /* g   outgoing share traffic           */
+#define LOG_BOTMSG   0x2000000  /* l   linked bot messages              */
+#define LOG_ALL      0x3ffffff  /* (dump to all logfiles)               */
 
 /* Internal logfile flags
  */
@@ -671,6 +693,30 @@ enum {
 #define HELP_TEXT       2
 #define HELP_IRC        16
 
+#ifdef TLS
+/* TLS generic flags */
+#  define TLS_LISTEN            0x80000000
+#  define TLS_CONNECT           0x40000000
+#  define TLS_DEPTH0            0x20000000
+
+/* TLS verification flags */
+#  define TLS_VERIFYPEER        0x00000001
+#  define TLS_VERIFYISSUER      0x00000002
+#  define TLS_VERIFYCN          0x00000004
+#  define TLS_VERIFYTO          0x00000008
+#  define TLS_VERIFYFROM        0x00000010
+#  define TLS_VERIFYREV         0x00000020
+
+/* Context information to attach to SSL sockets */
+typedef struct {
+  int flags;			/* listen/connect, generic ssl flags      */
+  int verify;			/* certificate validation mode            */
+  int loglevel;			/* log level to output TLS information to */
+  char host[256];		/* host or IP for certificate validation  */
+  IntFunc cb;
+} ssl_appdata;
+#endif /* TLS */
+
 /* These are used by the net module to keep track of sockets and what's
  * queued on them
  */
@@ -689,6 +735,9 @@ struct tclsock_handler {
 
 typedef struct sock_list {
   int sock;
+#ifdef TLS
+  SSL *ssl;
+#endif
   short flags;
   union {
     struct sock_handler sock;
