@@ -1,12 +1,10 @@
 /*
  * flags.c -- handles:
  *   all the flag matching/conversion functions in one neat package :)
- *
- * $Id: flags.c,v 1.37 2011/02/13 14:19:33 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -65,6 +63,10 @@ int logmodes(char *s)
     case 'B':
       res |= LOG_BOTS;
       break;
+    case 'l':
+    case 'L':
+      res |= LOG_BOTMSG;
+      break;
     case 'r':
     case 'R':
       res |= raw_log ? LOG_RAW : 0;
@@ -91,11 +93,19 @@ int logmodes(char *s)
       break;
     case 't':
     case 'T':
-      res |= raw_log ? LOG_BOTNET : 0;
+      res |= raw_log ? LOG_BOTNETIN : 0;
+      break;
+    case 'u':
+    case 'U':
+      res |= raw_log ? LOG_BOTNETOUT : 0;
       break;
     case 'h':
     case 'H':
-      res |= raw_log ? LOG_BOTSHARE : 0;
+      res |= raw_log ? LOG_BOTSHRIN : 0;
+      break;
+    case 'g':
+    case 'G':
+      res |= raw_log ? LOG_BOTSHROUT : 0;
       break;
     case '1':
       res |= LOG_LEV1;
@@ -130,7 +140,7 @@ int logmodes(char *s)
 
 char *masktype(int x)
 {
-  static char s[24];            /* Change this if you change the levels */
+  static char s[27];            /* Change this if you change the levels */
   char *p = s;
 
   if (x & LOG_MSGS)
@@ -147,6 +157,8 @@ char *masktype(int x)
     *p++ = 'o';
   if (x & LOG_BOTS)
     *p++ = 'b';
+  if (x & LOG_BOTMSG)
+    *p++ = 'l';
   if ((x & LOG_RAW) && raw_log)
     *p++ = 'r';
   if (x & LOG_FILES)
@@ -159,10 +171,14 @@ char *masktype(int x)
     *p++ = 'w';
   if ((x & LOG_SRVOUT) && raw_log)
     *p++ = 'v';
-  if ((x & LOG_BOTNET) && raw_log)
+  if ((x & LOG_BOTNETIN) && raw_log)
     *p++ = 't';
-  if ((x & LOG_BOTSHARE) && raw_log)
+  if ((x & LOG_BOTNETOUT) && raw_log)
+    *p++ = 'u';
+  if ((x & LOG_BOTSHRIN) && raw_log)
     *p++ = 'h';
+  if ((x & LOG_BOTSHROUT) && raw_log)
+    *p++ = 'g';
   if (x & LOG_LEV1)
     *p++ = '1';
   if (x & LOG_LEV2)
@@ -187,56 +203,62 @@ char *masktype(int x)
 
 char *maskname(int x)
 {
-  static char s[207];           /* Change this if you change the levels */
-  int i = 0;
+  static char s[275];           /* Change this if you change the levels */
+  int i = 0;			/* 6+8+7+13+6+6+6+17+5+7+8+7+9+15+17+17+24+24+(8*9)+1 */
 
   s[0] = 0;
   if (x & LOG_MSGS)
-    i += my_strcpy(s, "msgs, ");
+    i += my_strcpy(s, "msgs, "); /* 6 */
   if (x & LOG_PUBLIC)
-    i += my_strcpy(s + i, "public, ");
+    i += my_strcpy(s + i, "public, "); /* 8 */
   if (x & LOG_JOIN)
-    i += my_strcpy(s + i, "joins, ");
+    i += my_strcpy(s + i, "joins, "); /* 7 */
   if (x & LOG_MODES)
-    i += my_strcpy(s + i, "kicks/modes, ");
+    i += my_strcpy(s + i, "kicks/modes, "); /* 13 */
   if (x & LOG_CMDS)
-    i += my_strcpy(s + i, "cmds, ");
+    i += my_strcpy(s + i, "cmds, "); /* 6 */
   if (x & LOG_MISC)
-    i += my_strcpy(s + i, "misc, ");
+    i += my_strcpy(s + i, "misc, "); /* 6 */
   if (x & LOG_BOTS)
-    i += my_strcpy(s + i, "bots, ");
+    i += my_strcpy(s + i, "bots, "); /* 6 */
+  if (x & LOG_BOTMSG)
+    i += my_strcpy(s + i, "linked bot msgs, "); /* 17 */
   if ((x & LOG_RAW) && raw_log)
-    i += my_strcpy(s + i, "raw, ");
+    i += my_strcpy(s + i, "raw, "); /* 5 */
   if (x & LOG_FILES)
-    i += my_strcpy(s + i, "files, ");
+    i += my_strcpy(s + i, "files, "); /* 7 */
   if (x & LOG_SERV)
-    i += my_strcpy(s + i, "server, ");
+    i += my_strcpy(s + i, "server, "); /* 8 */
   if (x & LOG_DEBUG)
-    i += my_strcpy(s + i, "debug, ");
+    i += my_strcpy(s + i, "debug, "); /* 7 */
   if (x & LOG_WALL)
-    i += my_strcpy(s + i, "wallops, ");
+    i += my_strcpy(s + i, "wallops, "); /* 9 */
   if ((x & LOG_SRVOUT) && raw_log)
-    i += my_strcpy(s + i, "server output, ");
-  if ((x & LOG_BOTNET) && raw_log)
-    i += my_strcpy(s + i, "botnet traffic, ");
-  if ((x & LOG_BOTSHARE) && raw_log)
-    i += my_strcpy(s + i, "share traffic, ");
+    i += my_strcpy(s + i, "server output, "); /* 15 */
+  if ((x & LOG_BOTNETIN) && raw_log)
+    i += my_strcpy(s + i, "botnet incoming, "); /* 17 */
+  if ((x & LOG_BOTNETOUT) && raw_log)
+    i += my_strcpy(s + i, "botnet outgoing, "); /* 17 */
+  if ((x & LOG_BOTSHRIN) && raw_log)
+    i += my_strcpy(s + i, "incoming share traffic, "); /* 24 */
+  if ((x & LOG_BOTSHROUT) && raw_log)
+    i += my_strcpy(s + i, "outgoing share traffic, "); /* 24 */
   if (x & LOG_LEV1)
-    i += my_strcpy(s + i, "level 1, ");
+    i += my_strcpy(s + i, "level 1, "); /* 9 */
   if (x & LOG_LEV2)
-    i += my_strcpy(s + i, "level 2, ");
+    i += my_strcpy(s + i, "level 2, "); /* 9 */
   if (x & LOG_LEV3)
-    i += my_strcpy(s + i, "level 3, ");
+    i += my_strcpy(s + i, "level 3, "); /* 9 */
   if (x & LOG_LEV4)
-    i += my_strcpy(s + i, "level 4, ");
+    i += my_strcpy(s + i, "level 4, "); /* 9 */
   if (x & LOG_LEV5)
-    i += my_strcpy(s + i, "level 5, ");
+    i += my_strcpy(s + i, "level 5, "); /* 9 */
   if (x & LOG_LEV6)
-    i += my_strcpy(s + i, "level 6, ");
+    i += my_strcpy(s + i, "level 6, "); /* 9 */
   if (x & LOG_LEV7)
-    i += my_strcpy(s + i, "level 7, ");
+    i += my_strcpy(s + i, "level 7, "); /* 9 */
   if (x & LOG_LEV8)
-    i += my_strcpy(s + i, "level 8, ");
+    i += my_strcpy(s + i, "level 8, "); /* 9 */
   if (i)
     s[i - 2] = 0;
   else
@@ -466,7 +488,7 @@ static int bot2str(char *string, int bot)
 {
   char x = 'a', *old = string;
 
-  while (x < 'v') {
+  while (x <= 'z') {
     if (bot & 1)
       *string++ = x;
     x++;
@@ -515,7 +537,7 @@ int build_flags(char *string, struct flag_record *plus,
     string += flag2str(string, plus->chan, plus->udef_chan);
     if (minus && (minus->chan || minus->udef_chan)) {
       *string++ = '-';
-      string += flag2str(string, minus->global, minus->udef_chan);
+      string += flag2str(string, minus->chan, minus->udef_chan);
     }
   }
   if (string == old) {
