@@ -1,12 +1,10 @@
 /*
  * lang.h
  *   Conversion definitions for language support
- *
- * $Id: lang.h,v 1.40 2011/02/13 14:19:33 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -130,7 +128,6 @@
 #define IRC_BANNED              get_language(0x600)
 #define IRC_YOUREBANNED         get_language(0x601)
 #define IRC_IBANNEDME           get_language(0x602)
-#define IRC_FUNKICK             get_language(0x603)
 #define IRC_HI                  get_language(0x604)
 #define IRC_GOODBYE             get_language(0x605)
 #define IRC_BANNED2             get_language(0x606)
@@ -200,8 +197,8 @@
 #define IRC_MODEQUEUE           get_language(0x647)
 #define IRC_SERVERQUEUE         get_language(0x648)
 #define IRC_HELPQUEUE           get_language(0x649)
-/* was: IRC_BOTNOTONIRC         0x64a            */
-/* was: IRC_NOTACTIVECHAN       0x64b            */
+#define IRC_VHOSTWRONGNET	get_language(0x64a)
+#define IRC_VHOSTBADADDR	get_language(0x64b)
 #define IRC_PROCESSINGCHAN      get_language(0x64c)
 #define IRC_CHANNEL             get_language(0x64d)
 #define IRC_DESIRINGCHAN        get_language(0x64e)
@@ -465,7 +462,7 @@
 #define DCC_CLOSED              get_language(0xe16)
 #define DCC_FAILED              get_language(0xe17)
 #define DCC_BADSRC              get_language(0xe18)
-/* was: DCC_BADIP               0xe19            */
+#define DCC_RETURN              get_language(0xe19)
 #define DCC_BADHOST             get_language(0xe1a)
 #define DCC_TELCONN             get_language(0xe1b)
 #define DCC_IDENTFAIL           get_language(0xe1c)
