@@ -1,12 +1,10 @@
 /*
  * language.c -- handles:
  *   language support code
- *
- * $Id: language.c,v 1.31 2011/02/13 14:19:33 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -150,8 +148,7 @@ static int del_lang(char *lang)
         lpo->next = lp->next;
       else
         langpriority = lp->next;
-      if (lp->lang)
-        nfree(lp->lang);
+      nfree(lp->lang);
       nfree(lp);
       debug1("LANG: Language unloaded: %s", lang);
       return 1;
@@ -241,10 +238,13 @@ static void read_lang(char *langfile)
         lskip = 1;
       }
       if (lskip) {
-        while (!strchr(lbuf, '\n')) {
-          fgets(lbuf, 511, FLANG);
+        while (!strchr(lbuf, '\n') && fgets(lbuf, 511, FLANG) != NULL) {
           lline++;
         }
+        /* fgets == NULL means error or empty file, so check for error */
+        if (ferror(FLANG)) {
+          putlog(LOG_MISC, "*", "LANG: Error reading lang file.");
+        }
         lline++;
         lnew = 1;
         continue;
@@ -535,7 +535,7 @@ static int cmd_languagedump(struct userrec *u, int idx, char *par)
       sscanf(par, "%x", &idx2);
     else
       idx2 = (int) strtol(par, (char **) NULL, 10);
-    strcpy(ltext2, get_language(idx2));
+    strncpyz(ltext2, get_language(idx2), sizeof ltext2);
     dprintf(idx, "0x%x: %s\n", idx2, ltext2);
     return 0;
   }
