@@ -4,12 +4,10 @@
  *   signal handling
  *   command line arguments
  *   context and assert debugging
- *
- * $Id: main.c,v 1.141 2011/09/23 20:31:32 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,6 +29,22 @@
  * list available at eggheads@eggheads.org.
  */
 
+/* We need config.h for CYGWIN_HACKS, but windows.h must be included before
+ * eggdrop headers, because the malloc/free/Context macros break the inclusion.
+ * The SSL undefs are a workaround for bug #2182 in openssl with msys/mingw.
+ */
+#include <config.h>
+#ifdef CYGWIN_HACKS
+#  include <windows.h>
+#  undef X509_NAME
+#  undef X509_EXTENSIONS
+#  undef X509_CERT_PAIR
+#  undef PKCS7_ISSUER_AND_SERIAL
+#  undef PKCS7_SIGNER_INFO
+#  undef OCSP_REQUEST
+#  undef OCSP_RESPONSE
+#endif
+
 #include "main.h"
 
 #include <fcntl.h>
@@ -55,6 +69,7 @@
 #  define UAC_NOPRINT 0x00000001        /* Don't report unaligned fixups */
 #endif
 
+#include "version.h"
 #include "chan.h"
 #include "modules.h"
 #include "tandem.h"
@@ -64,10 +79,6 @@
 #  include <sys/resource.h>             /* setrlimit() */
 #endif
 
-#ifdef CYGWIN_HACKS
-#  include <windows.h>
-#endif
-
 #ifndef _POSIX_SOURCE
 #  define _POSIX_SOURCE 1               /* Solaris needs this */
 #endif
@@ -81,9 +92,6 @@ extern struct chanset_t *chanset;
 extern log_t *logs;
 extern Tcl_Interp *interp;
 extern tcl_timer_t *timer, *utimer;
-#ifdef REPLACE_NOTIFIER
-extern tclevent_t *tclevents;
-#endif
 extern sigjmp_buf alarmret;
 time_t now;
 static int argc;
@@ -96,8 +104,11 @@ static char **argv;
  * modified versions of this bot.
  */
 
-char egg_version[1024] = "1.6.21";
-int egg_numver = 1062100;
+char egg_version[1024] = EGG_STRINGVER;
+int egg_numver = EGG_NUMVER;
+#ifdef EGG_PATCH
+char egg_patch[] = EGG_PATCH;
+#endif
 
 char notify_new[121] = "";      /* Person to send a note to for new users */
 int default_flags = 0;          /* Default user flags                     */
@@ -109,29 +120,24 @@ int term_z = 0;         /* Foreground: use the terminal as a partyline?  */
 int use_stderr = 1;     /* Send stuff to stderr instead of logfiles?     */
 
 char configfile[121] = "eggdrop.conf";  /* Default config file name */
-char pid_file[120];                     /* Name of the pid file     */
+char pid_file[121];                     /* Name of the pid file     */
 char helpdir[121] = "help/";            /* Directory of help files  */
 char textdir[121] = "text/";            /* Directory for text files */
 
 int keep_all_logs = 0;                  /* Never erase logfiles?    */
-char logfile_suffix[21] = ".%d%b%Y";    /* Format of logfile suffix */
 int switch_logfiles_at = 300;           /* When to switch logfiles  */
 
 time_t online_since;    /* time that the bot was started */
 
 int make_userfile = 0; /* Using bot in userfile-creation mode? */
-char owner[121] = "";  /* Permanent owner(s) of the bot        */
 
 int save_users_at = 0;   /* Minutes past the hour to save the userfile?     */
 int notify_users_at = 0; /* Minutes past the hour to notify users of notes? */
 
 char version[81];    /* Version info (long form)  */
 char ver[41];        /* Version info (short form) */
-char egg_xtra[2048]; /* Patch info                */
 
 int do_restart = 0;       /* .restart has been called, restart ASAP */
-int die_on_sighup = 0;    /* Die if bot receives SIGHUP             */
-int die_on_sigterm = 1;   /* Die if bot receives SIGTERM            */
 int resolve_timeout = 15; /* Hostname/address lookup timeout        */
 char quit_msg[1024];      /* Quit message                           */
 
@@ -167,6 +173,9 @@ int cx_line[16];
 int cx_ptr = 0;
 #endif
 
+#ifdef TLS
+int ssl_cleanup();
+#endif
 
 void fatal(const char *s, int recoverable)
 {
@@ -177,10 +186,13 @@ void fatal(const char *s, int recoverable)
   for (i = 0; i < dcc_total; i++)
     if (dcc[i].sock >= 0)
       killsock(dcc[i].sock);
+#ifdef TLS
+  ssl_cleanup();
+#endif
   unlink(pid_file);
-  if (!recoverable) {
+  if (recoverable != 1) {
     bg_send_quit(BG_ABORT);
-    exit(1);
+    exit(!recoverable);
   }
 }
 
@@ -196,6 +208,10 @@ int expmem_modules(int);
 int expmem_language();
 int expmem_tcldcc();
 int expmem_tclmisc();
+int expmem_dns();
+#ifdef TLS
+int expmem_tls();
+#endif
 
 /* For mem.c : calculate memory we SHOULD be using
  */
@@ -206,7 +222,10 @@ int expected_memory(void)
   tot = expmem_chanprog() + expmem_users() + expmem_misc() + expmem_dccutil() +
         expmem_botnet() + expmem_tcl() + expmem_tclhash() + expmem_net() +
         expmem_modules(0) + expmem_language() + expmem_tcldcc() +
-        expmem_tclmisc();
+        expmem_tclmisc() + expmem_dns();
+#ifdef TLS
+  tot += expmem_tls();
+#endif
   return tot;
 }
 
@@ -247,13 +266,17 @@ static void write_debug()
      *       _not_ safe <cybah>
      */
     x = creat("DEBUG.DEBUG", 0644);
-    setsock(x, SOCK_NONSOCK);
     if (x >= 0) {
+      setsock(x, SOCK_NONSOCK);
       strncpyz(s, ctime(&now), sizeof s);
       dprintf(-x, "Debug (%s) written %s\n", ver, s);
       dprintf(-x, "Please report problem to bugs@eggheads.org\n");
       dprintf(-x, "after a visit to http://www.eggheads.org/bugzilla/\n");
-      dprintf(-x, "Full Patch List: %s\n", egg_xtra);
+#ifdef EGG_PATCH
+      dprintf(-x, "Patch level: %s\n", egg_patch);
+#else
+      dprintf(-x, "Patch level: %s\n", "stable");
+#endif
       dprintf(-x, "Context: ");
       cx_ptr = cx_ptr & 15;
       for (y = ((cx_ptr + 1) & 15); y != cx_ptr; y = ((y + 1) & 15))
@@ -278,7 +301,11 @@ static void write_debug()
   } else {
     strncpyz(s, ctime(&now), sizeof s);
     dprintf(-x, "Debug (%s) written %s\n", ver, s);
-    dprintf(-x, "Full Patch List: %s\n", egg_xtra);
+#ifdef EGG_PATCH
+    dprintf(-x, "Patch level: %s\n", egg_patch);
+#else
+    dprintf(-x, "Patch level: %s\n", "stable");
+#endif
 #ifdef STATIC
     dprintf(-x, "STATICALLY LINKED\n");
 #endif
@@ -292,14 +319,27 @@ static void write_debug()
     dprintf(-x, "Tcl version: %s (header version %s)\n",
             ((interp) && (Tcl_Eval(interp, "info patchlevel") == TCL_OK)) ?
             tcl_resultstring() : (Tcl_Eval(interp, "info tclversion") == TCL_OK) ?
-            tcl_resultstring() : "*unknown*", TCL_PATCH_LEVEL ? TCL_PATCH_LEVEL :
-            "*unknown*");
+            tcl_resultstring() : "*unknown*", TCL_PATCH_LEVEL);
 
     if (tcl_threaded())
       dprintf(-x, "Tcl is threaded\n");
+#ifdef IPV6
+    dprintf(-x, "Compiled with IPv6 support\n");
+#else
+    dprintf(-x, "Compiled without IPv6 support\n");
+#endif
 
-    dprintf(-x, "Configure flags: %s\n", EGG_AC_ARGS);
+#ifdef TLS
+    dprintf(-x, "Compiled with TLS support\n");
+#else
+    dprintf(-x, "Compiled without TLS support\n");
+#endif
 
+    if (!strcmp(EGG_AC_ARGS, "")) {
+      dprintf(-x, "Configure flags: none\n");
+    } else {
+      dprintf(-x, "Configure flags: %s\n", EGG_AC_ARGS);
+    }
 #ifdef CCFLAGS
     dprintf(-x, "Compile flags: %s\n", CCFLAGS);
 #endif
@@ -361,30 +401,29 @@ static void got_fpe(int z)
 
 static void got_term(int z)
 {
-  write_userfile(-1);
-  check_tcl_event("sigterm");
-  if (die_on_sigterm) {
-    botnet_send_chat(-1, botnetnick, "ACK, I've been terminated!");
-    fatal("TERMINATE SIGNAL -- SIGNING OFF", 0);
-  } else
-    putlog(LOG_MISC, "*", "RECEIVED TERMINATE SIGNAL (IGNORING)");
+  /* Now we die by default on sigterm, but scripts have the chance to
+   * catch the event themselves and cancel shutdown by returning 1
+   */
+  if (check_tcl_signal("sigterm"))
+    return;
+  kill_bot("ACK, I've been terminated!", "TERMINATE SIGNAL -- SIGNING OFF");
 }
 
 static void got_quit(int z)
 {
-  check_tcl_event("sigquit");
-  putlog(LOG_MISC, "*", "RECEIVED QUIT SIGNAL (IGNORING)");
+  if (check_tcl_signal("sigquit"))
+    return;
+  putlog(LOG_MISC, "*", "Received QUIT signal: restarting...");
+  do_restart = -1;
   return;
 }
 
 static void got_hup(int z)
 {
   write_userfile(-1);
-  check_tcl_event("sighup");
-  if (die_on_sighup) {
-    fatal("HANGUP SIGNAL -- SIGNING OFF", 0);
-  } else
-    putlog(LOG_MISC, "*", "Received HUP signal: rehashing...");
+  if (check_tcl_signal("sighup"))
+    return;
+  putlog(LOG_MISC, "*", "Received HUP signal: rehashing...");
   do_restart = -2;
   return;
 }
@@ -402,7 +441,7 @@ static void got_alarm(int z)
  */
 static void got_ill(int z)
 {
-  check_tcl_event("sigill");
+  check_tcl_signal("sigill");
 #ifdef DEBUG_CONTEXT
   putlog(LOG_MISC, "*", "* Context: %s/%d [%s]", cx_file[cx_ptr],
          cx_line[cx_ptr], (cx_note[cx_ptr][0]) ? cx_note[cx_ptr] : "");
@@ -460,50 +499,107 @@ void eggAssert(const char *file, int line, const char *module)
 }
 #endif
 
-static void do_arg(char *s)
-{
+void show_ver() {
   char x[512], *z = x;
-  int i;
 
-  if (s[0] == '-')
-    for (i = 1; i < strlen(s); i++) {
-      switch (s[i]) {
+  strncpyz(x, egg_version, sizeof x);
+  newsplit(&z);
+  newsplit(&z);
+  printf("%s\n", version);
+  if (z[0]) {
+    printf("  (patches: %s)\n", z);
+  }
+  if (!strcmp(EGG_AC_ARGS, "")) {
+    printf("Configure flags: none\n");
+  } else {
+    printf("Configure flags: %s\n", EGG_AC_ARGS);
+  }
+  printf("Compiled with: ");
+#ifdef IPV6
+  printf("IPv6, ");
+#endif
+#ifdef TLS
+  printf("TLS, ");
+#endif
+  printf("handlen=%d\n", HANDLEN);
+  bg_send_quit(BG_ABORT);
+}
+
+/* Hard coded text because config file isn't loaded yet,
+   meaning other languages can't be loaded yet.
+   English (or an error) is the only possible option.
+*/
+void show_help() {
+  printf("\n%s\n\n", version);
+  printf("Usage: eggdrop [options] [config-file]\n\n"
+         "Options:\n"
+         "-n Don't background; send all log entries to console.\n"
+         "-nc  Don't background; display channel stats every 10 seconds.\n"
+         "-nt  Don't background; use terminal to simulate DCC chat.\n"
+         "-m   Create userfile.\n"
+         "-h   Show this help.\n"
+         "-v   Show version info, then quit.\n\n");
+  bg_send_quit(BG_ABORT);
+}
+
+static void do_arg()
+{
+  int option = 0;
+/* Bitmask structure to hold cli flags
+   | QUIT| BAD FLAG| h| n| c| t| m| v|
+   |  128|       64|32|16| 8| 4| 2| 1|
+*/
+  unsigned char cliflags = 0;
+
+  while ((option = getopt(argc, argv, "hnctmv")) != -1) {
+    switch (option) {
       case 'n':
+        cliflags |= 16;
         backgrd = 0;
         break;
       case 'c':
+        cliflags |= 8;
         con_chan = 1;
         term_z = 0;
         break;
       case 't':
+        cliflags |= 4;
         con_chan = 0;
         term_z = 1;
         break;
       case 'm':
+        cliflags |= 2;
         make_userfile = 1;
         break;
       case 'v':
-        strncpyz(x, egg_version, sizeof x);
-        newsplit(&z);
-        newsplit(&z);
-        printf("%s\n", version);
-        if (z[0])
-          printf("  (patches: %s)\n", z);
-        printf("Configured with: " EGG_AC_ARGS "\n");
-        printf("handlen=%d\n", HANDLEN);
-        bg_send_quit(BG_ABORT);
-        exit(0);
-        break;                  /* this should never be reached */
+        cliflags |= 129;		//128 + 1
+        break;
       case 'h':
-        printf("\n%s\n\n", version);
-        printf(EGG_USAGE);
-        printf("\n");
-        bg_send_quit(BG_ABORT);
-        exit(0);
-        break;                  /* this should never be reached */
-      }
-    } else
-    strncpyz(configfile, s, sizeof configfile);
+        cliflags |= 160;		//128 + 32
+        break;
+      default:
+        cliflags |= 192;		//128 + 64
+        break;
+    }
+  }
+  if ((cliflags & 64) || (cliflags & 32)) {
+    show_help();
+    exit(0);
+  } else if (cliflags & 1) {
+    show_ver();
+    exit(0);
+  } else if (!(cliflags & 16) && ((cliflags & 8) || (cliflags & 4))) {
+    printf("\n%s\n", version);
+    printf("ERROR: The -n flag is required when using the -c or -t flags. Exiting...\n\n");
+    exit(1);
+  } else if (argc > (optind + 1)) {
+    printf("\n");
+    printf("WARNING: More than one config file value detected\n");
+    printf("         Using %s as config file\n", argv[optind]);
+  }
+  if (argc > optind) {
+    strncpyz(configfile, argv[optind], sizeof configfile);
+  }
 }
 
 void backup_userfile(void)
@@ -517,7 +613,7 @@ void backup_userfile(void)
 }
 
 /* Timer info */
-static int lastmin = 99;
+static time_t lastmin;
 static time_t then;
 static struct tm nowtm;
 
@@ -529,6 +625,7 @@ static void core_secondly()
 {
   static int cnt = 0;
   int miltime;
+  time_t nowmins;
 
   do_check_timers(&utimer);     /* Secondly timers */
   cnt++;
@@ -544,20 +641,21 @@ static void core_secondly()
     }
   }
   egg_memcpy(&nowtm, localtime(&now), sizeof(struct tm));
-  if (nowtm.tm_min != lastmin) {
+  nowmins = time(NULL) / 60;
+  if (nowmins > lastmin) {
     int i = 0;
 
     /* Once a minute */
-    lastmin = (lastmin + 1) % 60;
+    ++lastmin;
     call_hook(HOOK_MINUTELY);
     check_expired_ignores();
     autolink_cycle(NULL);       /* Attempt autolinks */
     /* In case for some reason more than 1 min has passed: */
-    while (nowtm.tm_min != lastmin) {
+    while (nowmins != lastmin) {
       /* Timer drift, dammit */
-      debug2("timer: drift (lastmin=%d, now=%d)", lastmin, nowtm.tm_min);
+      debug2("timer: drift (lastmin=%d, nowmins=%d)", lastmin, nowmins);
       i++;
-      lastmin = (lastmin + 1) % 60;
+      ++lastmin;
       call_hook(HOOK_MINUTELY);
     }
     if (i > 1)
@@ -690,17 +788,9 @@ int init_bots();
 int init_modules();
 int init_tcl(int, char **);
 int init_language(int);
-
-static void patch(const char *str)
-{
-  char *p = strchr(egg_version, '+');
-
-  if (!p)
-    p = &egg_version[strlen(egg_version)];
-  sprintf(p, "+%s", str);
-  egg_numver++;
-  sprintf(&egg_xtra[strlen(egg_xtra)], " %s", str);
-}
+#ifdef TLS
+int ssl_init();
+#endif
 
 static inline void garbage_collect(void)
 {
@@ -715,7 +805,7 @@ static inline void garbage_collect(void)
 int mainloop(int toplevel)
 {
   static int socket_cleanup = 0;
-  int xx, i, eggbusy = 1, tclbusy = 0, old_do_restart;
+  int xx, i, eggbusy = 1, tclbusy = 0;
   char buf[520];
 
   /* Lets move some of this here, reducing the numer of actual
@@ -757,9 +847,8 @@ int mainloop(int toplevel)
   } else
     socket_cleanup--;
 
-  /* Free deleted binds, only if not recursing. */
-  if (toplevel)
-    garbage_collect();
+  /* Free unused structures. */
+  garbage_collect();
 
   xx = sockgets(buf, &i);
   if (xx >= 0) {              /* Non-error */
@@ -780,6 +869,8 @@ int mainloop(int toplevel)
               itraffic_dcc_today += strlen(buf) + 1;
             else if (!strcmp(dcc[idx].type->name, "SEND"))
               itraffic_trans_today += strlen(buf) + 1;
+            else if (!strcmp(dcc[idx].type->name, "FORK_SEND"))
+              itraffic_trans_today += strlen(buf) + 1;
             else if (!strncmp(dcc[idx].type->name, "GET", 3))
               itraffic_trans_today += strlen(buf) + 1;
             else
@@ -837,12 +928,8 @@ int mainloop(int toplevel)
     tclbusy = 1;
   }
 
-/* restart/rehash is performed here, reset do_restart ASAP to prevent
- * doing it again if rehash() recurses in Tcl_Eval() */
   if (do_restart) {
-    old_do_restart = do_restart;
-    do_restart = 0;
-    if (old_do_restart == -2)
+    if (do_restart == -2)
       rehash();
     else if (!toplevel)
       return -1; /* Unwind to toplevel before restarting */
@@ -868,7 +955,7 @@ int mainloop(int toplevel)
             d = d->next;
           }
           if (ok) {
-            strcpy(name, p->name);
+            strncpyz(name, p->name, sizeof name);
             if (module_unload(name, botnetnick) == NULL) {
               f = 1;
               break;
@@ -904,33 +991,26 @@ int mainloop(int toplevel)
       }
 
       rehash();
+#ifdef TLS
+      ssl_cleanup();
+      ssl_init();
+#endif
       restart_chons();
       call_hook(HOOK_LOADED);
     }
     eggbusy = 1;
+    do_restart = 0;
   }
 
-#ifdef USE_TCL_EVENTS
   if (!eggbusy) {
 /* Process all pending tcl events */
-
 #  ifdef REPLACE_NOTIFIER
-    tclevent_t *e;
     if (Tcl_ServiceAll())
       tclbusy = 1;
-    while (tclevents) {
-      e = tclevents;
-      tclevents = tclevents->next;
-      tclbusy = 1;
-      do_tcl_sync(e->context, e->script, e->callback, 1);
-      nfree(e);
-    }
 #  else
     while (Tcl_DoOneEvent(TCL_DONT_WAIT | TCL_ALL_EVENTS))
       tclbusy = 1;
 #  endif /* REPLACE_NOTIFIER */
-
-#endif   /* USE_TCL_EVENTS   */
   }
 
   return (eggbusy || tclbusy);
@@ -969,20 +1049,20 @@ int main(int arg_c, char **arg_v)
     Context;
 #endif
 
-/* Include patch.h header for patch("...") */
-#include "patch.h"
-
   argc = arg_c;
   argv = arg_v;
 
   /* Version info! */
+#ifdef EGG_PATCH
+  egg_snprintf(&egg_version[strlen(egg_version)], sizeof egg_version, 
+               "+%s", egg_patch);
+#endif
   egg_snprintf(ver, sizeof ver, "eggdrop v%s", egg_version);
   egg_snprintf(version, sizeof version,
-               "Eggdrop v%s (C) 1997 Robey Pointer (C) 2011 Eggheads",
+               "Eggdrop v%s (C) 1997 Robey Pointer (C) 2010-2018 Eggheads",
                egg_version);
   /* Now add on the patchlevel (for Tcl) */
   sprintf(&egg_version[strlen(egg_version)], " %u", egg_numver);
-  strcat(egg_version, egg_xtra);
 
 /* For OSF/1 */
 #ifdef STOP_UAC
@@ -1025,13 +1105,12 @@ int main(int arg_c, char **arg_v)
   now = time(NULL);
   chanset = NULL;
   egg_memcpy(&nowtm, localtime(&now), sizeof(struct tm));
-  lastmin = nowtm.tm_min;
+  lastmin = now / 60;
   srandom((unsigned int) (now % (getpid() + getppid())));
   init_mem();
-  init_language(1);
   if (argc > 1)
-    for (i = 1; i < argc; i++)
-      do_arg(argv[i]);
+    do_arg();
+  init_language(1);
 
   printf("\n%s\n", version);
 
@@ -1059,11 +1138,11 @@ int main(int arg_c, char **arg_v)
   link_statics();
 #endif
   strncpyz(s, ctime(&now), sizeof s);
-  strcpy(&s[11], &s[20]);
+  memmove(&s[11], &s[20], strlen(&s[20])+1);
   putlog(LOG_ALL, "*", "--- Loading %s (%s)", ver, s);
   chanprog();
   if (!encrypt_pass) {
-    printf(MOD_NOCRYPT);
+    printf("%s", MOD_NOCRYPT);
     bg_send_quit(BG_ABORT);
     exit(1);
   }
@@ -1072,6 +1151,9 @@ int main(int arg_c, char **arg_v)
     i++;
   putlog(LOG_MISC, "*", "=== %s: %d channels, %d users.",
          botnetnick, i, count_users(userlist));
+#ifdef TLS
+  ssl_init();
+#endif
   cache_miss = 0;
   cache_hit = 0;
   if (!pid_file[0])
@@ -1080,16 +1162,20 @@ int main(int arg_c, char **arg_v)
   /* Check for pre-existing eggdrop! */
   f = fopen(pid_file, "r");
   if (f != NULL) {
-    fgets(s, 10, f);
-    xx = atoi(s);
-    i = kill(xx, SIGCHLD);      /* Meaningless kill to determine if pid
-                                 * is used */
-    if (i == 0 || errno != ESRCH) {
-      printf(EGG_RUNNING1, botnetnick);
-      printf(EGG_RUNNING2, pid_file);
-      bg_send_quit(BG_ABORT);
-      exit(1);
+    if (fgets(s, 10, f) != NULL) {
+      xx = atoi(s);
+      i = kill(xx, SIGCHLD);      /* Meaningless kill to determine if pid
+                                   * is used */
+      if (i == 0 || errno != ESRCH) {
+        printf(EGG_RUNNING1, botnetnick);
+        printf(EGG_RUNNING2, pid_file);
+        bg_send_quit(BG_ABORT);
+        exit(1);
+      }
+    } else {
+      printf("Error checking for existing Eggdrop process.\n");
     }
+    fclose(f);
   }
 
   /* Move into background? */
@@ -1124,9 +1210,15 @@ int main(int arg_c, char **arg_v)
     setpgid(0, 0);
 #endif
     /* Tcl wants the stdin, stdout and stderr file handles kept open. */
-    freopen("/dev/null", "r", stdin);
-    freopen("/dev/null", "w", stdout);
-    freopen("/dev/null", "w", stderr);
+    if (freopen("/dev/null", "r", stdin) == NULL) {
+      putlog(LOG_MISC, "*", "Error renaming stdin file handle: %s", strerror(errno));
+    }
+    if (freopen("/dev/null", "w", stdout) == NULL) {
+      putlog(LOG_MISC, "*", "Error renaming stdout file handle: %s", strerror(errno));
+    }
+    if (freopen("/dev/null", "w", stderr) == NULL) {
+      putlog(LOG_MISC, "*", "Error renaming stderr file handle: %s", strerror(errno));
+    }
 #ifdef CYGWIN_HACKS
     FreeConsole();
 #endif
@@ -1134,26 +1226,26 @@ int main(int arg_c, char **arg_v)
 
   /* Terminal emulating dcc chat */
   if (!backgrd && term_z) {
-    int n = new_dcc(&DCC_CHAT, sizeof(struct chat_info));
+    /* reuse term_z as glob var to pass it's index in the dcc table around */
+    term_z = new_dcc(&DCC_CHAT, sizeof(struct chat_info));
 
-    dcc[n].addr = iptolong(getmyip());
-    dcc[n].sock = STDOUT;
-    dcc[n].timeval = now;
-    dcc[n].u.chat->con_flags = conmask;
-    dcc[n].u.chat->strip_flags = STRIP_ALL;
-    dcc[n].status = STAT_ECHO;
-    strcpy(dcc[n].nick, "HQ");
-    strcpy(dcc[n].host, "llama@console");
-    /* HACK: Workaround not to pass literal "HQ" as a non-const arg */
-    dcc[n].user = get_user_by_handle(userlist, dcc[n].nick);
-    /* Make sure there's an innocuous HQ user if needed */
-    if (!dcc[n].user) {
-      userlist = adduser(userlist, dcc[n].nick, "none", "-", USER_PARTY);
-      dcc[n].user = get_user_by_handle(userlist, dcc[n].nick);
-    }
+    /* new_dcc returns -1 on error, and 0 should always be taken by the listening socket */
+    if (term_z < 1)
+      fatal("ERROR: Failed to initialize foreground chat.", 0);
+
+    getvhost(&dcc[term_z].sockname, AF_INET);
+    dcc[term_z].sock = STDOUT;
+    dcc[term_z].timeval = now;
+    dcc[term_z].u.chat->con_flags = conmask | EGG_BG_CONMASK;
+    dcc[term_z].u.chat->strip_flags = STRIP_ALL;
+    dcc[term_z].status = STAT_ECHO;
+    strcpy(dcc[term_z].nick, EGG_BG_HANDLE);
+    strcpy(dcc[term_z].host, "llama@console");
+    add_hq_user();
     setsock(STDOUT, 0);          /* Entry in net table */
-    dprintf(n, "\n### ENTERING DCC CHAT SIMULATION ###\n\n");
-    dcc_chatter(n);
+    dprintf(term_z, "\n### ENTERING DCC CHAT SIMULATION ###\n");
+    dprintf(term_z, "You can use the .su command to log into your Eggdrop account.\n\n");
+    dcc_chatter(term_z);
   }
 
   then = now;
