@@ -1,12 +1,10 @@
 /*
  * main.h
  *   include file to include most other include files
- *
- * $Id: main.h,v 1.43 2011/07/09 15:07:48 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,25 +28,14 @@
 #  include "config.h"
 #endif
 
+#include "eggint.h"
 #include "lush.h"
 
-#if ((TCL_MAJOR_VERSION > 7) || ((TCL_MAJOR_VERSION == 7) && (TCL_MINOR_VERSION >= 5)))
-#  define USE_TCL_EVENTS
-#  define USE_TCL_FINDEXEC
-#  define USE_TCL_PACKAGE
-#  define USE_TCL_VARARGS
+#ifndef TCL_PATCH_LEVEL
+#  define TCL_PATCH_LEVEL "*unknown*"
 #endif
 
-#if (TCL_MAJOR_VERSION >= 8)
-#  define USE_TCL_OBJ
-#endif
-
-#if (((TCL_MAJOR_VERSION == 8) && (TCL_MINOR_VERSION >= 1)) || (TCL_MAJOR_VERSION > 8))
-#  define USE_TCL_BYTE_ARRAYS
-#  define USE_TCL_ENCODING
-#endif
-
-#if defined(HAVE_TCL_SETNOTIFIER) && defined(HAVE_TCL_GETTHREADDATA) && defined(HAVE_TCL_NOTIFIER_INIT)
+#if defined(HAVE_TCL_NOTIFIER_INIT)
 #  define REPLACE_NOTIFIER
 #endif
 
@@ -69,7 +56,7 @@
 #endif
 
 /* UGH! Why couldn't Tcl pick a standard? */
-#if defined(USE_TCL_VARARGS) && (defined(__STDC__) || defined(HAS_STDARG))
+#if defined(__STDC__) || defined(HAS_STDARG)
 #  ifdef HAVE_STDARG_H
 #    include <stdarg.h>
 #  endif
@@ -114,16 +101,6 @@
 #include "users.h"
 #include "compat/compat.h"
 
-/* For pre Tcl7.5p1 versions */
-#ifndef HAVE_TCL_FREE
-#  define Tcl_Free(x) n_free(x, "", 0)
-#endif
-
-/* For pre7.6 Tcl versions */
-#ifndef TCL_PATCH_LEVEL
-#  define TCL_PATCH_LEVEL Tcl_GetVar(interp, "tcl_patchLevel", TCL_GLOBAL_ONLY)
-#endif
-
 #ifndef MAKING_MODS
 extern struct dcc_table DCC_CHAT, DCC_BOT, DCC_LOST, DCC_SCRIPT, DCC_BOT_NEW,
                         DCC_RELAY, DCC_RELAYING, DCC_FORK_RELAY, DCC_PRE_RELAY,
@@ -133,6 +110,18 @@ extern struct dcc_table DCC_CHAT, DCC_BOT, DCC_LOST, DCC_SCRIPT, DCC_BOT_NEW,
 #endif
 
 #define iptolong(a) (0xffffffff & (long) (htonl((unsigned long) a)))
+
+#ifdef IPV6
+# define setsnport(s, p) do {                                           \
+  if ((s).family == AF_INET6)                                           \
+    (s).addr.s6.sin6_port = htons((p));                                 \
+  else                                                                  \
+    (s).addr.s4.sin_port = htons((p));                                  \
+} while (0)
+#else
+# define setsnport(s, p) (s.addr.s4.sin_port = htons(p))
+#endif
+
 #define fixcolon(x) do {                                                \
         if ((x)[0] == ':')                                              \
           (x)++;                                                        \
@@ -152,4 +141,15 @@ extern struct dcc_table DCC_CHAT, DCC_BOT, DCC_LOST, DCC_SCRIPT, DCC_BOT_NEW,
 #  define O_NONBLOCK 00000004 /* POSIX non-blocking I/O */
 #endif /* BORGCUBES */
 
+/* Handle for the user that's used when starting eggdrop with -tn */
+#define EGG_BG_HANDLE "-HQ"
+/* Default recommended flags for this user, use | as splitter */
+#define EGG_BG_CONMASK LOG_MISC /* "o" */
+
+/* Stringify macros */
+#define EGG_MACRO_STR(x) EGG_STR(x)
+#define EGG_STR(x) #x
+
+#define EGG_AC_ARGS EGG_MACRO_STR(EGG_AC_ARGS_RAW)
+
 #endif /* _EGG_MAIN_H */
