@@ -4,8 +4,6 @@
  *   hostmask matching
  *   cidr matching
  *
- * $Id: match.c,v 1.18 2010/07/01 16:10:49 thommey Exp $
- *
  * Once this code was working, I added support for % so that I could
  * use the same code both in Eggdrop and in my IrcII client.
  * Pleased with this, I added the option of a fourth wildcard, ~,
@@ -235,6 +233,7 @@ int addr_match(char *m, char *n, int user, int cmp)
 {
   char *p, *q, *r = 0, *s = 0;
   char mu[UHOSTLEN], nu[UHOSTLEN];
+  int tmpscore, score = 0;
 
   /* copy the strings into our own buffers
      and convert to rfc uppercase */
@@ -250,18 +249,26 @@ int addr_match(char *m, char *n, int user, int cmp)
   }
   *p = *q = 0;
   if ((!user && !cidr_support) || !r || !s)
-    return wild_match(mu, nu) ? 1 : NOMATCH;
+    return wild_match(mu, nu);
 
   *r++ = *s++ = 0;
-  if (!wild_match(mu, nu))
+  tmpscore = wild_match(mu, nu);
+  if (!(tmpscore = wild_match(mu, nu)))
     return NOMATCH; /* nick!ident parts don't match */
+  score += tmpscore;
   if (!*r && !*s)
-    return 1; /* end of nonempty strings */
+    return score; /* end of nonempty strings */
 
   /* check for CIDR notation and perform
      generic string matching if not found */
-  if (!(p = strrchr(r, '/')) || !str_isdigit(p + 1))
-    return wild_match(r, s) ? 1 : NOMATCH;
+  if (!(p = strrchr(r, '/')) || !str_isdigit(p + 1)) {
+    tmpscore = wild_match(r, s);
+    if (!tmpscore)
+      return NOMATCH;
+    score += tmpscore;
+    return score;
+  }
+
   /* if the two strings are both cidr masks,
      use the broader prefix */
   if (cmp && (q = strrchr(s, '/')) && str_isdigit(q + 1)) {
@@ -271,7 +278,10 @@ int addr_match(char *m, char *n, int user, int cmp)
   }
   *p = 0;
   /* looks like a cidr mask */
-  return cidr_match(r, s, atoi(p + 1));
+  if (!(tmpscore = cidr_match(r, s, atoi(p + 1))))
+    return NOMATCH;
+  score += tmpscore;
+  return score;
 }
 
 /* Checks for overlapping masks
@@ -328,11 +338,9 @@ int mask_match(char *m, char *n)
 int cidr_match(char *m, char *n, int count)
 {
 #ifdef IPV6
-  int c, af = AF_INET;
+  int c, af = AF_INET, rest;
   u_8bit_t block[16], addr[16];
 
-  if (count < 1)
-    return NOMATCH;
   if (strchr(m, ':') || strchr(n, ':')) {
     af = AF_INET6;
     if (count > 128)
@@ -342,32 +350,38 @@ int cidr_match(char *m, char *n, int count)
   if (inet_pton(af, m, &block) != 1 ||
       inet_pton(af, n, &addr) != 1)
     return NOMATCH;
+  if (count < 1)
+    return 1;
   for (c = 0; c < (count / 8); c++)
     if (block[c] != addr[c])
       return NOMATCH;
   if (!(count % 8))
-    return 1;
-  count = 8 - (count % 8);
-  return ((block[c] >> count) == (addr[c] >> count));
+    return count;
+  rest = 8 - (count % 8);
+  /* Normalize returned score to max. 32 for a /128 ipv6, 32 for ipv4 */
+  return ((block[c] >> rest) == (addr[c] >> rest)) ? (count / (af == AF_INET6 ? 4 : 1)): 0;
 
 #else
+  int rest;
   IP block, addr;
 
-  if (count < 1 || count > 32)
+  if (count > 32)
     return NOMATCH;
   block = ntohl(inet_addr(m));
   addr = ntohl(inet_addr(n));
   if (block == INADDR_NONE || addr == INADDR_NONE)
     return NOMATCH;
-  count = 32 - count;
-  return ((block >> count) == (addr >> count));
+  if (count < 1)
+    return 1;
+  rest = 32 - count;
+  return ((block >> rest) == (addr >> rest)) ? count : 0;
 #endif
 }
 
 /* Inline for cron_match (obviously).
  * Matches a single field of a crontab expression.
  */
-inline int cron_matchfld(char *mask, int match)
+static inline int cron_matchfld(char *mask, int match)
 {
   int skip = 0, f, t;
   char *p, *q;
