@@ -20,6 +20,7 @@
 #include "md5.h"
 #include "compat/compat.h"
 
+#ifndef HAVE_OPENSSL_MD5
 /*
  * The basic MD5 functions.
  *
@@ -267,5 +268,6 @@ void MD5_Final(unsigned char *result, MD5_CTX *ctx)
 	result[14] = ctx->d >> 16;
 	result[15] = ctx->d >> 24;
 
-	egg_memset(ctx, 0, sizeof(ctx));
+	egg_memset(ctx, 0, sizeof(*ctx));
 }
+#endif /* HAVE_OPENSSL_MD5 */
