@@ -6,12 +6,10 @@
  *   resync buffers for sharebots
  *   help system
  *   motd display and %var substitution
- *
- * $Id: misc.c,v 1.88 2011/07/09 15:07:48 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -47,19 +45,19 @@ extern struct dcc_t *dcc;
 extern struct chanset_t *chanset;
 
 extern char helpdir[], version[], origbotname[], botname[], admin[], network[],
-            motdfile[], ver[], botnetnick[], bannerfile[], logfile_suffix[],
-            textdir[];
+            motdfile[], ver[], botnetnick[], bannerfile[], textdir[];
 extern int  backgrd, con_chan, term_z, use_stderr, dcc_total, keep_all_logs,
             quick_logs, strict_host;
 
 extern time_t now;
 extern Tcl_Interp *interp;
 
-char log_ts[32] = "[%H:%M:%S]"; /* Timestamp format for logfile entries */
+char logfile_suffix[21] = ".%d%b%Y";    /* Format of logfile suffix */
+char log_ts[33] = "[%H:%M:%S]"; /* Timestamp format for logfile entries */
 
 int shtime = 1;                 /* Display the time with console output */
 log_t *logs = 0;                /* Logfiles */
-int max_logs = 5;               /* Current maximum log files */
+int max_logs = 5;               /* Max log files, mismatch config on purpose */
 int max_logsize = 0;            /* Maximum logfile size, 0 for no limit */
 int raw_log = 0;                /* Disply output to server to LOG_SERVEROUT */
 
@@ -358,6 +356,11 @@ void maskaddr(const char *s, char *nw, int type)
   }
   *nw++ = '@';
 
+  if (type >= 30) {
+    strcpy(nw, "*");
+    return;
+  }
+
   /* The rest is for the host */
   h = (h ? h + 1 : ss);
   for (p = h; *p; p++) /* hostname? */
@@ -601,7 +604,7 @@ void putlog EGG_VARARGS_DEF(int, arg1)
                * then reset repeats. We want the current time here,
                * so put that in the file first.
                */
-              fprintf(logs[i].f, stamp);
+              fprintf(logs[i].f, "%s", stamp);
               fprintf(logs[i].f, MISC_LOGREPEAT, logs[i].repeats);
               logs[i].repeats = 0;
               /* No need to reset logs[i].szlast here
@@ -642,11 +645,11 @@ void logsuffix_change(char *s)
   char *s2 = logfile_suffix;
 
   /* If the suffix didn't really change, ignore. It's probably a rehash. */
-  if (s && s2 && !strcmp(s, s2))
+  if (!s || (s && s2 && !strcmp(s, s2)))
     return;
 
   debug0("Logfile suffix changed. Closing all open logs.");
-  strcpy(logfile_suffix, s);
+  strncpyz(logfile_suffix, s, sizeof logfile_suffix);
   while (s2[0]) {
     if (s2[0] == ' ')
       s2[0] = '_';
@@ -1054,7 +1057,7 @@ void help_subst(char *s, char *nick, struct flag_record *flags,
     }
   }
   if (cols) {
-    strcpy(xx, s);
+    strncpyz(xx, s, sizeof xx);
     s[0] = 0;
     subst_addcol(s, xx);
   }
@@ -1067,27 +1070,29 @@ static void scan_help_file(struct help_ref *current, char *filename, int type)
   struct help_list_t *list;
 
   if (is_file(filename) && (f = fopen(filename, "r"))) {
-    while (!feof(f)) {
-      fgets(s, HELP_BUF_LEN, f);
-      if (!feof(f)) {
-        p = s;
-        while ((q = strstr(p, "%{help="))) {
-          q += 7;
-          if ((p = strchr(q, '}'))) {
-            *p = 0;
-            list = nmalloc(sizeof *list);
+    /* don't check for feof after fgets, skips last line if it has no \n (ie on windows) */
+    while (!feof(f) && fgets(s, HELP_BUF_LEN, f) != NULL) {
+      p = s;
+      while ((q = strstr(p, "%{help="))) {
+        q += 7;
+        if ((p = strchr(q, '}'))) {
+          *p = 0;
+          list = nmalloc(sizeof *list);
 
-            list->name = nmalloc(p - q + 1);
-            strcpy(list->name, q);
-            list->next = current->first;
-            list->type = type;
-            current->first = list;
-            p++;
-          } else
-            p = "";
-        }
+          list->name = nmalloc(p - q + 1);
+          strcpy(list->name, q);
+          list->next = current->first;
+          list->type = type;
+          current->first = list;
+          p++;
+        } else
+          p = "";
       }
     }
+    /* fgets == NULL means error or empty file, so check for error */
+    if (ferror(f)) {
+      putlog(LOG_MISC, "*", "Error reading help file");
+    }
     fclose(f);
   }
 }
@@ -1216,20 +1221,22 @@ void showhelp(char *who, char *file, struct flag_record *flags, int fl)
 
   if (f) {
     help_subst(NULL, NULL, 0, HELP_IRC, NULL);  /* Clear flags */
-    while (!feof(f)) {
-      fgets(s, HELP_BUF_LEN, f);
-      if (!feof(f)) {
-        if (s[strlen(s) - 1] == '\n')
-          s[strlen(s) - 1] = 0;
-        if (!s[0])
-          strcpy(s, " ");
-        help_subst(s, who, flags, 0, file);
-        if ((s[0]) && (strlen(s) > 1)) {
-          dprintf(DP_HELP, "NOTICE %s :%s\n", who, s);
-          lines++;
-        }
+    /* don't check for feof after fgets, skips last line if it has no \n (ie on windows) */
+    while (!feof(f) && fgets(s, HELP_BUF_LEN, f) != NULL) {
+      if (s[strlen(s) - 1] == '\n')
+        s[strlen(s) - 1] = 0;
+      if (!s[0])
+        strcpy(s, " ");
+      help_subst(s, who, flags, 0, file);
+      if ((s[0]) && (strlen(s) > 1)) {
+        dprintf(DP_HELP, "NOTICE %s :%s\n", who, s);
+        lines++;
       }
     }
+    /* fgets == NULL means error or empty file, so check for error */
+    if (ferror(f)) {
+      putlog(LOG_MISC, "*", "Error reading help file");
+    }
     fclose(f);
   }
   if (!lines && !(fl & HELP_TEXT))
@@ -1245,20 +1252,22 @@ static int display_tellhelp(int idx, char *file, FILE *f,
   if (f) {
     help_subst(NULL, NULL, 0,
                (dcc[idx].status & STAT_TELNET) ? 0 : HELP_IRC, NULL);
-    while (!feof(f)) {
-      fgets(s, HELP_BUF_LEN, f);
-      if (!feof(f)) {
-        if (s[strlen(s) - 1] == '\n')
-          s[strlen(s) - 1] = 0;
-        if (!s[0])
-          strcpy(s, " ");
-        help_subst(s, dcc[idx].nick, flags, 1, file);
-        if (s[0]) {
-          dprintf(idx, "%s\n", s);
-          lines++;
-        }
+    /* don't check for feof after fgets, skips last line if it has no \n (ie on windows) */
+    while (!feof(f) && fgets(s, HELP_BUF_LEN, f) != NULL) {
+      if (s[strlen(s) - 1] == '\n')
+        s[strlen(s) - 1] = 0;
+      if (!s[0])
+        strcpy(s, " ");
+      help_subst(s, dcc[idx].nick, flags, 1, file);
+      if (s[0]) {
+        dprintf(idx, "%s\n", s);
+        lines++;
       }
     }
+    /* fgets == NULL means error or empty file, so check for error */
+    if (ferror(f)) {
+      putlog(LOG_MISC, "*", "Error displaying help");
+    }
     fclose(f);
   }
   return lines;
@@ -1375,17 +1384,19 @@ void show_motd(int idx)
   /* reset the help_subst variables to their defaults */
   help_subst(NULL, NULL, 0,
              (dcc[idx].status & STAT_TELNET) ? 0 : HELP_IRC, NULL);
-  while (!feof(vv)) {
-    fgets(s, 120, vv);
-    if (!feof(vv)) {
-      if (s[strlen(s) - 1] == '\n')
-        s[strlen(s) - 1] = 0;
-      if (!s[0])
-        strcpy(s, " ");
-      help_subst(s, dcc[idx].nick, &fr, 1, botnetnick);
-      if (s[0])
-        dprintf(idx, "%s\n", s);
-    }
+  /* don't check for feof after fgets, skips last line if it has no \n (ie on windows) */
+  while (!feof(vv) && fgets(s, sizeof s, vv) != NULL) {
+    if (s[strlen(s) - 1] == '\n')
+      s[strlen(s) - 1] = 0;
+    if (!s[0])
+      strcpy(s, " ");
+    help_subst(s, dcc[idx].nick, &fr, 1, botnetnick);
+    if (s[0])
+      dprintf(idx, "%s\n", s);
+  }
+  /* fgets == NULL means error or empty file, so check for error */
+  if (ferror(vv)) {
+    putlog(LOG_MISC, "*", "Error reading MOTD for DCC");
   }
   fclose(vv);
   dprintf(idx, "\n");
@@ -1409,14 +1420,19 @@ void show_banner(int idx)
   get_user_flagrec(dcc[idx].user, &fr, dcc[idx].u.chat->con_chan);
   /* reset the help_subst variables to their defaults */
   help_subst(NULL, NULL, 0, 0, NULL);
-  while (!feof(vv)) {
-    fgets(s, 120, vv);
-    if (!feof(vv)) {
-      if (!s[0])
-        strcpy(s, " \n");
-      help_subst(s, dcc[idx].nick, &fr, 0, botnetnick);
-      dprintf(idx, "%s", s);
-    }
+  /* don't check for feof after fgets, skips last line if it has no \n (ie on windows) */
+  while (!feof(vv) && fgets(s, sizeof s, vv) != NULL) {
+    if (s[strlen(s) - 1] == '\n')
+      s[strlen(s) - 1] = 0;
+    if (!s[0])
+      strcpy(s, " ");
+    help_subst(s, dcc[idx].nick, &fr, 0, botnetnick);
+    if (s[0])
+      dprintf(idx, "%s\n", s);
+  }
+  /* fgets == NULL means error or empty file, so check for error */
+  if (ferror(vv)) {
+      putlog(LOG_MISC, "*", "Error reading banner");
   }
   fclose(vv);
 }
@@ -1550,22 +1566,16 @@ void str_unescape(char *str, register const char esc_char)
   (void) strchr_unescape(str, 0, esc_char);
 }
 
-char *str_dup(const char *s)
-{
-  char *d = nmalloc(strlen(s)+1);
-  strcpy(d, s);
-  return d;
-}
-
 /* Kills the bot. s1 is the reason shown to other bots,
  * s2 the reason shown on the partyline. (Sup 25Jul2001)
  */
 void kill_bot(char *s1, char *s2)
 {
+  check_tcl_die(s2);
   call_hook(HOOK_DIE);
   chatout("*** %s\n", s1);
   botnet_send_chat(-1, botnetnick, s1);
   botnet_send_bye();
   write_userfile(-1);
-  fatal(s2, 0);
+  fatal(s2, 2);
 }
