@@ -1,12 +1,10 @@
 /*
  * channels.c -- part of channels.mod
  *   support for channels within the bot
- *
- * $Id: channels.c,v 1.109 2011/07/20 10:50:35 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,7 +29,7 @@
 
 static Function *global = NULL;
 
-static char chanfile[121], glob_chanmode[64];
+static char chanfile[121], glob_chanmode[65];
 static char *lastdeletedmask;
 
 static struct udef_struct *udef;
@@ -153,7 +151,7 @@ static void set_mode_protect(struct chanset_t *chan, char *set)
       if (pos) {
         s1 = newsplit(&set);
         if (s1[0])
-          strcpy(chan->key_prot, s1);
+          strncpyz(chan->key_prot, s1, sizeof chan->key_prot);
       }
       break;
     }
@@ -265,7 +263,7 @@ static int ismasked(masklist *m, char *user)
 
 /* Unlink chanset element from chanset list.
  */
-inline static int chanset_unlink(struct chanset_t *chan)
+static inline int chanset_unlink(struct chanset_t *chan)
 {
   struct chanset_t *c, *c_old = NULL;
 
@@ -776,10 +774,14 @@ static char *traced_globchanset(ClientData cdata, Tcl_Interp *irp,
 
   if (flags & (TCL_TRACE_READS | TCL_TRACE_UNSETS)) {
     Tcl_SetVar2(interp, name1, name2, glob_chanset, TCL_GLOBAL_ONLY);
-    if (flags & TCL_TRACE_UNSETS)
+    if (flags & TCL_TRACE_UNSETS) {
       Tcl_TraceVar(interp, "global-chanset",
+                   TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
+                   traced_globchanset, NULL); /* keep for backward compatibility */
+      Tcl_TraceVar(interp, "default-chanset",
                    TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
                    traced_globchanset, NULL);
+    }
   } else {                        /* Write */
     s2 = Tcl_GetVar2(interp, name1, name2, TCL_GLOBAL_ONLY);
     Tcl_SplitList(interp, s2, &items, &item);
@@ -807,10 +809,18 @@ static char *traced_globchanset(ClientData cdata, Tcl_Interp *irp,
 }
 
 static tcl_ints my_tcl_ints[] = {
-  {"share-greet",             NULL,                     0},
-  {"use-info",                &use_info,                0},
-  {"quiet-save",              &quiet_save,              0},
-  {"allow-ps",                &allow_ps,                0},
+  {"share-greet",              NULL,                     0},
+  {"use-info",                 &use_info,                0},
+  {"quiet-save",               &quiet_save,              0},
+  {"allow-ps",                 &allow_ps,                0},
+  {"default-stopnethack-mode", &global_stopnethack_mode, 0},
+  {"default-revenge-mode",     &global_revenge_mode,     0},
+  {"default-idle-kick",        &global_idle_kick,        0},
+  {"default-ban-time",         &global_ban_time,         0},
+  {"default-exempt-time",      &global_exempt_time,      0},
+  {"default-invite-time",      &global_invite_time,      0},
+  {"default-ban-type",         &global_ban_type,         0},
+  /* keep global-* for backward compatibility */
   {"global-stopnethack-mode", &global_stopnethack_mode, 0},
   {"global-revenge-mode",     &global_revenge_mode,     0},
   {"global-idle-kick",        &global_idle_kick,        0},
@@ -826,6 +836,14 @@ static tcl_ints my_tcl_ints[] = {
 };
 
 static tcl_coups mychan_tcl_coups[] = {
+  {"default-flood-chan", &gfld_chan_thr,  &gfld_chan_time},
+  {"default-flood-deop", &gfld_deop_thr,  &gfld_deop_time},
+  {"default-flood-kick", &gfld_kick_thr,  &gfld_kick_time},
+  {"default-flood-join", &gfld_join_thr,  &gfld_join_time},
+  {"default-flood-ctcp", &gfld_ctcp_thr,  &gfld_ctcp_time},
+  {"default-flood-nick", &gfld_nick_thr,  &gfld_nick_time},
+  {"default-aop-delay",  &global_aop_min, &global_aop_max},
+  /* keep global-* for backward compatibility */
   {"global-flood-chan", &gfld_chan_thr,  &gfld_chan_time},
   {"global-flood-deop", &gfld_deop_thr,  &gfld_deop_time},
   {"global-flood-kick", &gfld_kick_thr,  &gfld_kick_time},
@@ -837,7 +855,9 @@ static tcl_coups mychan_tcl_coups[] = {
 };
 
 static tcl_strings my_tcl_strings[] = {
-  {"chanfile",        chanfile,      120, STR_PROTECT},
+  {"chanfile",         chanfile,      120, STR_PROTECT},
+  {"default-chanmode", glob_chanmode, 64,            0},
+  /* keep global-chanmode for backward compatibility */
   {"global-chanmode", glob_chanmode, 64,            0},
   {NULL,              NULL,          0,             0}
 };
@@ -862,6 +882,9 @@ static char *channels_close()
   del_hook(HOOK_MINUTELY, (Function) check_expired_exempts);
   del_hook(HOOK_MINUTELY, (Function) check_expired_invites);
   Tcl_UntraceVar(interp, "global-chanset",
+                 TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
+                 traced_globchanset, NULL); /* keep for backward compatibility */
+  Tcl_UntraceVar(interp, "default-chanset",
                  TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
                  traced_globchanset, NULL);
   rem_help_reference("channels.help");
@@ -941,7 +964,7 @@ char *channels_start(Function *global_funcs)
 {
   global = global_funcs;
 
-  gfld_chan_thr = 10;
+  gfld_chan_thr = 15;
   gfld_chan_time = 60;
   gfld_deop_thr = 3;
   gfld_deop_time = 10;
@@ -949,8 +972,10 @@ char *channels_start(Function *global_funcs)
   gfld_kick_time = 10;
   gfld_join_thr = 5;
   gfld_join_time = 60;
-  gfld_ctcp_thr = 5;
+  gfld_ctcp_thr = 3;
   gfld_ctcp_time = 60;
+  gfld_nick_thr = 5;
+  gfld_nick_time = 60;
   global_idle_kick = 0;
   global_aop_min = 5;
   global_aop_max = 30;
@@ -976,7 +1001,7 @@ char *channels_start(Function *global_funcs)
          "-bitch "
          "+greet "
          "+protectops "
-         "+statuslog "
+         "-statuslog "
          "-revenge "
          "-secret "
          "-autovoice "
@@ -995,10 +1020,10 @@ char *channels_start(Function *global_funcs)
          "-autohalfop "
          "-nodesynch "
          "-static ");
-  module_register(MODULE_NAME, channels_table, 1, 1);
-  if (!module_depend(MODULE_NAME, "eggdrop", 106, 20)) {
+  module_register(MODULE_NAME, channels_table, 1, 2);
+  if (!module_depend(MODULE_NAME, "eggdrop", 108, 0)) {
     module_undepend(MODULE_NAME);
-    return "This module requires Eggdrop 1.6.20 or later.";
+    return "This module requires Eggdrop 1.8.0 or later.";
   }
   add_hook(HOOK_MINUTELY, (Function) check_expired_bans);
   add_hook(HOOK_MINUTELY, (Function) check_expired_exempts);
@@ -1008,6 +1033,9 @@ char *channels_start(Function *global_funcs)
   add_hook(HOOK_REHASH, (Function) channels_rehash);
   add_hook(HOOK_PRE_REHASH, (Function) channels_prerehash);
   Tcl_TraceVar(interp, "global-chanset",
+               TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
+               traced_globchanset, NULL); /* keep for backward compatibility */
+  Tcl_TraceVar(interp, "default-chanset",
                TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
                traced_globchanset, NULL);
   add_builtins(H_chon, my_chon);
@@ -1019,6 +1047,6 @@ char *channels_start(Function *global_funcs)
   my_tcl_ints[0].val = &share_greet;
   add_tcl_ints(my_tcl_ints);
   add_tcl_coups(mychan_tcl_coups);
-  read_channels(0, 1);
+  read_channels(0, 0);
   return NULL;
 }
