@@ -1,12 +1,10 @@
 /*
  * cmdschan.c -- part of channels.mod
  *   commands from a user via dcc that cause server interaction
- *
- * $Id: cmdschan.c,v 1.87 2011/10/02 18:25:17 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -41,7 +39,14 @@ static void cmd_pls_ban(struct userrec *u, int idx, char *par)
     dprintf(idx, "Usage: +ban <hostmask> [channel] [%%<XdXhXm>] [reason]\n");
   } else {
     who = newsplit(&par);
-    if (par[0] && strchr(CHANMETA, par[0]))
+    /* Sanity check for <channel> <ban> vs. <ban> <channel> */
+    if (par[0] && strchr(CHANMETA, who[0])) {
+      chname = who;
+      who = newsplit(&par);
+      dprintf(idx, "Usage: +ban <hostmask> [channel] [%%<XdXhXm>] [reason]\n");
+      dprintf(idx, "Did you mean: .+ban %s %s %s\n", who, chname, par);
+      return;
+    } else if (par[0] && strchr(CHANMETA, par[0]))
       chname = newsplit(&par);
     else
       chname = 0;
@@ -109,15 +114,15 @@ static void cmd_pls_ban(struct userrec *u, int idx, char *par)
       egg_snprintf(s, sizeof s, "%s@*", who);   /* brain-dead? */
     else
       strncpyz(s, who, sizeof s);
-    if ((me = module_find("server", 0, 0)) && me->funcs)
+    if ((me = module_find("server", 0, 0)) && me->funcs) {
       egg_snprintf(s1, sizeof s1, "%s!%s", me->funcs[SERVER_BOTNAME],
                    me->funcs[SERVER_BOTUSERHOST]);
-    else
-      egg_snprintf(s1, sizeof s1, "%s!%s@%s", origbotname, botuser, hostname);
-    if (match_addr(s, s1)) {
-      dprintf(idx, "I'm not going to ban myself.\n");
-      putlog(LOG_CMDS, "*", "#%s# attempted +ban %s", dcc[idx].nick, s);
-    } else {
+      if (match_addr(s, s1)) {
+        dprintf(idx, "I'm not going to ban myself.\n");
+        putlog(LOG_CMDS, "*", "#%s# attempted +ban %s", dcc[idx].nick, s);
+        return;
+      }
+    }
       /* IRC can't understand bans longer than 70 characters */
       if (strlen(s) > 70) {
         s[69] = '*';
@@ -160,16 +165,14 @@ static void cmd_pls_ban(struct userrec *u, int idx, char *par)
           for (chan = chanset; chan != NULL; chan = chan->next)
             (me->funcs[IRC_CHECK_THIS_BAN]) (chan, s, sticky);
       }
-    }
   }
 }
 
 static void cmd_pls_exempt(struct userrec *u, int idx, char *par)
 {
-  char *chname, *who, s[UHOSTLEN], s1[UHOSTLEN], *p, *p_expire;
+  char *chname, *who, s[UHOSTLEN], *p, *p_expire;
   unsigned long int expire_time = 0, expire_foo;
   struct chanset_t *chan = NULL;
-  module_entry *me;
 
   if (!use_exempts) {
     dprintf(idx, "This command can only be used with use-exempts enabled.\n");
@@ -247,11 +250,6 @@ static void cmd_pls_exempt(struct userrec *u, int idx, char *par)
       egg_snprintf(s, sizeof s, "%s@*", who);   /* brain-dead? */
     else
       strncpyz(s, who, sizeof s);
-    if ((me = module_find("server", 0, 0)) && me->funcs)
-      egg_snprintf(s1, sizeof s1, "%s!%s", me->funcs[SERVER_BOTNAME],
-                   me->funcs[SERVER_BOTUSERHOST]);
-    else
-      egg_snprintf(s1, sizeof s1, "%s!%s@%s", origbotname, botuser, hostname);
 
     /* IRC can't understand exempts longer than 70 characters */
     if (strlen(s) > 70) {
@@ -293,10 +291,9 @@ static void cmd_pls_exempt(struct userrec *u, int idx, char *par)
 
 static void cmd_pls_invite(struct userrec *u, int idx, char *par)
 {
-  char *chname, *who, s[UHOSTLEN], s1[UHOSTLEN], *p, *p_expire;
+  char *chname, *who, s[UHOSTLEN], *p, *p_expire;
   unsigned long int expire_time = 0, expire_foo;
   struct chanset_t *chan = NULL;
-  module_entry *me;
 
   if (!use_invites) {
     dprintf(idx, "This command can only be used with use-invites enabled.\n");
@@ -375,11 +372,6 @@ static void cmd_pls_invite(struct userrec *u, int idx, char *par)
       egg_snprintf(s, sizeof s, "%s@*", who);   /* brain-dead? */
     else
       strncpyz(s, who, sizeof s);
-    if ((me = module_find("server", 0, 0)) && me->funcs)
-      egg_snprintf(s1, sizeof s1, "%s!%s", me->funcs[SERVER_BOTNAME],
-                   me->funcs[SERVER_BOTUSERHOST]);
-    else
-      egg_snprintf(s1, sizeof s1, "%s!%s@%s", origbotname, botuser, hostname);
 
     /* IRC can't understand invites longer than 70 characters */
     if (strlen(s) > 70) {
@@ -431,7 +423,14 @@ static void cmd_mns_ban(struct userrec *u, int idx, char *par)
     return;
   }
   ban = newsplit(&par);
-  if (par[0] && strchr(CHANMETA, par[0]))
+  /* Sanity check for <channel> <ban> vs. <ban> <channel> */
+  if (par[0] && strchr(CHANMETA, ban[0])) {
+      chname = ban;
+      ban = newsplit(&par);
+      dprintf(idx, "Usage: -ban <hostmask|ban #> [channel]\n");
+      dprintf(idx, "Did you mean: .-ban %s %s\n", ban, chname);
+      return;
+  } else if (par[0] && strchr(CHANMETA, par[0]))
     chname = newsplit(&par);
   else {
     chname = dcc[idx].u.chat->con_chan;
@@ -1530,6 +1529,7 @@ static void cmd_chanset(struct userrec *u, int idx, char *par)
          * just ignore any non global +n's trying to set the need-commands.
          */
         if (strncmp(list[0], "need-", 5) || (u->flags & USER_OWNER)) {
+          Tcl_Interp *irp = NULL;
           if (!strncmp(list[0], "need-", 5) && !(isowner(dcc[idx].nick)) &&
               must_be_owner) {
             dprintf(idx, "Due to security concerns, only permanent owners can set these modes.\n");
@@ -1542,14 +1542,16 @@ static void cmd_chanset(struct userrec *u, int idx, char *par)
            */
           parcpy = nmalloc(strlen(par) + 1);
           strcpy(parcpy, par);
-          if (tcl_channel_modify(0, chan, 2, list) == TCL_OK) {
-            strcat(answers, list[0]);
-            strcat(answers, " { ");
-            strcat(answers, parcpy);
-            strcat(answers, " }");
+          irp = Tcl_CreateInterp();
+          if (tcl_channel_modify(irp, chan, 2, list) == TCL_OK) {
+            char tocat[sizeof answers];
+            egg_snprintf(tocat, sizeof tocat, "%s { %s }", list[0], parcpy);
+            strncat(answers, tocat, sizeof answers - strlen(answers) - 1);
           } else if (!all || !chan->next)
-            dprintf(idx, "Error trying to set %s for %s, invalid option\n",
-                    list[0], all ? "all channels" : chname);
+            dprintf(idx, "Error trying to set %s for %s, %s\n",
+                    list[0], all ? "all channels" : chname, Tcl_GetStringResult(irp));
+          Tcl_ResetResult(irp);
+          Tcl_DeleteInterp(irp);
           nfree(parcpy);
         }
         break;
