@@ -5,11 +5,9 @@
  *
  * Written by Fabian Knittel <fknittel@gmx.de>. Based on zlib examples
  * by Jean-loup Gailly and Miguel Albrecht.
- *
- * $Id: compress.c,v 1.31 2011/02/13 14:19:33 simple Exp $
  */
 /*
- * Copyright (C) 2000 - 2011 Eggheads Development Team
+ * Copyright (C) 2000 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -73,6 +71,7 @@ static int is_compressedfile(char *filename)
 {
   char buf1[50], buf2[50];
   FILE *fin;
+  gzFile zin;
   register int len1, len2, i;
 
   egg_memset(buf1, 0, 50);
@@ -82,13 +81,13 @@ static int is_compressedfile(char *filename)
 
   /* Read data with zlib routines.
    */
-  fin = gzopen(filename, "rb");
-  if (!fin)
+  zin = gzopen(filename, "rb");
+  if (!zin)
     return COMPF_FAILED;
-  len1 = gzread(fin, buf1, sizeof(buf1));
+  len1 = gzread(zin, buf1, sizeof(buf1));
   if (len1 < 0)
     return COMPF_FAILED;
-  if (gzclose(fin) != Z_OK)
+  if (gzclose(zin) != Z_OK)
     return COMPF_FAILED;
 
   /* Read raw data.
@@ -97,8 +96,10 @@ static int is_compressedfile(char *filename)
   if (!fin)
     return COMPF_FAILED;
   len2 = fread(buf2, 1, sizeof(buf2), fin);
-  if (ferror(fin))
+  if (ferror(fin)) {
+    fclose(fin);
     return COMPF_FAILED;
+  }
   fclose(fin);
 
   /* Compare what we found.
@@ -122,7 +123,8 @@ static int uncompress_to_file(char *f_src, char *f_target)
 {
   char buf[BUFLEN];
   int len;
-  FILE *fin, *fout;
+  FILE *fout;
+  gzFile fin;
 
   if (!is_file(f_src)) {
     putlog(LOG_MISC, "*", "Failed to uncompress file `%s': not a file.", f_src);
@@ -173,7 +175,7 @@ static int uncompress_to_file(char *f_src, char *f_target)
 
 /* Enforce limits.
  */
-inline static void adjust_mode_num(int *mode)
+static inline void adjust_mode_num(int *mode)
 {
   if (*mode > 9)
     *mode = 9;
@@ -184,7 +186,7 @@ inline static void adjust_mode_num(int *mode)
 #ifdef HAVE_MMAP
 /* Attempt to compress in one go, by mmap'ing the file to memory.
  */
-static int compress_to_file_mmap(FILE *fout, FILE *fin)
+static int compress_to_file_mmap(gzFile fout, FILE *fin)
 {
   int len, ifd = fileno(fin);
   char *buf;
@@ -219,7 +221,8 @@ static int compress_to_file_mmap(FILE *fout, FILE *fin)
 static int compress_to_file(char *f_src, char *f_target, int mode_num)
 {
   char buf[BUFLEN], mode[5];
-  FILE *fin, *fout;
+  FILE *fin;
+  gzFile fout;
   int len;
 
   adjust_mode_num(&mode_num);
@@ -238,6 +241,7 @@ static int compress_to_file(char *f_src, char *f_target, int mode_num)
 
   fout = gzopen(f_target, mode);
   if (!fout) {
+    fclose(fin);
     putlog(LOG_MISC, "*", "Failed to compress file `%s': gzopen failed.",
            f_src);
     return COMPF_ERROR;
@@ -259,6 +263,8 @@ static int compress_to_file(char *f_src, char *f_target, int mode_num)
   while (1) {
     len = fread(buf, 1, sizeof(buf), fin);
     if (ferror(fin)) {
+      fclose(fin);
+      gzclose(fout);
       putlog(LOG_MISC, "*", "Failed to compress file `%s': fread failed: %s",
              f_src, strerror(errno));
       return COMPF_ERROR;
@@ -266,6 +272,8 @@ static int compress_to_file(char *f_src, char *f_target, int mode_num)
     if (!len)
       break;
     if (gzwrite(fout, buf, (unsigned int) len) != len) {
+      fclose(fin);
+      gzclose(fout);
       putlog(LOG_MISC, "*", "Failed to compress file `%s': gzwrite failed.",
              f_src);
       return COMPF_ERROR;
@@ -430,10 +438,10 @@ char *compress_start(Function *global_funcs)
   share_compressed = 0;
   compress_level = 9;
 
-  module_register(MODULE_NAME, compress_table, 1, 1);
-  if (!module_depend(MODULE_NAME, "eggdrop", 106, 0)) {
+  module_register(MODULE_NAME, compress_table, 1, 2);
+  if (!module_depend(MODULE_NAME, "eggdrop", 108, 0)) {
     module_undepend(MODULE_NAME);
-    return "This module requires Eggdrop 1.6.0 or later.";
+    return "This module requires Eggdrop 1.8.0 or later.";
   }
 
   share_funcs = module_depend(MODULE_NAME, "share", 2, 3);
