@@ -1,12 +1,10 @@
 /*
  * ctcp.c -- part of ctcp.mod
  *   all the ctcp handling (except DCC, it's special ;)
- *
- * $Id: ctcp.c,v 1.29 2011/02/13 14:19:33 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -145,6 +143,11 @@ static int ctcp_CHAT(char *nick, char *uhost, char *handle, char *object,
 {
   struct userrec *u = get_user_by_handle(userlist, handle);
   int atr = u ? u->flags : 0, i;
+  int chatv = AF_UNSPEC;
+  char s[INET6_ADDRSTRLEN];
+#ifdef TLS
+  int ssl = 0;
+#endif
 
   if ((atr & (USER_PARTY | USER_XFER)) || ((atr & USER_OP) && !require_p)) {
 
@@ -154,20 +157,55 @@ static int ctcp_CHAT(char *nick, char *uhost, char *handle, char *object,
       return 1;
     }
 
+// * Check if SSL, IPv4, or IPv6 were requested
+    if (
+#ifdef IPV6
+    (!egg_strcasecmp(keyword, "CHAT6")) ||
+        (!egg_strcasecmp(keyword, "SCHAT6"))) {
+      chatv = AF_INET6;
+    } else if (
+#endif
+#ifdef TLS
+    (!egg_strcasecmp(keyword, "SCHAT")) ||
+#ifdef IPV6
+        (!egg_strcasecmp(keyword, "SCHAT6")) ||
+#endif
+        (!egg_strcasecmp(keyword, "SCHAT4"))) {
+      ssl = 1;
+    } else if (
+#endif
+    (!egg_strcasecmp(keyword, "CHAT4")) ||
+        (!egg_strcasecmp(keyword, "SCHAT4"))) {
+      chatv = AF_INET;
+    }
+  
     for (i = 0; i < dcc_total; i++) {
       if ((dcc[i].type->flags & DCT_LISTEN) &&
+#ifdef TLS
+          (ssl == dcc[i].ssl) &&
+#endif
           (!strcmp(dcc[i].nick, "(telnet)") ||
-           !strcmp(dcc[i].nick, "(users)"))) {
+           !strcmp(dcc[i].nick, "(users)")) &&
+          getdccfamilyaddr(&dcc[i].sockname, s, sizeof s, chatv)) {
         /* Do me a favour and don't change this back to a CTCP reply,
          * CTCP replies are NOTICE's this has to be a PRIVMSG
          * -poptix 5/1/1997 */
-        dprintf(DP_SERVER, "PRIVMSG %s :\001DCC CHAT chat %lu %u\001\n",
-                nick, iptolong(natip[0] ? (IP) inet_addr(natip) : getmyip()),
-                dcc[i].port);
+#ifdef TLS
+	  dprintf(DP_SERVER, "PRIVMSG %s :\001DCC %sCHAT chat %s %u\001\n",
+		  nick, (ssl ? "S" : ""), s, dcc[i].port);
+#else
+          dprintf(DP_SERVER, "PRIVMSG %s :\001DCC CHAT chat %s %u\001\n",
+                  nick, s, dcc[i].port);
+#endif
         return 1;
       }
     }
+#ifdef TLS
+    simple_sprintf(ctcp_reply, "%s\001ERROR no %stelnet port\001", ctcp_reply,
+	 	   (ssl ? "SSL enabled " : ""));
+#else
     simple_sprintf(ctcp_reply, "%s\001ERROR no telnet port\001", ctcp_reply);
+#endif
   }
   return 1;
 }
@@ -182,6 +220,17 @@ static cmd_t myctcp[] = {
   {"CLIENTINFO", "",   ctcp_CLIENTINFO, NULL},
   {"TIME",       "",   ctcp_TIME,       NULL},
   {"CHAT",       "",   ctcp_CHAT,       NULL},
+  {"CHAT4",      "",   ctcp_CHAT,       NULL},
+#ifdef IPV6
+  {"CHAT6",      "",   ctcp_CHAT,       NULL},
+#endif
+#ifdef TLS
+  {"SCHAT",      "",   ctcp_CHAT,       NULL},
+  {"SCHAT4",     "",   ctcp_CHAT,       NULL},
+#ifdef IPV6
+  {"SCHAT6",     "",   ctcp_CHAT,       NULL},
+#endif
+#endif
   {NULL,         NULL, NULL,            NULL}
 };
 
@@ -220,10 +269,10 @@ char *ctcp_start(Function *global_funcs)
 {
   global = global_funcs;
 
-  module_register(MODULE_NAME, ctcp_table, 1, 0);
-  if (!module_depend(MODULE_NAME, "eggdrop", 106, 0)) {
+  module_register(MODULE_NAME, ctcp_table, 1, 1);
+  if (!module_depend(MODULE_NAME, "eggdrop", 108, 0)) {
     module_undepend(MODULE_NAME);
-    return "This module requires Eggdrop 1.6.0 or later.";
+    return "This module requires Eggdrop 1.8.0 or later.";
   }
   if (!(server_funcs = module_depend(MODULE_NAME, "server", 1, 0))) {
     module_undepend(MODULE_NAME);
