@@ -3,11 +3,9 @@
  *   dns module header file
  *
  * Written by Fabian Knittel <fknittel@gmx.de>
- *
- * $Id: dns.h,v 1.20 2011/02/13 14:19:33 simple Exp $
  */
 /*
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -49,6 +47,9 @@ struct resolve {
   time_t expiretime;
   char *hostn;
   IP ip;
+  u_32bit_t ttl;
+  u_16bit_t type;
+  sockname_t sockname;
   u_16bit_t id;
   u_8bit_t state;
   u_8bit_t sends;
