@@ -1,12 +1,10 @@
 /*
  * files.c - part of filesys.mod
  *   handles all file system commands
- *
- * $Id: files.c,v 1.57 2011/02/13 14:19:33 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -91,7 +89,7 @@ static int welcome_to_files(int idx)
   sub_lang(idx, FILES_WELCOME);
   sub_lang(idx, FILES_WELCOME1);
   if (p)
-    strcpy(dcc[idx].u.file->dir, p);
+    strncpyz(dcc[idx].u.file->dir, p, sizeof dcc[idx].u.file->dir);
   else
     dcc[idx].u.file->dir[0] = 0;
   /* Does this dir even exist any more? */
@@ -1391,7 +1389,7 @@ static cmd_t myfiles[] = {
 static int files_reget(int idx, char *fn, char *nick, int resend)
 {
   int i = 0;
-  char *p = NULL, *what = NULL, *destdir, *s = NULL;
+  char *p = NULL, *what = NULL, *destdir = NULL, *s = NULL;
   filedb_entry *fdbe = NULL;
   FILE *fdb = NULL;
 
@@ -1495,7 +1493,7 @@ static void files_setpwd(int idx, char *where)
 
   if (!resolve_dir(dcc[idx].u.file->dir, where, &s, idx))
     return;
-  strcpy(dcc[idx].u.file->dir, s);
+  strncpyz(dcc[idx].u.file->dir, s, sizeof dcc[idx].u.file->dir);
   set_user(&USERENTRY_DCCDIR, get_user_by_handle(userlist, dcc[idx].nick),
            dcc[idx].u.file->dir);
   my_free(s);
