@@ -1,12 +1,10 @@
 /*
  * filesys.c -- part of filesys.mod
  *   main file of the filesys eggdrop module
- *
- * $Id: filesys.c,v 1.79 2011/02/13 14:19:33 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,6 +27,7 @@
 #include <fcntl.h>
 #include <sys/stat.h>
 #include <sys/file.h>
+#include <errno.h>
 
 #include "src/mod/module.h"
 
@@ -130,7 +129,8 @@ static struct user_entry_type USERENTRY_DCCDIR = {
   NULL,
   NULL,
   NULL,
-  "DCCDIR"
+  "DCCDIR",
+  NULL
 };
 
 #include "files.c"
@@ -261,8 +261,7 @@ static void dcc_files(int idx, char *buf, int i)
       dcc[idx].status &= (~STAT_CHAT);
       dcc[idx].type = &DCC_CHAT;
       if (dcc[idx].u.chat->channel >= 0) {
-        chanout_but(-1, dcc[idx].u.chat->channel,
-                    "*** %s has returned.\n", dcc[idx].nick);
+        chanout_but(-1, dcc[idx].u.chat->channel, DCC_RETURN, dcc[idx].nick);
         if (dcc[idx].u.chat->channel < GLOBAL_CHANS)
           botnet_send_join_idx(idx, -1);
       }
@@ -345,8 +344,7 @@ static int cmd_files(struct userrec *u, int idx, char *par)
         dcc[idx].type = &DCC_CHAT;
         putlog(LOG_FILES, "*", "File system broken.");
         if (dcc[idx].u.chat->channel >= 0) {
-          chanout_but(-1, dcc[idx].u.chat->channel,
-                      "*** %s has returned.\n", dcc[idx].nick);
+          chanout_but(-1, dcc[idx].u.chat->channel, DCC_RETURN, dcc[idx].nick);
           if (dcc[idx].u.chat->channel < GLOBAL_CHANS)
             botnet_send_join_idx(idx, -1);
         }
@@ -357,8 +355,7 @@ static int cmd_files(struct userrec *u, int idx, char *par)
   return 0;
 }
 
-static int _dcc_send(int idx, char *filename, char *nick, char *dir,
-                     int resend)
+static int _dcc_send(int idx, char *filename, char *nick, int resend)
 {
   int x;
   char *nfn, *buf = NULL;
@@ -366,9 +363,9 @@ static int _dcc_send(int idx, char *filename, char *nick, char *dir,
   if (strlen(nick) > NICKMAX)
     nick[NICKMAX] = 0;
   if (resend)
-    x = raw_dcc_resend(filename, nick, dcc[idx].nick, dir);
+    x = raw_dcc_resend(filename, nick, dcc[idx].nick);
   else
-    x = raw_dcc_send(filename, nick, dcc[idx].nick, dir);
+    x = raw_dcc_send(filename, nick, dcc[idx].nick);
   if (x == DCCSEND_FULL) {
     dprintf(idx, "Sorry, too many DCC connections.  (try again later)\n");
     putlog(LOG_FILES, "*", "DCC connections full: %sGET %s [%s]", filename,
@@ -399,9 +396,16 @@ static int _dcc_send(int idx, char *filename, char *nick, char *dir,
            dcc[idx].nick);
     return 0;
   }
-  nfn = strrchr(dir, '/');
+  if (x == DCCSEND_FCOPY) {
+    dprintf(idx, "Can't make temporary copy of file!\n");
+    putlog(LOG_FILES | LOG_MISC, "*",
+           "Refused dcc %sget %s: copy to temporary location FAILED!",
+           resend ? "re" : "", filename);
+    return 0;
+  }
+  nfn = strrchr(filename, '/');
   if (nfn == NULL)
-    nfn = dir;
+    nfn = filename;
   else
     nfn++;
 
@@ -425,7 +429,7 @@ static int _dcc_send(int idx, char *filename, char *nick, char *dir,
 
 static int do_dcc_send(int idx, char *dir, char *fn, char *nick, int resend)
 {
-  char *s = NULL, *s1 = NULL;
+  char *s = NULL;
   int x;
 
   if (nick && strlen(nick) > NICKMAX)
@@ -470,35 +474,8 @@ static int do_dcc_send(int idx, char *dir, char *fn, char *nick, int resend)
     my_free(s);
     return 1;
   }
-  if (copy_to_tmp) {
-    char *tempfn = mktempfile(fn);
-
-    /* Copy this file to /tmp, add a random prefix to the filename. */
-    s = nrealloc(s, strlen(dccdir) + strlen(dir) + strlen(fn) + 2);
-    sprintf(s, "%s%s%s%s", dccdir, dir, dir[0] ? "/" : "", fn);
-    s1 = nrealloc(s1, strlen(tempdir) + strlen(tempfn) + 1);
-    sprintf(s1, "%s%s", tempdir, tempfn);
-    my_free(tempfn);
-    if (copyfile(s, s1) != 0) {
-      dprintf(idx, "Can't make temporary copy of file!\n");
-      putlog(LOG_FILES | LOG_MISC, "*",
-             "Refused dcc %sget %s: copy to %s FAILED!",
-             resend ? "re" : "", fn, tempdir);
-      my_free(s);
-      my_free(s1);
-      return 0;
-    }
-  } else {
-    s1 = nrealloc(s1, strlen(dccdir) + strlen(dir) + strlen(fn) + 2);
-    sprintf(s1, "%s%s%s%s", dccdir, dir, dir[0] ? "/" : "", fn);
-  }
-  s = nrealloc(s, strlen(dir) + strlen(fn) + 2);
-  sprintf(s, "%s%s%s", dir, dir[0] ? "/" : "", fn);
-  x = _dcc_send(idx, s1, nick, s, resend);
-  if (x != DCCSEND_OK)
-    wipe_tmp_filename(s1, -1);
+  x = _dcc_send(idx, s, nick, resend);
   my_free(s);
-  my_free(s1);
   return x;
 }
 
@@ -632,7 +609,11 @@ static void filesys_dcc_send_hostresolved(int);
 /* Received a ctcp-dcc.
  */
 static void filesys_dcc_send(char *nick, char *from, struct userrec *u,
+#ifdef TLS
+                             char *text, int ssl)
+#else
                              char *text)
+#endif
 {
   char *param, *ip, *prt, *buf = NULL, *msg;
   int atr = u ? u->flags : 0, i, j = 0;
@@ -696,21 +677,24 @@ static void filesys_dcc_send(char *nick, char *from, struct userrec *u,
                nick, from);
         return;
       }
-      dcc[i].addr = my_atoul(ip);
       dcc[i].port = atoi(prt);
+      (void) setsockname(&dcc[i].sockname, ip, dcc[i].port, 0);
+      dcc[i].u.dns->ip = &dcc[i].sockname;
       dcc[i].sock = -1;
+#ifdef TLS
+      dcc[i].ssl = ssl;
+#endif
       dcc[i].user = u;
-      strcpy(dcc[i].nick, nick);
+      strncpyz(dcc[i].nick, nick, sizeof dcc[i].nick);
       strcpy(dcc[i].host, from);
       dcc[i].u.dns->cbuf = get_data_ptr(strlen(param) + 1);
       strcpy(dcc[i].u.dns->cbuf, param);
       dcc[i].u.dns->ibuf = atoi(msg);
-      dcc[i].u.dns->ip = dcc[i].addr;
       dcc[i].u.dns->dns_type = RES_HOSTBYIP;
       dcc[i].u.dns->dns_success = filesys_dcc_send_hostresolved;
       dcc[i].u.dns->dns_failure = filesys_dcc_send_hostresolved;
       dcc[i].u.dns->type = &DCC_FORK_SEND;
-      dcc_dnshostbyip(dcc[i].addr);
+      dcc_dnshostbyip(&dcc[i].sockname);
     }
   }
   my_free(buf);
@@ -748,12 +732,11 @@ static char *mktempfile(char *filename)
 
 static void filesys_dcc_send_hostresolved(int i)
 {
-  char *s1, *param, prt[100], ip[100], *tempf;
+  char *s1, *param, prt[100], *tempf;
   int len = dcc[i].u.dns->ibuf, j;
 
   sprintf(prt, "%d", dcc[i].port);
-  sprintf(ip, "%lu", iptolong(htonl(dcc[i].addr)));
-  if (!hostsanitycheck_dcc(dcc[i].nick, dcc[i].u.dns->host, dcc[i].addr,
+  if (!hostsanitycheck_dcc(dcc[i].nick, dcc[i].u.dns->host, &dcc[i].sockname,
                            dcc[i].u.dns->host, prt)) {
     lostdcc(i);
     return;
@@ -808,11 +791,8 @@ static void filesys_dcc_send_hostresolved(int i)
           }
         }
       }
-    /* Put uploads in /tmp first */
-    s1 = nmalloc(strlen(tempdir) + strlen(dcc[i].u.xfer->filename) + 1);
-    sprintf(s1, "%s%s", tempdir, dcc[i].u.xfer->filename);
-    dcc[i].u.xfer->f = fopen(s1, "w");
-    my_free(s1);
+    /* Put uploads in a temp file first */
+    dcc[i].u.xfer->f = tmpfile();
     if (dcc[i].u.xfer->f == NULL) {
       dprintf(DP_HELP,
               "NOTICE %s :Can't create file `%s' (temp dir error)\n",
@@ -820,9 +800,15 @@ static void filesys_dcc_send_hostresolved(int i)
       lostdcc(i);
     } else {
       dcc[i].timeval = now;
-      dcc[i].sock = getsock(SOCK_BINARY);
-      if (dcc[i].sock < 0 || open_telnet_dcc(dcc[i].sock, ip, prt) < 0)
+      dcc[i].sock = getsock(dcc[i].sockname.family, SOCK_BINARY);
+      if (dcc[i].sock < 0 ||
+      open_telnet_raw(dcc[i].sock, &dcc[i].sockname) < 0)
         dcc[i].type->eof(i);
+#ifdef TLS
+      else if (dcc[i].ssl && ssl_handshake(dcc[i].sock, TLS_CONNECT, tls_vfydcc,
+                                           LOG_FILES, dcc[i].host, NULL))
+        dcc[i].type->eof(i);
+#endif
     }
   }
 }
@@ -833,19 +819,29 @@ static int filesys_DCC_CHAT(char *nick, char *from, char *handle,
                             char *object, char *keyword, char *text)
 {
   char *param, *ip, *prt, buf[512], *msg = buf;
-  int i, sock;
+  int i;
   struct userrec *u = get_user_by_handle(userlist, handle);
   struct flag_record fr = { FR_GLOBAL | FR_CHAN | FR_ANYWH, 0, 0, 0, 0, 0 };
 
   if (egg_strcasecmp(object, botname))
     return 0;
   if (!egg_strncasecmp(text, "SEND ", 5)) {
+#ifdef TLS
+    filesys_dcc_send(nick, from, u, text + 5, 0);
+#else
     filesys_dcc_send(nick, from, u, text + 5);
+#endif
     return 1;
   }
+#ifdef TLS
+  if (!egg_strncasecmp(text, "SSEND ", 5)) {
+    filesys_dcc_send(nick, from, u, text + 5, 1);
+    return 1;
+  }
+#endif
   if (egg_strncasecmp(text, "CHAT ", 5) || !u)
     return 0;
-  strcpy(buf, text + 5);
+  strncpyz(buf, text + 5, sizeof buf);
   get_user_flagrec(u, &fr, 0);
   param = newsplit(&msg);
   if (dcc_total == max_dcc && increase_socks_max()) {
@@ -865,27 +861,24 @@ static int filesys_DCC_CHAT(char *nick, char *from, char *handle,
   } else {
     ip = newsplit(&msg);
     prt = newsplit(&msg);
-    sock = getsock(0);
-    if (sock < 0 || open_telnet_dcc(sock, ip, prt) < 0) {
-      neterror(buf);
-      if (!quiet_reject)
-        dprintf(DP_HELP, "NOTICE %s :%s (%s)\n", nick, DCC_CONNECTFAILED1, buf);
-      putlog(LOG_MISC, "*", "%s: CHAT(file) (%s!%s)", DCC_CONNECTFAILED2, nick,
-             from);
-      putlog(LOG_MISC, "*", "    (%s)", buf);
-      killsock(sock);
-    } else if (atoi(prt) < 1024 || atoi(prt) > 65535) {
+    if (atoi(prt) < 1024 || atoi(prt) > 65535) {
       /* Invalid port */
       if (!quiet_reject)
         dprintf(DP_HELP, "NOTICE %s :%s (invalid port)\n", nick,
                 DCC_CONNECTFAILED1);
       putlog(LOG_FILES, "*", "%s: %s!%s", DCC_REFUSED7, nick, from);
-
+      return 1;
+    }
+    i = new_dcc(&DCC_FILES_PASS, sizeof(struct file_info));
+    dcc[i].sock = open_telnet(i, ip, atoi(prt));
+    if (dcc[i].sock < 0) {
+      lostdcc(i);
+      if (!quiet_reject)
+        dprintf(DP_HELP, "NOTICE %s :%s (%s)\n", nick, DCC_CONNECTFAILED1, strerror(errno));
+      putlog(LOG_MISC, "*", "%s: CHAT(file) (%s!%s)", DCC_CONNECTFAILED2, nick,
+             from);
+      putlog(LOG_MISC, "*", "    (%s)", strerror(errno));
     } else {
-      i = new_dcc(&DCC_FILES_PASS, sizeof(struct file_info));
-      dcc[i].addr = my_atoul(ip);
-      dcc[i].port = atoi(prt);
-      dcc[i].sock = sock;
       strcpy(dcc[i].nick, u->handle);
       strcpy(dcc[i].host, from);
       dcc[i].status = STAT_ECHO;
@@ -1004,10 +997,10 @@ char *filesys_start(Function *global_funcs)
 {
   global = global_funcs;
 
-  module_register(MODULE_NAME, filesys_table, 2, 0);
-  if (!module_depend(MODULE_NAME, "eggdrop", 106, 0)) {
+  module_register(MODULE_NAME, filesys_table, 2, 1);
+  if (!module_depend(MODULE_NAME, "eggdrop", 108, 0)) {
     module_undepend(MODULE_NAME);
-    return "This module requires Eggdrop 1.6.0 or later.";
+    return "This module requires Eggdrop 1.8.0 or later.";
   }
   if (!(transfer_funcs = module_depend(MODULE_NAME, "transfer", 2, 0))) {
     module_undepend(MODULE_NAME);
