@@ -5,12 +5,10 @@
  *   'who' response
  *   user kickban, kick, op, deop
  *   idle kicking
- *
- * $Id: chan.c,v 1.136 2011/02/13 14:19:33 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -281,7 +279,7 @@ static int detect_chan_flood(char *floodnick, char *floodhost, char *from,
     if (!rfc_casecmp(chan->deopd, victim))
       return 0;
     else
-      strcpy(chan->deopd, victim);
+      strncpyz(chan->deopd, victim, sizeof chan->deopd);
   }
   chan->floodnum[which]++;
   if (chan->floodnum[which] >= thr) {   /* FLOOD */
@@ -1042,7 +1040,7 @@ static int got352or4(struct chanset_t *chan, char *user, char *host,
     m->flags = 0;               /* No flags for now */
     m->last = now;              /* Last time I saw him */
   }
-  strcpy(m->nick, nick);        /* Store the nick in list */
+  strncpyz(m->nick, nick, sizeof m->nick);        /* Store the nick in list */
   /* Store the userhost */
   simple_sprintf(m->userhost, "%s@%s", user, host);
   simple_sprintf(userhost, "%s!%s", nick, m->userhost);
@@ -1140,7 +1138,7 @@ static int got315(char *from, char *msg)
   if (!ismember(chan, botname)) {      /* Am I on the channel now?          */
     putlog(LOG_MISC | LOG_JOIN, chan->dname, "Oops, I'm not really on %s.",
            chan->dname);
-    clear_channel(chan, 1);
+    clear_channel(chan, CHAN_RESETALL);
     chan->status &= ~CHAN_ACTIVE;
 
     key = chan->channel.key[0] ? chan->channel.key : chan->key_prot;
@@ -1790,8 +1788,8 @@ static int gotjoin(char *from, char *chname)
         m->flags = 0;
         m->last = now;
         m->delay = 0L;
-        strcpy(m->nick, nick);
-        strcpy(m->userhost, uhost);
+        strncpyz(m->nick, nick, sizeof m->nick);
+        strncpyz(m->userhost, uhost, sizeof m->userhost);
         m->user = u;
         m->flags |= STOPWHO;
 
@@ -1972,7 +1970,7 @@ static int gotpart(char *from, char *msg)
   fixcolon(msg);
   chan = findchan(chname);
   if (chan && channel_inactive(chan)) {
-    clear_channel(chan, 1);
+    clear_channel(chan, CHAN_RESETALL);
     chan->status &= ~(CHAN_ACTIVE | CHAN_PEND);
     return 0;
   }
@@ -2006,7 +2004,7 @@ static int gotpart(char *from, char *msg)
              chan->dname);
     /* If it was me, all hell breaks loose... */
     if (match_my_nick(nick)) {
-      clear_channel(chan, 1);
+      clear_channel(chan, CHAN_RESETALL);
       chan->status &= ~(CHAN_ACTIVE | CHAN_PEND);
       if (!channel_inactive(chan)) {
 
@@ -2054,13 +2052,13 @@ static int gotkick(char *from, char *origmsg)
     else
       dprintf(DP_SERVER, "JOIN %s\n",
               chan->name[0] ? chan->name : chan->dname);
-    clear_channel(chan, 1);
+    clear_channel(chan, CHAN_RESETALL);
     return 0;                   /* rejoin if kicked before getting needed info <Wcc[08/08/02]> */
   }
   if (channel_active(chan)) {
     fixcolon(msg);
     u = get_user_by_host(from);
-    strcpy(uhost, from);
+    strncpyz(uhost, from, sizeof uhost);
     whodid = splitnick(&uhost);
     detect_chan_flood(whodid, uhost, from, chan, FLOOD_KICK, nick);
 
@@ -2102,7 +2100,7 @@ static int gotkick(char *from, char *origmsg)
       else
         dprintf(DP_SERVER, "JOIN %s\n",
                 chan->name[0] ? chan->name : chan->dname);
-      clear_channel(chan, 1);
+      clear_channel(chan, CHAN_RESETALL);
     } else {
       killmember(chan, nick);
       check_lonely_channel(chan);
@@ -2152,7 +2150,7 @@ static int gotnick(char *from, char *msg)
        */
       /* Compose a nick!user@host for the new nick */
       sprintf(s1, "%s!%s", msg, uhost);
-      strcpy(m->nick, msg);
+      strncpyz(m->nick, msg, sizeof m->nick);
       detect_chan_flood(msg, uhost, from, chan, FLOOD_NICK, NULL);
 
       if (!findchan_by_dname(chname)) {
@@ -2289,8 +2287,6 @@ static int gotmsg(char *from, char *msg)
   int ctcp_count = 0, ignoring;
   struct chanset_t *chan;
   struct userrec *u;
-  memberlist *m;
-  struct flag_record fr = { FR_GLOBAL | FR_CHAN, 0, 0, 0, 0, 0 };
 
   /* Only handle if message is to a channel, or to @#channel. */
   /* FIXME: Properly handle ovNotices (@+#channel), vNotices (+#channel), etc. */
@@ -2307,42 +2303,7 @@ static int gotmsg(char *from, char *msg)
   strcpy(uhost, from);
   nick = splitnick(&uhost);
   ignoring = match_ignore(from);
-  /* Only check if flood-ctcp is active */
-  if (flud_ctcp_thr && detect_avalanche(msg)) {
-    u = get_user_by_host(from);
-    get_user_flagrec(u, &fr, chan->dname);
-    m = ismember(chan, nick);
-    /* Discard -- kick user if it was to the channel */
-    if (m && (me_op(chan) || (me_halfop(chan) && !chan_hasop(m))) &&
-        !chan_sentkick(m) && !chan_friend(fr) && !glob_friend(fr) &&
-        !(channel_dontkickops(chan) && (chan_op(fr) || (glob_op(fr) &&
-        !chan_deop(fr)))) && !(use_exempts && ban_fun &&
-        (u_match_mask(global_exempts, from) ||
-        u_match_mask(chan->exempts, from)))) {
-      if (ban_fun) {
-        check_exemptlist(chan, from);
-        u_addban(chan, quickban(chan, uhost), botnetnick, IRC_FUNKICK,
-                 now + (60 * chan->ban_time), 0);
-      }
-      if (kick_fun) {
-        /* This can induce kickflood - arthur2 */
-        dprintf(DP_SERVER, "KICK %s %s :%s\n", chan->name, nick, IRC_FUNKICK);
-        m->flags |= SENTKICK;
-      }
-    }
-    if (!ignoring) {
-      putlog(LOG_MODES, "*", "Avalanche from %s!%s in %s - ignoring",
-             nick, uhost, chan->dname);
-      p = strchr(uhost, '@');
-      if (p)
-        p++;
-      else
-        p = uhost;
-      simple_sprintf(buf2, "*!*@%s", p);
-      addignore(buf2, botnetnick, "ctcp avalanche", now + (60 * ignore_time));
-    }
-    return 0;
-  }
+
   /* Check for CTCP: */
   ctcp_reply[0] = 0;
   p = strchr(msg, 1);
@@ -2354,7 +2315,7 @@ static int gotmsg(char *from, char *msg)
     if (*p == 1) {
       *p = 0;
       ctcp = buf2;
-      strcpy(ctcp, p1);
+      strncpyz(buf2, p1, sizeof buf2);
       strcpy(p1 - 1, p + 1);
       detect_chan_flood(nick, uhost, from, chan, strncmp(ctcp, "ACTION ", 7) ?
                         FLOOD_CTCP : FLOOD_PRIVMSG, NULL);
@@ -2447,9 +2408,7 @@ static int gotnotice(char *from, char *msg)
   char *to, *realto, *nick, buf2[512], *p, *p1, buf[512], *uhost = buf;
   char *ctcp, *code;
   struct userrec *u;
-  memberlist *m;
   struct chanset_t *chan;
-  struct flag_record fr = { FR_GLOBAL | FR_CHAN, 0, 0, 0, 0, 0 };
   int ignoring;
 
   if (!strchr(CHANMETA "@", *msg))
@@ -2464,30 +2423,7 @@ static int gotnotice(char *from, char *msg)
   strcpy(uhost, from);
   nick = splitnick(&uhost);
   u = get_user_by_host(from);
-  if (flud_ctcp_thr && detect_avalanche(msg)) {
-    get_user_flagrec(u, &fr, chan->dname);
-    m = ismember(chan, nick);
-    /* Discard -- kick user if it was to the channel */
-    if (me_op(chan) && m && !chan_sentkick(m) && !chan_friend(fr) &&
-        !glob_friend(fr) && !(channel_dontkickops(chan) && (chan_op(fr) ||
-        (glob_op(fr) && !chan_deop(fr)))) && !(use_exempts && ban_fun &&
-        (u_match_mask(global_exempts, from) ||
-        u_match_mask(chan->exempts, from)))) {
-      if (ban_fun) {
-        check_exemptlist(chan, from);
-        u_addban(chan, quickban(chan, uhost), botnetnick,
-                 IRC_FUNKICK, now + (60 * chan->ban_time), 0);
-      }
-      if (kick_fun) {
-        /* This can induce kickflood - arthur2 */
-        dprintf(DP_SERVER, "KICK %s %s :%s\n", chan->name, nick, IRC_FUNKICK);
-        m->flags |= SENTKICK;
-      }
-    }
-    if (!ignoring)
-      putlog(LOG_MODES, "*", "Avalanche from %s", from);
-    return 0;
-  }
+
   /* Check for CTCP: */
   p = strchr(msg, 1);
   while (p && *p) {
