@@ -1,12 +1,10 @@
 /*
  * irc.c -- part of irc.mod
  *   support for channels within the bot
- *
- * $Id: irc.c,v 1.120 2011/02/13 14:19:33 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -48,7 +46,7 @@ static int max_bans = 20;       /* Modified by net-type 1-4 */
 static int max_exempts = 20;    /* Modified by net-type 1-4 */
 static int max_invites = 20;    /* Modified by net-type 1-4 */
 static int max_modes = 20;      /* Modified by net-type 1-4 */
-static int bounce_bans = 1;
+static int bounce_bans = 0;
 static int bounce_exempts = 0;
 static int bounce_invites = 0;
 static int bounce_modes = 0;
@@ -62,8 +60,6 @@ static int use_354 = 0;         /* Use ircu's short 354 /who responses. */
 static int kick_method = 1;     /* How many kicks does the IRC network support
                                  * at once? Use 0 for as many as possible.
                                  * (Ernst 18/3/1998) */
-static int kick_fun = 0;
-static int ban_fun = 0;
 static int keepnick = 1;        /* Keep nick */
 static int prevent_mixing = 1;  /* Prevent mixing old/new modes */
 static int rfc_compliant = 1;   /* Value depends on net-type. */
@@ -174,7 +170,7 @@ static void punish_badguy(struct chanset_t *chan, char *whobad,
     else {
       strcpy(s1, whobad);
       maskaddr(s1, s, chan->ban_type);
-      strcpy(s1, badnick);
+      strncpyz(s1, badnick, sizeof s1);
       /* If that handle exists use "badX" (where X is an increasing number)
        * instead.
        */
@@ -412,9 +408,8 @@ static int any_ops(struct chanset_t *chan)
 
 /* Reset channel information.
  */
-static void reset_chan_info(struct chanset_t *chan, int reset)
+void reset_chan_info(struct chanset_t *chan, int reset)
 {
-  char beI[4] = "\0";
   /* Leave the channel if we aren't supposed to be there */
   if (channel_inactive(chan)) {
     dprintf(DP_MODE, "PART %s\n", chan->name);
@@ -428,20 +423,18 @@ static void reset_chan_info(struct chanset_t *chan, int reset)
   clear_channel(chan, reset);
   if ((reset & CHAN_RESETBANS) && !(chan->status & CHAN_ASKEDBANS)) {
     chan->status |= CHAN_ASKEDBANS;
-    strcat(beI, "b");
+    dprintf(DP_MODE, "MODE %s +b\n", chan->name);
   }
   if ((reset & CHAN_RESETEXEMPTS) &&
       !(chan->ircnet_status & CHAN_ASKED_EXEMPTS) && (use_exempts == 1)) {
     chan->ircnet_status |= CHAN_ASKED_EXEMPTS;
-    strcat(beI, "e");
+    dprintf(DP_MODE, "MODE %s +e\n", chan->name);
   }
   if ((reset & CHAN_RESETINVITED) &&
       !(chan->ircnet_status & CHAN_ASKED_INVITED) && (use_invites == 1)) {
     chan->ircnet_status |= CHAN_ASKED_INVITED;
-    strcat(beI, "I");
+    dprintf(DP_MODE, "MODE %s +I\n", chan->name);
   }
-  if (beI[0])
-    dprintf(DP_MODE, "MODE %s +%s\n", chan->name, beI);
   if (reset & CHAN_RESETMODES) {
     /* done here to keep expmem happy, as this is accounted in
        irc.mod, not channels.mod where clear_channel() resides */
@@ -857,7 +850,7 @@ static int check_tcl_pub(char *nick, char *from, char *chname, char *msg)
   char buf[512], *args = buf, *cmd, host[161], *hand;
   struct userrec *u;
 
-  strcpy(args, msg);
+  strncpyz(buf, msg, sizeof buf);
   cmd = newsplit(&args);
   simple_sprintf(host, "%s!%s", nick, from);
   u = get_user_by_host(host);
@@ -937,8 +930,6 @@ static tcl_ints myints[] = {
   {"mode-buf-length", &mode_buf_len,    0},
   {"use-354",         &use_354,         0},
   {"kick-method",     &kick_method,     0},
-  {"kick-fun",        &kick_fun,        0},
-  {"ban-fun",         &ban_fun,         0},
   {"invite-key",      &invite_key,      0},
   {"no-chanrec-info", &no_chanrec_info, 0},
   {"max-bans",        &max_bans,        0},
@@ -1050,10 +1041,10 @@ static void do_nettype()
     use_354 = 0;
     use_exempts = 1;
     use_invites = 1;
-    max_bans = 30;
-    max_exempts = 30;
-    max_invites = 30;
-    max_modes = 30;
+    max_bans = 64;
+    max_exempts = 64;
+    max_invites = 64;
+    max_modes = 64;
     rfc_compliant = 1;
     include_lk = 1;
     break;
@@ -1136,7 +1127,7 @@ static char *irc_close()
   dprintf(DP_MODE, "JOIN 0\n");
 
   for (chan = chanset; chan; chan = chan->next)
-    clear_channel(chan, 1);
+    clear_channel(chan, CHAN_RESETALL);
   del_bind_table(H_topc);
   del_bind_table(H_splt);
   del_bind_table(H_sign);
@@ -1205,6 +1196,7 @@ static Function irc_table[] = {
   (Function) me_voice,
   /* 24 - 27 */
   (Function) getchanmode,
+  (Function) reset_chan_info
 };
 
 char *irc_start(Function *global_funcs)
@@ -1213,10 +1205,10 @@ char *irc_start(Function *global_funcs)
 
   global = global_funcs;
 
-  module_register(MODULE_NAME, irc_table, 1, 4);
-  if (!module_depend(MODULE_NAME, "eggdrop", 106, 20)) {
+  module_register(MODULE_NAME, irc_table, 1, 5);
+  if (!module_depend(MODULE_NAME, "eggdrop", 108, 0)) {
     module_undepend(MODULE_NAME);
-    return "This module requires Eggdrop 1.6.20 or later.";
+    return "This module requires Eggdrop 1.8.0 or later.";
   }
   if (!(server_funcs = module_depend(MODULE_NAME, "server", 1, 0))) {
     module_undepend(MODULE_NAME);
