@@ -1,12 +1,10 @@
 /*
  * msgcmds.c -- part of irc.mod
  *   all commands entered via /MSG
- *
- * $Id: msgcmds.c,v 1.59 2011/02/13 14:19:34 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -113,7 +111,7 @@ static int msg_hello(char *nick, char *h, struct userrec *u, char *p)
       if (p1 == NULL)
         s1[0] = 0;
       else
-        strcpy(s1, p1);
+        strncpyz(s1, p1, sizeof s1);
     }
   }
   return 1;
@@ -173,7 +171,7 @@ static int msg_ident(char *nick, char *host, struct userrec *u, char *par)
   }
   pass = newsplit(&par);
   if (!par[0])
-    strcpy(who, nick);
+    strncpyz(who, nick, sizeof who);
   else {
     strncpy(who, par, NICKMAX);
     who[NICKMAX] = 0;
@@ -795,7 +793,7 @@ static int msg_status(char *nick, char *host, struct userrec *u, char *par)
 #ifdef HAVE_UNAME
   struct utsname un;
 
-  if (!uname(&un) < 0) {
+  if (uname(&un) < 0) {
 #endif
     vers_t = " ";
     uni_t  = "*unknown*";
@@ -1075,6 +1073,9 @@ static int msg_go(char *nick, char *host, struct userrec *u, char *par)
 static int msg_jump(char *nick, char *host, struct userrec *u, char *par)
 {
   char *s;
+#ifdef TLS
+  char *sport;
+#endif
   int port;
 
   if (match_my_nick(nick))
@@ -1090,11 +1091,26 @@ static int msg_jump(char *nick, char *host, struct userrec *u, char *par)
   if (u_pass_match(u, s)) {
     if (par[0]) {
       s = newsplit(&par);
+#ifdef TLS
+      sport = newsplit(&par);
+      if (*sport == '+')
+        use_ssl = 1;
+      else
+        use_ssl = 0;
+      port = atoi(sport);
+      if (!port) {
+        port = default_port;
+        use_ssl = 0;
+      }
+      putlog(LOG_CMDS, "*", "(%s!%s) !%s! JUMP %s %s%d %s", nick, host,
+             u->handle, s, use_ssl ? "+" : "", port, par);
+#else
       port = atoi(newsplit(&par));
       if (!port)
         port = default_port;
       putlog(LOG_CMDS, "*", "(%s!%s) !%s! JUMP %s %d %s", nick, host,
              u->handle, s, port, par);
+#endif
       strcpy(newserver, s);
       newserverport = port;
       strcpy(newserverpass, par);
@@ -1131,6 +1147,7 @@ static cmd_t C_msg[] = {
   {"op",      "",    (IntFunc) msg_op,      NULL},
   {"halfop",  "",    (IntFunc) msg_halfop,  NULL},
   {"pass",    "",    (IntFunc) msg_pass,    NULL},
+  {"password","",    (IntFunc) msg_pass,    NULL},
   {"rehash",  "m",   (IntFunc) msg_rehash,  NULL},
   {"reset",   "m",   (IntFunc) msg_reset,   NULL},
   {"save",    "m",   (IntFunc) msg_save,    NULL},
