@@ -1,11 +1,9 @@
 /*
  * tclirc.c -- part of irc.mod
- *
- * $Id: tclirc.c,v 1.59 2011/02/13 14:19:34 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -530,7 +528,7 @@ static int tcl_getchanidle STDVAR
     Tcl_AppendResult(irp, s, NULL);
     return TCL_OK;
   }
-  Tcl_AppendResult(irp, "0", NULL);
+  Tcl_AppendResult(irp, "-1", NULL);
   return TCL_OK;
 }
 
@@ -978,8 +976,8 @@ static int tcl_putkick STDVAR
       continue;                 /* Skip non-existant nicks */
     m->flags |= SENTKICK;       /* Mark as pending kick */
     if (kicknick[0])
-      strcat(kicknick, ",");
-    strcat(kicknick, nick);     /* Add to local queue */
+      strncat(kicknick, ",", sizeof kicknick - strlen(kicknick) - 1);
+    strncat(kicknick, nick, sizeof kicknick - strlen(kicknick) - 1);     /* Add to local queue */
     k++;
 
     /* Check if we should send the kick command yet */
