@@ -1,11 +1,9 @@
 /*
  * module.h
- *
- * $Id: module.h,v 1.97 2011/07/09 15:07:48 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -43,7 +41,6 @@
  * held responsible for mental break-downs caused by this file <G>
  */
 
-#undef nstrdup
 #undef nmalloc
 #undef nfree
 #undef nrealloc
@@ -80,9 +77,6 @@
 #ifdef egg_strncasecmp
 #  undef egg_strncasecmp
 #endif
-#ifdef egg_strdup
-#  undef egg_strdup
-#endif
 
 #if defined (__CYGWIN__) && !defined(STATIC)
 #  define EXPORT_SCOPE  __declspec(dllexport)
@@ -196,22 +190,25 @@
 #define egg_list_append ((int (*) ( struct list_type **, struct list_type *))global[74])
 #define egg_list_contains ((int (*) (struct list_type *, struct list_type *))global[75])
 /* 76 - 79 */
-#define answer ((int (*) (int,char *,unsigned long *,unsigned short *,int))global[76])
-#define getmyip ((IP (*) (void))global[77])
-#define neterror ((void (*) (char *))global[78])
+#define answer ((int (*) (int, sockname_t *, unsigned short *, int))global[76])
+#define getvhost ((void (*) (sockname_t *, int))global[77])
+/* was neterror() */
+#ifdef TLS
+#  define ssl_handshake ((int (*)(int,int,int,int,char *,IntFunc))global[78])
+#endif
 #define tputs ((void (*) (int, char *,unsigned int))global[79])
 /* 80 - 83 */
 #define new_dcc ((int (*) (struct dcc_table *, int))global[80])
 #define lostdcc ((void (*) (int))global[81])
-#define getsock ((int (*) (int))global[82])
+#define getsock ((int (*) (int, int))global[82])
 #define killsock ((void (*) (int))global[83])
 /* 84 - 87 */
 #define open_listen ((int (*) (int *))global[84])
-#define open_telnet_dcc ((int (*) (int,char *,char *))global[85])
+#define getdccaddr ((int (*) (sockname_t *, char *, socklen_t))global[85])
 #define get_data_ptr(x) ((void *(*)(int,char*,int))global[86])(x,__FILE__,__LINE__)
-#define open_telnet ((int (*) (char *, int))global[87])
+#define open_telnet ((int (*) (int, char *, int))global[87])
 /* 88 - 91 */
-#define check_tcl_event ((void * (*) (const char *))global[88])
+#define check_tcl_event ((void (*) (const char *))global[88])
 #define my_memcpy ((void * (*) (void *, const void *, size_t))global[89])
 #define my_atoul ((IP(*)(char *))global[90])
 #define my_strcpy ((int (*)(char *, const char *))global[91])
@@ -229,21 +226,32 @@
 #define max_dcc (*(int *)global[100])
 #define require_p (*(int *)global[101])
 #define ignore_time (*(int *)(global[102]))
-/* was use_console_r <Wcc[02/02/03]> */
+#ifdef TLS
+#  define dcc_fingerprint ((int (*) (int))global[103])
+#endif
 /* 104 - 107 */
 #define reserved_port_min (*(int *)(global[104]))
 #define reserved_port_max (*(int *)(global[105]))
 #define raw_log (*(int *)(global[106]))
 #define noshare (*(int *)(global[107]))
 /* 108 - 111 */
+#ifdef TLS
+#  define tls_vfybots (*(int *)(global[108]))
+#else
 /* 108: gban_total -- UNUSED (Eule) */
+#endif
 #define make_userfile (*(int*)global[109])
 #define default_flags (*(int*)global[110])
 #define dcc_total (*(int*)global[111])
 /* 112 - 115 */
-#define tempdir ((char *)(global[112]))
-#define natip ((char *)(global[113]))
-#define hostname ((char *)(global[114]))
+/* was tempdir -- UNUSED */
+#ifdef TLS
+#  define tls_vfyclients (*(int *)(global[113]))
+#  define tls_vfydcc (*(int *)(global[114]))
+#else
+/* was natip -- UNUSED */
+/* was hostname -- UNUSED */
+#endif
 #define origbotname ((char *)(global[115]))
 /* 116 - 119 */
 #define botuser ((char *)(global[116]))
@@ -371,8 +379,8 @@
 #define sanitycheck_dcc ((int (*)(char *, char *, char *, char *))global[214])
 #define isowner ((int (*)(char *))global[215])
 /* 216 - 219 */
-/* 216: min_dcc_port -- UNUSED (guppy) */
-/* 217: max_dcc_port -- UNUSED (guppy) */
+#define fcopyfile ((int (*) (FILE *, char *))global[216])
+#define copyfilef ((int (*) (char *, FILE *))global[217])
 #define rfc_casecmp ((int(*)(char *, char *))(*(Function**)(global[218])))
 #define rfc_ncasecmp ((int(*)(char *, char *, int *))(*(Function**)(global[219])))
 /* 220 - 223 */
@@ -404,16 +412,16 @@
 #else
 #  define Assert(expr) do {} while (0)
 #endif
-#define allocsock ((int(*)(int sock,int options))global[234])
-#define call_hostbyip ((void(*)(IP, char *, int))global[235])
+#define allocsock ((int(*)(int sock, int options))global[234])
+#define call_hostbyip ((void(*)(sockname_t *, char *, int))global[235])
 /* 236 - 239 */
-#define call_ipbyhost ((void(*)(char *, IP, int))global[236])
-#define iptostr ((char *(*)(IP))global[237])
+#define call_ipbyhost ((void(*)(char *, sockname_t *, int))global[236])
+#define iptostr ((char *(*)(struct sockaddr *))global[237])
 #define DCC_DNSWAIT (*(struct dcc_table *)(global[238]))
-#define hostsanitycheck_dcc ((int(*)(char *, char *, IP, char *, char *))global[239])
+#define hostsanitycheck_dcc ((int(*)(char *, char *, sockname_t *, char *, char *))global[239])
 /* 240 - 243 */
 #define dcc_dnsipbyhost ((void (*)(char *))global[240])
-#define dcc_dnshostbyip ((void (*)(IP))global[241])
+#define dcc_dnshostbyip ((void (*)(sockname_t *))global[241])
 #define changeover_dcc ((void (*)(int, struct dcc_table *, int))global[242])
 #define make_rand_str ((void (*) (char *, int))global[243])
 /* 244 - 247 */
@@ -438,7 +446,7 @@
 #define tandbot (*(tand_t **)(global[259]))
 /* 260 - 263 */
 #define party (*(party_t **)(global[260]))
-#define open_address_listen ((int (*)(IP addr, int *port))global[261])
+#define open_address_listen ((int (*)(sockname_t *, int *))global[261])
 #define str_escape ((char *(*)(const char *, const char, const char))global[262])
 #define strchr_unescape ((char *(*)(char *, const char, register const char))global[263])
 /* 264 - 267 */
@@ -473,10 +481,12 @@
 /* 284 - 287 */
 #define quiet_reject (*(int *)(global[284]))
 #define file_readable ((int (*) (char *))global[285])
-/* IPv6 leftovers: 286 */
-/* IPv6 leftovers: 287 */
+#define setsockname ((int (*)(sockname_t *, char *, int, int))global[286])
+#define open_telnet_raw ((int (*)(int, sockname_t *))global[287])
 /* 288 - 291 */
-/* IPv6 leftovers: 288 */
+#ifdef IPV6
+# define pref_af (*(int *)(global[288]))
+#endif
 #define strip_mirc_codes ((void (*)(int, char *))global[289])
 #define check_ansi ((int (*) (char *))global[290])
 #define oatoi ((int (*) (const char *))global[291])
@@ -489,7 +499,12 @@
 #define check_conflags ((int (*) (struct flag_record *, int))global[296])
 #define increase_socks_max ((int (*)(void))global[297])
 #define LOG_TS ((char *)(global[298]))
-#define nstrdup(x) (((char *(*)())global[299])((x),MODULE_NAME,__FILE__,__LINE__))
+#define tcl_resultempty ((int (*)(void))global[299])
+/* 300 - 303 */
+#define tcl_resultint ((int (*)(void))global[300])
+#define tcl_resultstring ((const char *(*)(void))global[301])
+#define getdccfamilyaddr ((int (*) (sockname_t *, char *, socklen_t, int))global[302])
+
 
 /* hostmasking */
 #define maskhost(a,b) maskaddr((a),(b),3)
