@@ -9,11 +9,9 @@
  * 1.1     1997-08-05      Add nick->handle lookup for NICK's. [BB]
  * 1.2     1997-08-20      Minor fixes. [BB]
  * 1.2a    1997-08-24      Minor fixes. [BB]
- *
- * $Id: seen.c,v 1.42 2011/02/13 14:19:34 simple Exp $
  */
 /*
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -297,7 +295,7 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand,
     /* "your admin" */
     if (!egg_strcasecmp(word1, "owner") || !egg_strcasecmp(word1, "admin")) {
       if (admin[0]) {
-        strcpy(word2, admin);
+        strncpyz(word2, admin, sizeof word2);
         wordshift(whotarget, word2);
         strcat(whoredirect, "My owner is ");
         strcat(whoredirect, whotarget);
@@ -507,20 +505,24 @@ targetcont:
 static char fixit[512];
 static char *fixnick(char *nick)
 {
-  strcpy(fixit, nick);
-  strcat(fixit, "'");
-  switch (nick[strlen(nick) - 1]) {
-  case 's':
-  case 'S':
-  case 'x':
-  case 'X':
-  case 'z':
-  case 'Z':
-    break;
-  default:
-    strcat(fixit, "s");
-    break;
-  }
+  if (!nick)
+    return NULL;
+  if (!nick[0])
+    fixit[0] = '\0';
+  else
+    switch (nick[strlen(nick) - 1]) {
+    case 's':
+    case 'S':
+    case 'x':
+    case 'X':
+    case 'z':
+    case 'Z':
+      egg_snprintf(fixit, sizeof fixit, "%s'", nick);
+      break;
+    default:
+      egg_snprintf(fixit, sizeof fixit, "%s's", nick);
+      break;
+    }
   return fixit;
 }
 
@@ -652,10 +654,10 @@ char *seen_start(Function *egg_func_table)
 {
   global = egg_func_table;
 
-  module_register(MODULE_NAME, seen_table, 2, 0);
-  if (!module_depend(MODULE_NAME, "eggdrop", 106, 0)) {
+  module_register(MODULE_NAME, seen_table, 2, 1);
+  if (!module_depend(MODULE_NAME, "eggdrop", 108, 0)) {
     module_undepend(MODULE_NAME);
-    return "This module requires Eggdrop 1.6.0 or later.";
+    return "This module requires Eggdrop 1.8.0 or later.";
   }
   add_builtins(H_load, seen_load);
   add_builtins(H_dcc, seen_dcc);
