@@ -1,12 +1,10 @@
 /*
  * cmdsserv.c -- part of server.mod
  *   handles commands from a user via dcc that cause server interaction
- *
- * $Id: cmdsserv.c,v 1.27 2011/02/13 14:19:34 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -37,12 +35,22 @@ static void cmd_servers(struct userrec *u, int idx, char *par)
     i = 0;
     for (; x; x = x->next) {
       if ((i == curserv) && realservername)
-        egg_snprintf(s, sizeof s, "  %s:%d (%s) <- I am here", x->name,
-                     x->port ? x->port : default_port, realservername);
+#ifdef TLS
+        egg_snprintf(s, sizeof s, "  [%s]:%s%d (%s) <- I am here", x->name,
+                     x->ssl ? "+" : "", x->port ? x->port : default_port,
+                     realservername);
       else
-        egg_snprintf(s, sizeof s, "  %s:%d %s", x->name,
+        egg_snprintf(s, sizeof s, "  [%s]:%s%d %s", x->name, x->ssl ? "+" : "",
                      x->port ? x->port : default_port,
                      (i == curserv) ? "<- I am here" : "");
+#else
+        egg_snprintf(s, sizeof s, "  [%s]:%d (%s) <- I am here", x->name,
+                     x->port ? x->port : default_port, realservername);
+      else
+        egg_snprintf(s, sizeof s, "  [%s]:%d %s", x->name,
+                     x->port ? x->port : default_port,
+                     (i == curserv) ? "<- I am here" : "");
+#endif
       dprintf(idx, "%s\n", s);
       i++;
     }
@@ -67,15 +75,36 @@ static void cmd_dump(struct userrec *u, int idx, char *par)
 static void cmd_jump(struct userrec *u, int idx, char *par)
 {
   char *other;
+  char *sport;
   int port;
 
   if (par[0]) {
     other = newsplit(&par);
-    port = atoi(newsplit(&par));
+    sport = newsplit(&par);
+    if (*sport == '+') {
+#ifdef TLS
+      use_ssl = 1;
+    }
+    else
+      use_ssl = 0;
+    port = atoi(sport);
+    if (!port) {
+      port = default_port;
+      use_ssl = 0;
+    }
+    putlog(LOG_CMDS, "*", "#%s# jump %s %s%d %s", dcc[idx].nick, other,
+           use_ssl ? "+" : "", port, par);
+#else
+    putlog(LOG_MISC, "*", "Error: Attempted to jump to SSL-enabled \
+server, but Eggdrop was not compiled with SSL libraries. Skipping...");
+      return;
+    }
+    port = atoi(sport);
     if (!port)
       port = default_port;
     putlog(LOG_CMDS, "*", "#%s# jump %s %d %s", dcc[idx].nick, other,
            port, par);
+#endif
     strncpyz(newserver, other, sizeof newserver);
     newserverport = port;
     strncpyz(newserverpass, par, sizeof newserverpass);
