@@ -1,12 +1,10 @@
 /*
  * server.c -- part of server.mod
  *   basic irc server support
- *
- * $Id: server.c,v 1.140 2011/10/25 16:45:08 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,6 +24,7 @@
 #define MODULE_NAME "server"
 #define MAKING_SERVER
 
+#include <errno.h>
 #include "src/mod/module.h"
 #include "server.h"
 
@@ -62,10 +61,8 @@ static time_t lastpingcheck;    /* set when i unidle myself, cleared when
 static time_t server_online;    /* server connection time */
 static time_t server_cycle_wait;        /* seconds to wait before
                                          * re-beginning the server list */
-static char botrealname[121];   /* realname of bot */
-static int min_servs;           /* minimum number of servers to be around */
+static char botrealname[81];    /* realname of bot */
 static int server_timeout;      /* server timeout for connecting */
-static int never_give_up;       /* never give up when connecting to servers? */
 static struct server_list *serverlist;  /* old-style queue, still used by
                                          * server list */
 static int cycle_time;          /* cycle time till next server connect */
@@ -97,6 +94,14 @@ static int kick_method;
 static int optimize_kicks;
 static int msgrate;             /* Number of seconds between sending
                                  * queued lines to server. */
+#ifdef TLS
+static int use_ssl;		/* Use SSL for the next server connection? */
+static int tls_vfyserver;       /* Certificate validation mode for servrs  */
+#endif
+
+#ifndef TLS
+static char sslserver = 0;
+#endif
 
 static p_tcl_bind_list H_wall, H_raw, H_notc, H_msgm, H_msg, H_flud, H_ctcr,
                        H_ctcp, H_out;
@@ -953,46 +958,46 @@ static void queue_server(int which, char *msg, int len)
 static void add_server(const char *ss)
 {
   struct server_list *x, *z;
-  char *p, *q;
+  char name[256] = "", port[11] = "", pass[121] = "";
 
   for (z = serverlist; z && z->next; z = z->next);
-  while (ss) {
-    p = strchr(ss, ',');
-    if (p)
-      *p++ = 0;
-    x = nmalloc(sizeof(struct server_list));
 
-    x->next = 0;
-    x->realname = 0;
-    x->port = 0;
-    if (z)
-      z->next = x;
-    else
-      serverlist = x;
-    z = x;
-    q = strchr(ss, ':');
-    if (!q) {
-      x->port = default_port;
-      x->pass = 0;
-      x->name = nmalloc(strlen(ss) + 1);
-      strcpy(x->name, ss);
-    } else {
-      *q++ = 0;
-      x->name = nmalloc(q - ss);
-      strcpy(x->name, ss);
-      ss = q;
-      q = strchr(ss, ':');
-      if (!q) {
-        x->pass = 0;
-      } else {
-        *q++ = 0;
-        x->pass = nmalloc(strlen(q) + 1);
-        strcpy(x->pass, q);
-      }
-      x->port = atoi(ss);
-    }
-    ss = p;
+  /* Allow IPv6 and IPv4-mapped addresses in [] */
+  if (!sscanf(ss, "[%255[0-9.A-F:a-f]]:%10[+0-9]:%120[^\r\n]", name, port, pass) &&
+      !sscanf(ss, "%255[^:]:%10[+0-9]:%120[^\r\n]", name, port, pass))
+    return;
+
+#ifndef TLS
+  if (port[0] == '+') {
+    putlog(LOG_MISC, "*", "ERROR: Attempted to add SSL-enabled server, but \
+Eggdrop was not compiled with SSL libraries. Skipping...");
+    sslserver = 1;
+    return;
   }
+#endif
+
+  x = nmalloc(sizeof(struct server_list));
+  x->next = 0;
+  x->realname = 0;
+  x->port = default_port;
+  if (z)
+    z->next = x;
+  else
+    serverlist = x;
+  z = x;
+
+  x->name = nmalloc(strlen(name) + 1);
+  strcpy(x->name, name);
+  if (pass[0]) {
+    x->pass = nmalloc(strlen(pass) + 1);
+    strcpy(x->pass, pass);
+  } else
+    x->pass = NULL;
+  if (port[0])
+    x->port = atoi(port);
+#ifdef TLS
+  x->ssl = (port[0] == '+') ? 1 : 0;
+#endif
 }
 
 
@@ -1030,10 +1035,16 @@ static void next_server(int *ptr, char *serv, unsigned int *port, char *pass)
       if (x->port == *port) {
         if (!egg_strcasecmp(x->name, serv)) {
           *ptr = i;
+#ifdef TLS
+            x->ssl = use_ssl;
+#endif
           return;
         } else if (x->realname && !egg_strcasecmp(x->realname, serv)) {
           *ptr = i;
           strncpyz(serv, x->realname, sizeof serv);
+#ifdef TLS
+          use_ssl = x->ssl;
+#endif
           return;
         }
       }
@@ -1052,6 +1063,9 @@ static void next_server(int *ptr, char *serv, unsigned int *port, char *pass)
       strcpy(x->pass, pass);
     } else
       x->pass = NULL;
+#ifdef TLS
+    x->ssl = use_ssl;
+#endif
     egg_list_append((struct list_type **) (&serverlist),
                     (struct list_type *) x);
     *ptr = i;
@@ -1073,6 +1087,9 @@ static void next_server(int *ptr, char *serv, unsigned int *port, char *pass)
     x = serverlist;
     *ptr = 0;
   }                             /* Start over at the beginning */
+#ifdef TLS
+  use_ssl = x->ssl;
+#endif
   strcpy(serv, x->name);
   *port = x->port ? x->port : default_port;
   if (x->pass)
@@ -1196,7 +1213,7 @@ static char *get_altbotnick(void)
 {
   /* A random-number nick? */
   if (strchr(altnick, '?')) {
-    if (!raltnick[0]) {
+    if (!raltnick[0] && !wild_match(altnick, botname)) {
       strncpyz(raltnick, altnick, NICKLEN);
       rand_nick(raltnick);
     }
@@ -1223,13 +1240,20 @@ static char *traced_serveraddress(ClientData cdata, Tcl_Interp *irp,
   if (server_online) {
     int servidx = findanyidx(serv);
 
+#ifdef TLS
+    simple_sprintf(s, "%s:%s%u", dcc[servidx].host, dcc[servidx].ssl ? "+" : "",
+                   dcc[servidx].port);
+#else
     simple_sprintf(s, "%s:%u", dcc[servidx].host, dcc[servidx].port);
+#endif
   } else
     s[0] = 0;
   Tcl_SetVar2(interp, name1, name2, s, TCL_GLOBAL_ONLY);
   if (flags & TCL_TRACE_UNSETS)
     Tcl_TraceVar(irp, name1, TCL_TRACE_READS | TCL_TRACE_WRITES |
                  TCL_TRACE_UNSETS, traced_serveraddress, cdata);
+  if (flags & TCL_TRACE_WRITES)
+    return "read-only variable";
   return NULL;
 }
 
@@ -1250,6 +1274,8 @@ static char *traced_server(ClientData cdata, Tcl_Interp *irp,
   if (flags & TCL_TRACE_UNSETS)
     Tcl_TraceVar(irp, name1, TCL_TRACE_READS | TCL_TRACE_WRITES |
                  TCL_TRACE_UNSETS, traced_server, cdata);
+  if (flags & TCL_TRACE_WRITES)
+    return "read-only variable";
   return NULL;
 }
 
@@ -1265,6 +1291,8 @@ static char *traced_botname(ClientData cdata, Tcl_Interp *irp,
   if (flags & TCL_TRACE_UNSETS)
     Tcl_TraceVar(irp, name1, TCL_TRACE_READS | TCL_TRACE_WRITES |
                  TCL_TRACE_UNSETS, traced_botname, cdata);
+  if (flags & TCL_TRACE_WRITES)
+    return "read-only variable";
   return NULL;
 }
 
@@ -1357,11 +1385,9 @@ static tcl_coups my_tcl_coups[] = {
 };
 
 static tcl_ints my_tcl_ints[] = {
-  {"servlimit",         &min_servs,                 0},
   {"server-timeout",    &server_timeout,            0},
   {"lowercase-ctcp",    &lowercase_ctcp,            0},
   {"server-online",     (int *) &server_online,     2},
-  {"never-give-up",     &never_give_up,             0},
   {"keep-nick",         &keepnick,                  0},
   {"check-stoned",      &check_stoned,              0},
   {"serverror-quit",    &serverror_quit,            0},
@@ -1385,6 +1411,9 @@ static tcl_ints my_tcl_ints[] = {
   {"stack-limit",       &stack_limit,               0},
   {"exclusive-binds",   &exclusive_binds,           0},
   {"msg-rate",          &msgrate,                   0},
+#ifdef TLS
+  {"ssl-verify-server", &tls_vfyserver,             0},
+#endif
   {NULL,                NULL,                       0}
 };
 
@@ -1410,9 +1439,18 @@ static char *tcl_eggserver(ClientData cdata, Tcl_Interp *irp,
     /* Create server list */
     Tcl_DStringInit(&ds);
     for (q = serverlist; q; q = q->next) {
-      egg_snprintf(x, sizeof x, "%s:%d%s%s %s", q->name,
+#ifdef TLS
+      egg_snprintf(x, sizeof x, "%s%s%s:%s%d%s%s %s", strchr(q->name, ':') ?
+                   "[" : "", q->name, strchr(q->name, ':') ? "]" : "",
+                   q->ssl ? "+" : "", q->port ? q->port : default_port,
+                   q->pass ? ":" : "", q->pass ? q->pass : "",
+                   q->realname ? q->realname : "");
+#else
+      egg_snprintf(x, sizeof x, "%s%s%s:%d%s%s %s", strchr(q->name, ':') ?
+                   "[" : "", q->name, strchr(q->name, ':') ? "]" : "",
                    q->port ? q->port : default_port, q->pass ? ":" : "",
                    q->pass ? q->pass : "", q->realname ? q->realname : "");
+#endif
       Tcl_DStringAppendElement(&ds, x);
     }
     slist = Tcl_DStringValue(&ds);
@@ -1470,16 +1508,29 @@ static int ctcp_DCC_CHAT(char *nick, char *from, char *handle,
 {
   char *action, *param, *ip, *prt, buf[512], *msg = buf;
   int i;
+#ifdef TLS
+  int ssl = 0;
+#endif
   struct userrec *u = get_user_by_handle(userlist, handle);
   struct flag_record fr = { FR_GLOBAL | FR_CHAN | FR_ANYWH, 0, 0, 0, 0, 0 };
 
-  strcpy(msg, text);
+  strncpyz(buf, text, sizeof buf);
   action = newsplit(&msg);
   param = newsplit(&msg);
   ip = newsplit(&msg);
   prt = newsplit(&msg);
+#ifdef TLS
+  if (egg_strcasecmp(action, "CHAT") || egg_strcasecmp(object, botname) || !u)
+  {
+    if (!egg_strcasecmp(action, "SCHAT"))
+      ssl = 1;
+    else
+      return 0;
+  }
+#else
   if (egg_strcasecmp(action, "CHAT") || egg_strcasecmp(object, botname) || !u)
     return 0;
+#endif
   get_user_flagrec(u, &fr, 0);
   if (dcc_total == max_dcc && increase_socks_max()) {
     if (!quiet_reject)
@@ -1509,38 +1560,59 @@ static int ctcp_DCC_CHAT(char *nick, char *from, char *handle,
       putlog(LOG_MISC, "*", "DCC connection: CHAT (%s!%s)", nick, ip);
       return 1;
     }
-    dcc[i].addr = my_atoul(ip);
+#ifdef TLS
+    dcc[i].ssl = ssl;
+#endif
     dcc[i].port = atoi(prt);
+    (void) setsockname(&dcc[i].sockname, ip, dcc[i].port, 0);
+    dcc[i].u.dns->ip = &dcc[i].sockname;
     dcc[i].sock = -1;
     strcpy(dcc[i].nick, u->handle);
     strcpy(dcc[i].host, from);
     dcc[i].timeval = now;
     dcc[i].user = u;
-    dcc[i].u.dns->ip = dcc[i].addr;
     dcc[i].u.dns->dns_type = RES_HOSTBYIP;
     dcc[i].u.dns->dns_success = dcc_chat_hostresolved;
     dcc[i].u.dns->dns_failure = dcc_chat_hostresolved;
     dcc[i].u.dns->type = &DCC_CHAT_PASS;
-    dcc_dnshostbyip(dcc[i].addr);
+    dcc_dnshostbyip(&dcc[i].sockname);
   }
   return 1;
 }
 
+#ifdef TLS
+int dcc_chat_sslcb(int sock)
+{
+  int idx;
+
+  idx = findanyidx(sock);
+  if ((idx >= 0) && dcc_fingerprint(idx))
+    dprintf(idx, "%s\n", DCC_ENTERPASS);
+  return 0;
+}
+#endif
+
 static void dcc_chat_hostresolved(int i)
 {
-  char buf[512], ip[512];
+  char buf[512];
   struct flag_record fr = { FR_GLOBAL | FR_CHAN | FR_ANYWH, 0, 0, 0, 0, 0 };
 
   egg_snprintf(buf, sizeof buf, "%d", dcc[i].port);
-  if (!hostsanitycheck_dcc(dcc[i].nick, dcc[i].host, dcc[i].addr,
-                           dcc[i].u.dns->host, buf)) {
+  if (!hostsanitycheck_dcc(dcc[i].nick, dcc[i].host, &dcc[i].sockname,
+      dcc[i].u.dns->host, buf)) {
     lostdcc(i);
     return;
   }
-  egg_snprintf(ip, sizeof ip, "%lu", iptolong(htonl(dcc[i].addr)));
-  dcc[i].sock = getsock(0);
-  if (dcc[i].sock < 0 || open_telnet_dcc(dcc[i].sock, ip, buf) < 0) {
-    neterror(buf);
+  buf[0] = 0;
+  dcc[i].sock = getsock(dcc[i].sockname.family, 0);
+  if (dcc[i].sock < 0 || open_telnet_raw(dcc[i].sock, &dcc[i].sockname) < 0)
+    egg_snprintf(buf, sizeof buf, "%s", strerror(errno));
+#ifdef TLS
+  else if (dcc[i].ssl && ssl_handshake(dcc[i].sock, TLS_CONNECT, tls_vfydcc,
+                                       LOG_MISC, dcc[i].host, &dcc_chat_sslcb))
+    egg_snprintf(buf, sizeof buf, "TLS negotiation error");
+#endif
+  if (buf[0]) {
     if (!quiet_reject)
       dprintf(DP_HELP, "NOTICE %s :%s (%s)\n", dcc[i].nick,
               DCC_CONNECTFAILED1, buf);
@@ -1560,6 +1632,11 @@ static void dcc_chat_hostresolved(int i)
     /* Ok, we're satisfied with them now: attempt the connect */
     putlog(LOG_MISC, "*", "DCC connection: CHAT (%s!%s)", dcc[i].nick,
            dcc[i].host);
+#ifdef TLS
+    /* For SSL connections, the handshake callback will determine
+       if we should request a password */
+    if (!dcc[i].ssl)
+#endif
     dprintf(i, "%s\n", DCC_ENTERPASS);
   }
   return;
@@ -1601,7 +1678,7 @@ static void server_5minutely()
 
 static void server_prerehash()
 {
-  strcpy(oldnick, botname);
+  strncpyz(oldnick, botname, sizeof oldnick);
 }
 
 static void server_postrehash()
@@ -1609,8 +1686,15 @@ static void server_postrehash()
   strncpyz(botname, origbotname, NICKLEN);
   if (!botname[0])
     fatal("NO BOT NAME.", 0);
-  if (serverlist == NULL)
-    fatal("NO SERVER.", 0);
+  if ((serverlist == NULL)
+#ifndef TLS
+  && (sslserver)) {
+    fatal("NO NON-SSL SERVERS ADDED (TLS IS DISABLED).", 0);
+  } else if (serverlist == NULL
+#endif
+  ) {
+    fatal("NO SERVERS ADDED.", 0);
+  }
   if (oldnick[0] && !rfc_casecmp(oldnick, botname) &&
       !rfc_casecmp(oldnick, get_altbotnick())) {
     /* Change botname back, don't be premature. */
@@ -1704,8 +1788,14 @@ static void server_report(int idx, int details)
 
   if ((trying_server || server_online) &&
       ((servidx = findanyidx(serv)) != -1)) {
-    dprintf(idx, "    Server %s:%d %s\n", dcc[servidx].host, dcc[servidx].port,
-            trying_server ? "(trying)" : s);
+#ifdef TLS
+    dprintf(idx, "    Server [%s]:%s%d %s\n", dcc[servidx].host,
+            dcc[servidx].ssl ? "+" : "", dcc[servidx].port, trying_server ?
+            "(trying)" : s);
+#else
+    dprintf(idx, "    Server [%s]:%d %s\n", dcc[servidx].host,
+            dcc[servidx].port, trying_server ? "(trying)" : s);
+#endif
   } else
     dprintf(idx, "    %s\n", IRC_NOSERVER);
 
@@ -1723,9 +1813,6 @@ static void server_report(int idx, int details)
   if (details) {
     int size = server_expmem();
 
-    if (min_servs)
-      dprintf(idx, "    Requiring a network with at least %d server%s\n",
-              min_servs, (min_servs != 1) ? "s" : "");
     if (initserver[0])
       dprintf(idx, "    On connect, I do: %s\n", initserver);
     if (connectserver[0])
@@ -1812,7 +1899,11 @@ static Function server_table[] = {
   /* 4 - 7 */
   (Function) NULL,              /* char * (points to botname later on)  */
   (Function) botuserhost,       /* char *                               */
+#ifdef TLS
+  (Function) & use_ssl,		/* int					*/
+#else
   (Function) NULL,              /* Was quiet_reject <Wcc[01/21/03]>.    */
+#endif
   (Function) & serv,            /* int                                  */
   /* 8 - 11 */
   (Function) & flud_thr,        /* int                                  */
@@ -1827,7 +1918,7 @@ static Function server_table[] = {
   /* 16 - 19 */
   (Function) & trigger_on_ignore, /* int                                */
   (Function) check_tcl_ctcpr,
-  (Function) detect_avalanche,
+  (Function) NULL,
   (Function) nuke_server,
   /* 20 - 23 */
   (Function) newserver,         /* char *                               */
@@ -1837,7 +1928,7 @@ static Function server_table[] = {
   /* 24 - 27 */
   (Function) & default_port,    /* int                                  */
   (Function) & server_online,   /* int                                  */
-  (Function) & min_servs,       /* int                                  */
+  (Function) NULL,              /* min_servs -- removed (guppy)         */
   (Function) & H_raw,           /* p_tcl_bind_list                      */
   /* 28 - 31 */
   (Function) & H_wall,          /* p_tcl_bind_list                      */
@@ -1889,9 +1980,7 @@ char *server_start(Function *global_funcs)
   server_online = 0;
   server_cycle_wait = 60;
   strcpy(botrealname, "A deranged product of evil coders");
-  min_servs = 0;
   server_timeout = 60;
-  never_give_up = 0;
   serverlist = NULL;
   cycle_time = 0;
   default_port = 6667;
@@ -1920,12 +2009,15 @@ char *server_start(Function *global_funcs)
   stack_limit = 4;
   realservername = 0;
   msgrate = 2;
+#ifdef TLS
+  tls_vfyserver = 0;
+#endif
 
   server_table[4] = (Function) botname;
-  module_register(MODULE_NAME, server_table, 1, 3);
-  if (!module_depend(MODULE_NAME, "eggdrop", 106, 7)) {
+  module_register(MODULE_NAME, server_table, 1, 4);
+  if (!module_depend(MODULE_NAME, "eggdrop", 108, 0)) {
     module_undepend(MODULE_NAME);
-    return "This module requires Eggdrop 1.6.7 or later.";
+    return "This module requires Eggdrop 1.8.0 or later.";
   }
 
   /* Fool bot in reading the values. */
