@@ -1,11 +1,9 @@
 /*
  * server.h -- part of server.mod
- *
- * $Id: server.h,v 1.29 2011/02/13 14:19:34 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,7 +30,11 @@
 /* 4 - 7 */
 /* Empty */
 #define botuserhost ((char *)(server_funcs[5]))
+#ifdef TLS
+#define use_ssl (*(int *)(server_funcs[6]))
+#else
 /* Was quiet_reject (moved to core) <Wcc[01/21/03]>. */
+#endif
 #define serv (*(int *)(server_funcs[7]))
 /* 8 - 11 */
 #define flud_thr (*(int*)(server_funcs[8]))
@@ -47,7 +49,7 @@
 /* 16 - 19 */
 #define trigger_on_ignore (*(int *)(server_funcs[16]))
 #define check_tcl_ctcpr ((int(*)(char*,char*,struct userrec*,char*,char*,char*,p_tcl_bind_list))server_funcs[17])
-#define detect_avalanche ((int(*)(char *))server_funcs[18])
+/* Was detect_avalanche */
 #define nuke_server ((void(*)(char *))server_funcs[19])
 /* 20 - 23 */
 #define newserver ((char *)(server_funcs[20]))
@@ -57,7 +59,7 @@
 /* 24 - 27 */
 #define default_port (*(int *)(server_funcs[24]))
 #define server_online (*(int *)(server_funcs[25]))
-#define min_servs (*(int *)(server_funcs[26]))
+/* Was min_servs */
 #define H_raw (*(p_tcl_bind_list *)(server_funcs[27]))
 /* 28 - 31 */
 #define H_wall (*(p_tcl_bind_list *)(server_funcs[28]))
@@ -96,6 +98,9 @@ struct server_list {
 
   char *name;
   int port;
+#ifdef TLS
+  int ssl;
+#endif
   char *pass;
   char *realname;
 };
