@@ -1,11 +1,9 @@
 /*
  * servmsg.c -- part of server.mod
- *
- * $Id: servmsg.c,v 1.107 2011/02/13 14:19:34 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,6 +20,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#include  "../irc.mod/irc.h"
+#include  "../channels.mod/channels.h"
 
 static time_t last_ctcp = (time_t) 0L;
 static int count_ctcp = 0;
@@ -122,7 +122,7 @@ static int check_tcl_msgm(char *cmd, char *nick, char *uhost,
   if (arg[0])
     simple_sprintf(args, "%s %s", cmd, arg);
   else
-    strcpy(args, cmd);
+    strncpyz(args, cmd, sizeof args);
   get_user_flagrec(u, &fr, NULL);
   Tcl_SetVar(interp, "_msgm1", nick, 0);
   Tcl_SetVar(interp, "_msgm2", uhost, 0);
@@ -266,7 +266,7 @@ static int check_tcl_out(int which, char *msg, int sent)
   default:
     queue = "noqueue";
   }
-  snprintf(args, sizeof args, "%s %s", queue, sent ? "sent" : "queued");
+  egg_snprintf(args, sizeof args, "%s %s", queue, sent ? "sent" : "queued");
   Tcl_SetVar(interp, "_out1", queue, 0);
   Tcl_SetVar(interp, "_out2", msg, 0);
   Tcl_SetVar(interp, "_out3", sent ? "sent" : "queued", 0);
@@ -359,7 +359,7 @@ static int got442(char *from, char *msg)
 
     putlog(LOG_MISC, chname, IRC_SERVNOTONCHAN, chname);
     if (me && me->funcs)
-      (me->funcs[CHANNEL_CLEAR]) (chan, 1);
+      (me->funcs[CHANNEL_CLEAR]) (chan, CHAN_RESETALL);
     chan->status &= ~CHAN_ACTIVE;
 
     key = chan->channel.key[0] ? chan->channel.key : chan->key_prot;
@@ -375,11 +375,20 @@ static int got442(char *from, char *msg)
  */
 static void nuke_server(char *reason)
 {
+  struct chanset_t *chan;
+  module_entry *me;
+
   if (serv >= 0) {
     int servidx = findanyidx(serv);
 
     if (reason && (servidx > 0))
       dprintf(servidx, "QUIT :%s\n", reason);
+    for (chan = chanset; chan; chan = chan->next) {
+      if (channel_active(chan))
+        if ((me = module_find("irc", 1, 3)) != NULL)
+          (me->funcs[CHANNEL_CLEAR]) (chan, CHAN_RESETALL);
+    }
+
     disconnect_server(servidx);
     lostdcc(servidx);
   }
@@ -433,7 +442,7 @@ static int detect_flood(char *floodnick, char *floodhost, char *from, int which)
   if (p) {
     p++;
     if (egg_strcasecmp(lastmsghost[which], p)) {        /* New */
-      strcpy(lastmsghost[which], p);
+      strncpyz(lastmsghost[which], p, sizeof lastmsghost[which]);
       lastmsgtime[which] = now;
       lastmsgs[which] = 0;
       return 0;
@@ -465,23 +474,6 @@ static int detect_flood(char *floodnick, char *floodhost, char *from, int which)
   return 0;
 }
 
-/* Check for more than 8 control characters in a line.
- * This could indicate: beep flood CTCP avalanche.
- */
-static int detect_avalanche(char *msg)
-{
-  int count = 0;
-  unsigned char *p;
-
-  for (p = (unsigned char *) msg; (*p) && (count < 8); p++)
-    if ((*p == 7) || (*p == 1))
-      count++;
-  if (count >= 8)
-    return 1;
-  else
-    return 0;
-}
-
 /* Got a private message.
  */
 static int gotmsg(char *from, char *msg)
@@ -499,22 +491,12 @@ static int gotmsg(char *from, char *msg)
   ignoring = match_ignore(from);
   to = newsplit(&msg);
   fixcolon(msg);
-  /* Only check if flood-ctcp is active */
   strncpyz(uhost, from, sizeof(buf));
   nick = splitnick(&uhost);
-  if (flud_ctcp_thr && detect_avalanche(msg)) {
-    if (!ignoring) {
-      putlog(LOG_MODES, "*", "Avalanche from %s - ignoring", from);
-      p = strchr(uhost, '@');
-      if (p != NULL)
-        p++;
-      else
-        p = uhost;
-      egg_snprintf(ctcpbuf, sizeof(ctcpbuf), "*!*@%s", p);
-      addignore(ctcpbuf, botnetnick, "ctcp avalanche",
-                now + (60 * ignore_time));
-    }
-  }
+  /* Apparently servers can send CTCPs now too, not just nicks */
+  if (nick[0] == '\0')
+    nick = uhost;
+
   /* Check for CTCP: */
   ctcp_reply[0] = 0;
   p = strchr(msg, 1);
@@ -646,12 +628,7 @@ static int gotnotice(char *from, char *msg)
   fixcolon(msg);
   strcpy(uhost, from);
   nick = splitnick(&uhost);
-  if (flud_ctcp_thr && detect_avalanche(msg)) {
-    /* Discard -- kick user if it was to the channel */
-    if (!ignoring)
-      putlog(LOG_MODES, "*", "Avalanche from %s", from);
-    return 0;
-  }
+
   /* Check for CTCP: */
   p = strchr(msg, 1);
   while ((p != NULL) && (*p)) {
@@ -724,34 +701,6 @@ static int gotnotice(char *from, char *msg)
   return 0;
 }
 
-/* got 251: lusers
- * <server> 251 <to> :there are 2258 users and 805 invisible on 127 servers
- */
-static int got251(char *from, char *msg)
-{
-  int i;
-  char *servs;
-
-  if (min_servs == 0)
-    return 0;                   /* No minimum limit on servers */
-  newsplit(&msg);
-  fixcolon(msg);                /* NOTE!!! If servlimit is not set or is 0 */
-  for (i = 0; i < 8; i++)
-    newsplit(&msg);             /* lusers IS NOT SENT AT ALL!! */
-  servs = newsplit(&msg);
-  if (strncmp(msg, "servers", 7))
-    return 0;                   /* Was invalid format */
-  while (*servs && (*servs < 32))
-    servs++;                    /* I've seen some lame nets put bolds &
-                                 * stuff in here :/ */
-  i = atoi(servs);
-  if (i < min_servs) {
-    putlog(LOG_SERV, "*", IRC_AUTOJUMP, min_servs, i);
-    nuke_server(IRC_CHANGINGSERV);
-  }
-  return 0;
-}
-
 /* WALLOPS: oper's nuisance
  */
 static int gotwall(char *from, char *msg)
@@ -770,15 +719,11 @@ static int gotwall(char *from, char *msg)
   return 0;
 }
 
-/* Called once a minute... but if we're the only one on the
- * channel, we only wanna send out "lusers" once every 5 mins.
+/* Called once a minute...
  */
 static void minutely_checks()
 {
   char *alt;
-  static int count = 4;
-  int ok = 0;
-  struct chanset_t *chan;
 
   /* Only check if we have already successfully logged in.  */
   if (!server_online)
@@ -796,20 +741,6 @@ static void minutely_checks()
         dprintf(DP_SERVER, "ISON :%s %s\n", botname, origbotname);
     }
   }
-  if (min_servs == 0)
-    return;
-  for (chan = chanset; chan; chan = chan->next)
-    if (channel_active(chan) && chan->channel.members == 1) {
-      ok = 1;
-      break;
-    }
-  if (!ok)
-    return;
-  count++;
-  if (count >= 5) {
-    dprintf(DP_SERVER, "LUSERS\n");
-    count = 0;
-  }
 }
 
 /* Pong from server.
@@ -1028,15 +959,21 @@ static int gotmode(char *from, char *msg)
   ch = newsplit(&msg);
   /* Usermode changes? */
   if (strchr(CHANMETA, ch[0]) == NULL) {
-    if (match_my_nick(ch) && check_mode_r) {
-      /* umode +r? - D0H dalnet uses it to mean something different */
+    if (match_my_nick(ch)) {
       fixcolon(msg);
-      if ((msg[0] == '+') && strchr(msg, 'r')) {
-        int servidx = findanyidx(serv);
+      if ((msg[0] == '+') || (msg[0] == '-')) {
+        /* send a WHOIS in case our host was cloaked */
+        dprintf(DP_SERVER, "WHOIS %s\n", botname);
+      }
+      if (check_mode_r) {
+        /* umode +r? - D0H dalnet uses it to mean something different */
+        if ((msg[0] == '+') && strchr(msg, 'r')) {
+          int servidx = findanyidx(serv);
 
-        putlog(LOG_MISC | LOG_JOIN, "*",
-               "%s has me i-lined (jumping)", dcc[servidx].host);
-        nuke_server("i-lines suck");
+          putlog(LOG_MISC | LOG_JOIN, "*",
+                 "%s has me i-lined (jumping)", dcc[servidx].host);
+          nuke_server("i-lines suck");
+        }
       }
     }
   }
@@ -1081,7 +1018,7 @@ static void kill_server(int idx, void *x)
     struct chanset_t *chan;
 
     for (chan = chanset; chan; chan = chan->next)
-      (me->funcs[CHANNEL_CLEAR]) (chan, 1);
+      (me->funcs[CHANNEL_CLEAR]) (chan, CHAN_RESETALL);
   }
   /* A new server connection will be automatically initiated in
    * about 2 seconds. */
@@ -1091,6 +1028,7 @@ static void timeout_server(int idx)
 {
   putlog(LOG_SERV, "*", "Timeout: connect to %s", dcc[idx].host);
   disconnect_server(idx);
+  check_tcl_event("fail-server");
   lostdcc(idx);
 }
 
@@ -1202,7 +1140,6 @@ static int got311(char *from, char *msg)
  */
 static int got465(char *from, char *msg)
 {
-  newsplit(&msg); /* 465 */
   newsplit(&msg); /* nick */
 
   fixcolon(msg);
@@ -1222,7 +1159,6 @@ static cmd_t my_raw_binds[] = {
   {"PONG",    "",   (IntFunc) gotpong,      NULL},
   {"WALLOPS", "",   (IntFunc) gotwall,      NULL},
   {"001",     "",   (IntFunc) got001,       NULL},
-  {"251",     "",   (IntFunc) got251,       NULL},
   {"303",     "",   (IntFunc) got303,       NULL},
   {"432",     "",   (IntFunc) got432,       NULL},
   {"433",     "",   (IntFunc) got433,       NULL},
@@ -1249,16 +1185,12 @@ static void server_resolve_failure(int);
 static void connect_server(void)
 {
   char pass[121], botserver[UHOSTLEN];
-  static int oldserv = -1;
   int servidx;
   unsigned int botserverport = 0;
 
   lastpingcheck = 0;
   trying_server = now;
   empty_msgq();
-  /* Start up the counter (always reset it if "never-give-up" is on) */
-  if ((oldserv < 0) || (never_give_up))
-    oldserv = curserv;
   if (newserverport) {          /* Jump to specified server */
     curserv = -1;             /* Reset server list */
     strcpy(botserver, newserver);
@@ -1267,7 +1199,6 @@ static void connect_server(void)
     newserver[0] = 0;
     newserverport = 0;
     newserverpass[0] = 0;
-    oldserv = -1;
   } else {
     if (curserv == -1)
       curserv = 999;
@@ -1294,8 +1225,14 @@ static void connect_server(void)
       do_tcl("connect-server", connectserver);
     check_tcl_event("connect-server");
     next_server(&curserv, botserver, &botserverport, pass);
-    putlog(LOG_SERV, "*", "%s %s:%d", IRC_SERVERTRY, botserver, botserverport);
-
+#ifdef TLS
+    putlog(LOG_SERV, "*", "%s [%s]:%s%d", IRC_SERVERTRY, botserver,
+           use_ssl ? "+" : "", botserverport);
+    dcc[servidx].ssl = use_ssl;
+#else
+    putlog(LOG_SERV, "*", "%s [%s]:%d", IRC_SERVERTRY, botserver,
+           botserverport);
+#endif
     dcc[servidx].port = botserverport;
     strcpy(dcc[servidx].nick, "(server)");
     strncpyz(dcc[servidx].host, botserver, UHOSTLEN);
@@ -1338,44 +1275,61 @@ static void server_resolve_failure(int servidx)
   resolvserv = 0;
   putlog(LOG_SERV, "*", "%s %s (%s)", IRC_FAILEDCONNECT, dcc[servidx].host,
          IRC_DNSFAILED);
+  check_tcl_event("fail-server");
   lostdcc(servidx);
 }
 
 static void server_resolve_success(int servidx)
 {
-  int oldserv = dcc[servidx].u.dns->ibuf;
-  char s[121], pass[121];
+  char pass[121];
 
   resolvserv = 0;
-  dcc[servidx].addr = dcc[servidx].u.dns->ip;
-  strcpy(pass, dcc[servidx].u.dns->cbuf);
+  strncpyz(pass, dcc[servidx].u.dns->cbuf, sizeof pass);
   changeover_dcc(servidx, &SERVER_SOCKET, 0);
-  serv = open_telnet(iptostr(htonl(dcc[servidx].addr)), dcc[servidx].port);
+  dcc[servidx].sock = getsock(dcc[servidx].sockname.family, 0);
+  setsnport(dcc[servidx].sockname, dcc[servidx].port);
+  serv = open_telnet_raw(dcc[servidx].sock, &dcc[servidx].sockname);
   if (serv < 0) {
-    neterror(s);
+    char *errstr = NULL;
+    if (errno == EINVAL) {
+      errstr = IRC_VHOSTWRONGNET;
+    } else if (errno == EADDRNOTAVAIL) {
+      errstr = IRC_VHOSTBADADDR;
+    } else {
+      errstr = strerror(errno);
+    }
     putlog(LOG_SERV, "*", "%s %s (%s)", IRC_FAILEDCONNECT, dcc[servidx].host,
-           s);
+           errstr);
+    check_tcl_event("fail-server");
     lostdcc(servidx);
-    if (oldserv == curserv && !never_give_up)
-      fatal("NO SERVERS WILL ACCEPT MY CONNECTION.", 0);
-  } else {
-    dcc[servidx].sock = serv;
-    /* Queue standard login */
-    dcc[servidx].timeval = now;
-    SERVER_SOCKET.timeout_val = &server_timeout;
-    /* Another server may have truncated it, so use the original */
-    strcpy(botname, origbotname);
-    /* Start alternate nicks from the beginning */
-    altnick_char = 0;
-    if (pass[0])
-      dprintf(DP_MODE, "PASS %s\n", pass);
-    dprintf(DP_MODE, "NICK %s\n", botname);
-
-    rmspace(botrealname);
-    if (botrealname[0] == 0)
-      strcpy(botrealname, "/msg LamestBot hello");
-    dprintf(DP_MODE, "USER %s . . :%s\n", botuser, botrealname);
-
-    /* Wait for async result now. */
+    return;
   }
+#ifdef TLS
+  if (dcc[servidx].ssl && ssl_handshake(serv, TLS_CONNECT, tls_vfyserver,
+                                        LOG_SERV, dcc[servidx].host, NULL)) {
+    putlog(LOG_SERV, "*", "%s %s (%s)", IRC_FAILEDCONNECT, dcc[servidx].host,
+           "TLS negotiation failure");
+    check_tcl_event("fail-server");
+    lostdcc(servidx);
+    return;
+  }
+#endif
+  /* Queue standard login */
+  dcc[servidx].timeval = now;
+  SERVER_SOCKET.timeout_val = &server_timeout;
+  /* Another server may have truncated it, so use the original */
+  strcpy(botname, origbotname);
+  /* Start alternate nicks from the beginning */
+  altnick_char = 0;
+  check_tcl_event("preinit-server");
+  if (pass[0])
+    dprintf(DP_MODE, "PASS %s\n", pass);
+  dprintf(DP_MODE, "NICK %s\n", botname);
+
+  rmspace(botrealname);
+  if (botrealname[0] == 0)
+    strcpy(botrealname, "/msg LamestBot hello");
+  dprintf(DP_MODE, "USER %s . . :%s\n", botuser, botrealname);
+
+  /* Wait for async result now. */
 }
