@@ -1,11 +1,9 @@
 /*
  * tclserv.c -- part of server.mod
- *
- * $Id: tclserv.c,v 1.26 2011/02/13 14:19:34 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -175,7 +173,17 @@ static int tcl_jump STDVAR
   if (argc >= 2) {
     strncpyz(newserver, argv[1], sizeof newserver);
     if (argc >= 3)
+#ifdef TLS
+    {
+      if (*argv[2] == '+')
+        use_ssl = 1;
+      else
+        use_ssl = 0;
       newserverport = atoi(argv[2]);
+    }
+#else
+      newserverport = atoi(argv[2]);
+#endif
     else
       newserverport = default_port;
     if (argc == 4)
