@@ -1,10 +1,9 @@
 /*
  * uf_features.c -- part of share.mod
  *
- * $Id: uf_features.c,v 1.20 2011/02/13 14:19:34 simple Exp $
  */
 /*
- * Copyright (C) 2000 - 2011 Eggheads Development Team
+ * Copyright (C) 2000 - 2018 Eggheads Development Team
  * Written by Fabian Knittel <fknittel@gmx.de>
  *
  * This program is free software; you can redistribute it and/or
@@ -222,7 +221,7 @@ static void uff_deltable(uff_table_t *ut)
   if (!ut)
     return;
   for (; ut->feature; ++ut)
-    (int) uff_delfeature(ut);
+    uff_delfeature(ut);
 }
 
 
@@ -253,8 +252,9 @@ static void uf_features_parse(int idx, char *par)
     ul = uff_findentry_byname(p);
     if (ul && (ul->entry->ask_func == NULL || ul->entry->ask_func(idx))) {
       dcc[idx].u.bot->uff_flags |= ul->entry->flag; /* Set flag */
-      strcat(uff_sbuf, ul->entry->feature); /* Add feature to list */
-      strcat(uff_sbuf, " ");
+      if (uff_sbuf[0])
+        strncat(uff_sbuf, " ", sizeof uff_sbuf - strlen(uff_sbuf) - 1);
+      strncat(uff_sbuf, ul->entry->feature, sizeof uff_sbuf - strlen(uff_sbuf) - 1); /* Add feature to list */
     }
     p = ++s;
   }
@@ -274,8 +274,9 @@ static char *uf_features_dump(int idx)
   uff_sbuf[0] = 0;
   for (ul = uff_list.start; ul; ul = ul->next)
     if (ul->entry->ask_func == NULL || ul->entry->ask_func(idx)) {
-      strcat(uff_sbuf, ul->entry->feature); /* Add feature to list  */
-      strcat(uff_sbuf, " ");
+      if (uff_sbuf[0])
+        strncat(uff_sbuf, " ", sizeof uff_sbuf - strlen(uff_sbuf) - 1);
+      strncat(uff_sbuf, ul->entry->feature, sizeof uff_sbuf - strlen(uff_sbuf) - 1); /* Add feature to list  */
     }
   return uff_sbuf;
 }
