@@ -3,12 +3,10 @@
  *   support for modules in eggdrop
  *
  * by Darrin Smith (beldin@light.iinet.net.au)
- *
- * $Id: modules.c,v 1.110 2011/07/09 15:07:48 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -84,9 +82,8 @@ extern struct dcc_t *dcc;
 extern struct userrec *userlist, *lastuser;
 extern struct chanset_t *chanset;
 
-extern char tempdir[], botnetnick[], botname[], natip[], hostname[],
-            origbotname[], botuser[], admin[], userfile[], ver[], notify_new[],
-            helpdir[], version[], quit_msg[], log_ts[];
+extern char botnetnick[], botname[], origbotname[], botuser[], ver[], log_ts[],
+            admin[], userfile[], notify_new[], helpdir[], version[], quit_msg[];
 
 extern int parties, noshare, dcc_total, egg_numver, userfile_perm, do_restart,
            ignore_time, must_be_owner, raw_log, max_dcc, make_userfile,
@@ -94,6 +91,14 @@ extern int parties, noshare, dcc_total, egg_numver, userfile_perm, do_restart,
            password_timeout, force_expire, protect_readonly, reserved_port_min,
            reserved_port_max, copy_to_tmp, quiet_reject;
 
+#ifdef IPV6
+extern int pref_af;
+#endif
+
+#ifdef TLS
+extern int tls_vfyclients, tls_vfydcc, tls_vfybots;
+#endif
+
 extern party_t *party;
 extern time_t now, online_since;
 extern tand_t *tandbot;
@@ -180,7 +185,7 @@ int (*rfc_casecmp) (const char *, const char *) = _rfc_casecmp;
 int (*rfc_ncasecmp) (const char *, const char *, int) = _rfc_ncasecmp;
 int (*rfc_toupper) (int) = _rfc_toupper;
 int (*rfc_tolower) (int) = _rfc_tolower;
-void (*dns_hostbyip) (IP) = block_dns_hostbyip;
+void (*dns_hostbyip) (sockname_t *) = block_dns_hostbyip;
 void (*dns_ipbyhost) (char *) = block_dns_ipbyhost;
 
 module_entry *module_list;
@@ -292,8 +297,12 @@ Function global_table[] = {
   (Function) egg_list_contains,
   /* 76 - 79 */
   (Function) answer,
-  (Function) getmyip,
-  (Function) neterror,
+  (Function) getvhost,
+#ifdef TLS
+  (Function) ssl_handshake,
+#else
+  (Function) 0,
+#endif
   (Function) tputs,
   /* 80 - 83 */
   (Function) new_dcc,
@@ -302,7 +311,7 @@ Function global_table[] = {
   (Function) killsock,
   /* 84 - 87 */
   (Function) open_listen,
-  (Function) open_telnet_dcc,
+  (Function) getdccaddr,
   (Function) _get_data_ptr,
   (Function) open_telnet,
   /* 88 - 91 */
@@ -324,21 +333,34 @@ Function global_table[] = {
   (Function) & max_dcc,           /* int                                 */
   (Function) & require_p,         /* int                                 */
   (Function) & ignore_time,       /* int                                 */
+#ifdef TLS
+  (Function) dcc_fingerprint,
+#else
   (Function) 0,                   /* was use_console_r <Wcc[02/02/03]>   */
+#endif
   /* 104 - 107 */
   (Function) & reserved_port_min,
   (Function) & reserved_port_max,
   (Function) & raw_log,           /* int                                 */
   (Function) & noshare,           /* int                                 */
   /* 108 - 111 */
+#ifdef TLS
+  (Function) & tls_vfybots,       /* int                                 */
+#else
   (Function) 0,                   /* gban_total -- UNUSED! (Eule)        */
+#endif
   (Function) & make_userfile,     /* int                                 */
   (Function) & default_flags,     /* int                                 */
   (Function) & dcc_total,         /* int                                 */
   /* 112 - 115 */
-  (Function) tempdir,             /* char *                              */
-  (Function) natip,               /* char *                              */
-  (Function) hostname,            /* char *                              */
+  (Function) 0,                   /* was tempdir                         */
+#ifdef TLS
+  (Function) & tls_vfyclients,    /* int                                 */
+  (Function) & tls_vfydcc,        /* int                                 */
+#else
+  (Function) 0,                   /* was natip -- use getmyip() instead  */
+  (Function) 0,                   /* was myip -- use getvhost() instead  */
+#endif
   (Function) origbotname,         /* char *                              */
   /* 116 - 119 */
   (Function) botuser,             /* char *                              */
@@ -466,8 +488,8 @@ Function global_table[] = {
   (Function) sanitycheck_dcc,
   (Function) isowner,
   /* 216 - 219 */
-  (Function) 0,                   /* min_dcc_port -- UNUSED! (guppy)     */
-  (Function) 0,                   /* max_dcc_port -- UNUSED! (guppy)     */
+  (Function) fcopyfile,
+  (Function) copyfilef,
   (Function) & rfc_casecmp,       /* Function *                          */
   (Function) & rfc_ncasecmp,      /* Function *                          */
   /* 220 - 223 */
@@ -561,10 +583,14 @@ Function global_table[] = {
   /* 284 - 287 */
   (Function) & quiet_reject,      /* int                                 */
   (Function) file_readable,
-  (Function) 0,                   /* IPv6 leftovers: 286                 */
-  (Function) 0,                   /* IPv6 leftovers: 287                 */
+  (Function) setsockname,
+  (Function) open_telnet_raw,
   /* 288 - 291 */
+#ifdef IPV6
+  (Function) & pref_af,		  /* int                                 */
+#else
   (Function) 0,                   /* IPv6 leftovers: 288                 */
+#endif
   (Function) strip_mirc_codes,
   (Function) check_ansi,
   (Function) oatoi,
@@ -577,7 +603,11 @@ Function global_table[] = {
   (Function) check_conflags,
   (Function) increase_socks_max,
   (Function) log_ts,
-  (Function) mod_strdup
+  (Function) tcl_resultempty,
+  /* 300 - 303 */
+  (Function) tcl_resultint,
+  (Function) tcl_resultstring,
+  (Function) getdccfamilyaddr
 };
 
 void init_modules(void)
@@ -942,20 +972,6 @@ int module_undepend(char *name1)
   return ok;
 }
 
-char *mod_strdup(const char *s, const char *modname, const char *filename, int line)
-{
-#ifdef DEBUG_MEM
-  char x[100], *p;
-
-  p = strrchr(filename, '/');
-  egg_snprintf(x, sizeof x, "%s:%s", modname, p ? p + 1 : filename);
-  x[19] = 0;
-  return n_strdup(s, x, line);
-#else
-  return nstrdup(s);
-#endif
-}
-
 void *mod_malloc(int size, const char *modname, const char *filename, int line)
 {
 #ifdef DEBUG_MEM
@@ -1056,7 +1072,7 @@ void add_hook(int hook_num, Function func)
       break;
     case HOOK_DNS_HOSTBYIP:
       if (dns_hostbyip == block_dns_hostbyip)
-        dns_hostbyip = (void (*)(IP)) func;
+        dns_hostbyip = (void (*)(sockname_t *)) func;
       break;
     case HOOK_DNS_IPBYHOST:
       if (dns_ipbyhost == block_dns_ipbyhost)
@@ -1117,7 +1133,7 @@ void del_hook(int hook_num, Function func)
         match_noterej = false_func;
       break;
     case HOOK_DNS_HOSTBYIP:
-      if (dns_hostbyip == (void (*)(IP)) func)
+      if (dns_hostbyip == (void (*)(sockname_t *)) func)
         dns_hostbyip = block_dns_hostbyip;
       break;
     case HOOK_DNS_IPBYHOST:
