@@ -6,12 +6,10 @@
  * with full prototyping, some have been moved to other .h files
  * because they use structures in those
  * (saves including those .h files EVERY time) - Beldin
- *
- * $Id: proto.h,v 1.84 2011/07/09 15:07:48 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -60,6 +58,7 @@ void bot_share(int, char *);
 int base64_to_int(char *);
 
 /* botnet.c */
+void set_botnetnick(const char *);
 void answer_local_whom(int, int);
 char *lastbot(char *);
 int nextbot(char *);
@@ -104,6 +103,7 @@ int isowner(char *);
 char *masktype(int);
 char *maskname(int);
 void reaffirm_owners();
+void add_hq_user();
 void rehash();
 void reload();
 void chanprog();
@@ -114,10 +114,12 @@ void check_timers();
 void set_chanlist(const char *host, struct userrec *rec);
 void clear_chanlist(void);
 void clear_chanlist_member(const char *nick);
+float getcputime();
 
 /* cmds.c */
 int check_dcc_attrs(struct userrec *, int);
 int check_dcc_chanattrs(struct userrec *, char *, int, int);
+int check_int_range(char *value, int min, int max);
 int stripmodes(char *);
 char *stripmasktype(int);
 
@@ -126,12 +128,16 @@ void failed_link(int);
 void strip_mirc_codes(int, char *);
 int check_ansi(char *);
 void dupwait_notify(char *);
+#ifdef TLS
+int dcc_fingerprint(int);
+#endif
 
 /* dccutil.c */
 int increase_socks_max();
 int findidx(int);
 int findanyidx(int);
 char *add_cr(char *);
+void dprint(int, char *, int);
 void dprintf EGG_VARARGS(int, arg1);
 void chatout EGG_VARARGS(char *, arg1);
 extern void (*shareout) ();
@@ -158,13 +164,13 @@ void del_dcc(int);
 void changeover_dcc(int, struct dcc_table *, int);
 
 /* dns.c */
-extern void (*dns_hostbyip) (IP);
+extern void (*dns_hostbyip) (sockname_t *);
+void block_dns_hostbyip(sockname_t *);
+void call_hostbyip(sockname_t *, char *, int);
+void call_ipbyhost(char *, sockname_t *, int);
+void dcc_dnshostbyip(sockname_t *);
 extern void (*dns_ipbyhost) (char *);
-void block_dns_hostbyip(IP);
 void block_dns_ipbyhost(char *);
-void call_hostbyip(IP, char *, int);
-void call_ipbyhost(char *, IP, int);
-void dcc_dnshostbyip(IP);
 void dcc_dnsipbyhost(char *);
 
 /* language.c */
@@ -208,7 +214,6 @@ int cron_match(const char *, const char *);
 #define cmp_usermasks(a,b) addr_match((char *)(a),(char *)(b),1,1)
 
 /* mem.c */
-char *n_strdup(const char *, const char *, int);
 void *n_malloc(int, const char *, int);
 void *n_realloc(void *, int, const char *, int);
 void n_free(void *, const char *, int);
@@ -262,20 +267,21 @@ void maskaddr(const char *, char *, int);
 /* net.c */
 IP my_atoul(char *);
 unsigned long iptolong(IP);
-IP getmyip();
-void neterror(char *);
 void setsock(int, int);
 int allocsock(int, int);
 int alloctclsock(int, int, Tcl_FileProc *, ClientData);
-int getsock(int);
+int getsock(int, int);
 void killsock(int);
 void killtclsock(int);
-int answer(int, char *, unsigned long *, unsigned short *, int);
-inline int open_listen(int *);
-int open_address_listen(IP addr, int *);
-int open_telnet(char *, int);
-int open_telnet_dcc(int, char *, char *);
-int open_telnet_raw(int, char *, int);
+int open_listen(int *);
+void getvhost(sockname_t *, int);
+int setsockname(sockname_t *, char *, int, int);
+int open_address_listen(sockname_t *);
+int open_telnet_raw(int, sockname_t *);
+int open_telnet(int, char *, int);
+int answer(int, sockname_t *, unsigned short *, int);
+int getdccaddr(sockname_t *, char *, size_t);
+int getdccfamilyaddr(sockname_t *, char *, size_t, int);
 void tputs(int, char *, unsigned int);
 void dequeue_sockets();
 int preparefdset(fd_set *, sock_list *, int, int, int);
@@ -283,11 +289,13 @@ int sockread(char *, int *, sock_list *, int, int);
 int sockgets(char *, int *);
 void tell_netdebug(int);
 int sanitycheck_dcc(char *, char *, char *, char *);
-int hostsanitycheck_dcc(char *, char *, IP, char *, char *);
-char *iptostr(IP);
+int hostsanitycheck_dcc(char *, char *, sockname_t *, char *, char *);
+char *iptostr(struct sockaddr *);
 int sock_has_data(int, int);
 int sockoptions(int sock, int operation, int sock_options);
 int flush_inbuf(int idx);
+int findsock(int sock);
+void safe_write(int, const void *, size_t);
 
 /* tcl.c */
 struct threaddata *threaddata();
@@ -297,6 +305,14 @@ void unprotect_tcl();
 void do_tcl(char *, char *);
 int readtclprog(char *fname);
 
+/* tls.c */
+#ifdef TLS
+int ssl_handshake(int, int, int, int, char *, IntFunc);
+char *ssl_fpconv(char *in, char *out);
+const char *ssl_getuid(int sock);
+char *ssl_getfp(int sock);
+#endif
+
 /* userent.c */
 void list_type_kill(struct list_type *);
 int list_type_expmem(struct list_type *);
