@@ -3,12 +3,10 @@
  *   the code for every command eggdrop adds to Tcl
  *   Tcl initialization
  *   getting and setting Tcl/eggdrop variables
- *
- * $Id: tcl.c,v 1.102 2011/09/09 21:37:53 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -46,20 +44,32 @@ typedef struct {
 extern time_t online_since;
 
 extern char origbotname[], botuser[], motdfile[], admin[], userfile[],
-            firewall[], helpdir[], notify_new[], hostname[], myip[], moddir[],
-            tempdir[], owner[], network[], botnetnick[], bannerfile[],
-            egg_version[], natip[], configfile[], logfile_suffix[], log_ts[],
-            textdir[], pid_file[];
+            firewall[], helpdir[], notify_new[], vhost[], moddir[], owner[],
+            network[], botnetnick[], bannerfile[], egg_version[], natip[],
+            configfile[], logfile_suffix[], log_ts[], textdir[], pid_file[],
+            listen_ip[];
+
 
 extern int flood_telnet_thr, flood_telnet_time, shtime, share_greet,
            require_p, keep_all_logs, allow_new_telnets, stealth_telnets,
            use_telnet_banner, default_flags, conmask, switch_logfiles_at,
            connect_timeout, firewallport, notify_users_at, flood_thr, tands,
-           ignore_time, reserved_port_min, reserved_port_max, die_on_sighup,
-           die_on_sigterm, max_logs, max_logsize, dcc_total, raw_log,
-           identtimeout, dcc_sanitycheck, dupwait_timeout, egg_numver,
-           share_unlinks, protect_telnet, sort_users, strict_host,
-           resolve_timeout, default_uflags, userfile_perm, cidr_support;
+           ignore_time, reserved_port_min, reserved_port_max, max_logs,
+           max_logsize, dcc_total, raw_log, identtimeout, dcc_sanitycheck,
+           dupwait_timeout, egg_numver, share_unlinks, protect_telnet,
+           strict_host, resolve_timeout, default_uflags, userfile_perm,
+           cidr_support;
+
+#ifdef IPV6
+extern char vhost6[];
+extern int pref_af;
+#endif
+
+#ifdef TLS
+extern int tls_maxdepth, tls_vfybots, tls_vfyclients, tls_vfydcc, tls_auth;
+extern char tls_capath[], tls_cafile[], tls_certfile[], tls_keyfile[],
+            tls_ciphers[];
+#endif
 
 extern struct dcc_t *dcc;
 extern tcl_timer_t *timer, *utimer;
@@ -87,16 +97,7 @@ int handlen = HANDLEN;
 int utftot = 0;
 int clientdata_stuff = 0;
 
-/* Compatability for removed settings.*/
-int strict_servernames = 0, enable_simul = 1, use_console_r = 0,
-    debug_output = 0;
-
-#ifdef REPLACE_NOTIFIER
-tclevent_t *tclevents = NULL; /* Queue of Tcl snippets to execute */
-#endif
-
-/* Prototypes for Tcl */
-Tcl_Interp *Tcl_CreateInterp();
+extern Tcl_VarTraceProc traced_myiphostname;
 
 int expmem_tcl()
 {
@@ -115,7 +116,7 @@ static void botnet_change(char *new)
     } else {
       if (botnetnick[0])
         putlog(LOG_MISC, "*", "* IDENTITY CHANGE: %s -> %s", botnetnick, new);
-      strcpy(botnetnick, new);
+      set_botnetnick(new);
     }
   }
 }
@@ -183,7 +184,7 @@ static char *tcl_eggint(ClientData cdata, Tcl_Interp *irp,
   if (flags & (TCL_TRACE_READS | TCL_TRACE_UNSETS)) {
     /* Special cases */
     if ((int *) ii->var == &conmask)
-      strcpy(s1, masktype(conmask));
+      strncpyz(s1, masktype(conmask), sizeof s1);
     else if ((int *) ii->var == &default_flags) {
       struct flag_record fr = { FR_GLOBAL, 0, 0, 0, 0, 0 };
       fr.global = default_flags;
@@ -219,20 +220,22 @@ static char *tcl_eggint(ClientData cdata, Tcl_Interp *irp,
         int p = oatoi(s);
 
         if (p <= 0)
-          return "invalid userfile permissions";
+          return "Invalid userfile permissions";
         userfile_perm = p;
       } else if ((ii->ro == 2) || ((ii->ro == 1) && protect_readonly))
-        return "read-only variable";
+        return "Read-only variable";
       else {
         if (Tcl_ExprLong(interp, s, &l) == TCL_ERROR)
-          return "variable must have integer value";
+          return "Variable must have integer value";
         if ((int *) ii->var == &max_socks) {
           if (l < threaddata()->MAXSOCKS)
-            return "you can't DECREASE max-socks below current usage";
+            return "Decreasing max-socks requires a restart";
           max_socks = l;
         } else if ((int *) ii->var == &max_logs) {
+          if (l < 5)
+            return "ERROR: max-logs cannot be less than 5";
           if (l < max_logs)
-            return "you can't DECREASE max-logs";
+            return "ERROR: Decreasing max-logs requires a restart";
           max_logs = l;
           init_misc();
         } else
@@ -272,28 +275,7 @@ static char *tcl_eggstr(ClientData cdata, Tcl_Interp *irp,
       Tcl_SetVar2(interp, name1, name2, st->str, TCL_GLOBAL_ONLY);
       return "read-only variable";
     }
-#ifdef USE_TCL_BYTE_ARRAYS
-#  undef malloc
-#  undef free
-    {
-      Tcl_Obj *obj;
-      unsigned char *bytes;
-      int len;
-
-      obj = Tcl_GetVar2Ex(interp, name1, name2, 0);
-      if (!obj)
-        return NULL;
-      len = 0;
-      bytes = Tcl_GetByteArrayFromObj(obj, &len);
-      if (!bytes)
-        return NULL;
-      s = malloc(len + 1);
-      egg_memcpy(s, bytes, len);
-      s[len] = 0;
-    }
-#else
     s = (char *) Tcl_GetVar2(interp, name1, name2, 0);
-#endif /* USE_TCL_BYTE_ARRAYS */
     if (s != NULL) {
       if (strlen(s) > abs(st->max))
         s[abs(st->max)] = 0;
@@ -313,9 +295,6 @@ static char *tcl_eggstr(ClientData cdata, Tcl_Interp *irp,
         if (st->str[strlen(st->str) - 1] != '/')
           strcat(st->str, "/");
       }
-#ifdef USE_TCL_BYTE_ARRAYS
-      free(s);
-#endif /* USE_TCL_BYTE_ARRAYS */
     }
     return NULL;
   }
@@ -324,66 +303,6 @@ static char *tcl_eggstr(ClientData cdata, Tcl_Interp *irp,
 /* Add/remove tcl commands
  */
 
-#ifdef USE_TCL_BYTE_ARRAYS
-static int utf_converter(ClientData cdata, Tcl_Interp *myinterp, int objc,
-                         Tcl_Obj *CONST objv[])
-{
-  char **strings, *byteptr;
-  int i, len, retval, diff;
-  void **callback_data;
-  Function func;
-  ClientData cd;
-
-  objc += 5;
-  strings = (char **) nmalloc(sizeof(char *) * objc);
-  egg_memset(strings, 0, sizeof(char *) * objc);
-  diff = utftot;
-  utftot += sizeof(char *) * objc;
-  objc -= 5;
-  for (i = 0; i < objc; i++) {
-    byteptr = (char *) Tcl_GetByteArrayFromObj(objv[i], &len);
-    strings[i] = (char *) nmalloc(len + 1);
-    utftot += len + 1;
-    strncpy(strings[i], byteptr, len);
-    strings[i][len] = 0;
-  }
-  callback_data = (void **) cdata;
-  func = (Function) callback_data[0];
-  cd = (ClientData) callback_data[1];
-  diff -= utftot;
-  retval = func(cd, myinterp, objc, strings);
-  for (i = 0; i < objc; i++)
-    nfree(strings[i]);
-  nfree(strings);
-  utftot += diff;
-  return retval;
-}
-
-void cmd_delete_callback(ClientData cdata)
-{
-  nfree(cdata);
-  clientdata_stuff -= sizeof(void *) * 2;
-}
-#endif /* USE_TCL_BYTE_ARRAYS */
-
-#ifdef USE_TCL_BYTE_ARRAYS
-void add_tcl_commands(tcl_cmds *table)
-{
-  void **cdata;
-
-  while (table->name) {
-    cdata = (void **) nmalloc(sizeof(void *) * 2);
-    clientdata_stuff += sizeof(void *) * 2;
-    cdata[0] = (void *)table->func;
-    cdata[1] = NULL;
-    Tcl_CreateObjCommand(interp, table->name, utf_converter, (ClientData) cdata,
-                         cmd_delete_callback);
-    table++;
-  }
-}
-
-#else /* USE_TCL_BYTE_ARRAYS */
-
 void add_tcl_commands(tcl_cmds *table)
 {
   int i;
@@ -391,25 +310,6 @@ void add_tcl_commands(tcl_cmds *table)
   for (i = 0; table[i].name; i++)
     Tcl_CreateCommand(interp, table[i].name, table[i].func, NULL, NULL);
 }
-#endif /* USE_TCL_BYTE_ARRAYS */
-
-#ifdef USE_TCL_BYTE_ARRAYS
-void add_cd_tcl_cmds(cd_tcl_cmd *table)
-{
-  void **cdata;
-
-  while (table->name) {
-    cdata = nmalloc(sizeof(void *) * 2);
-    clientdata_stuff += sizeof(void *) * 2;
-    cdata[0] = (void *)table->callback;
-    cdata[1] = table->cdata;
-    Tcl_CreateObjCommand(interp, table->name, utf_converter, (ClientData) cdata,
-                         cmd_delete_callback);
-    table++;
-  }
-}
-
-#else /* USE_TCL_BYTE_ARRAYS */
 
 void add_cd_tcl_cmds(cd_tcl_cmd *table)
 {
@@ -419,7 +319,6 @@ void add_cd_tcl_cmds(cd_tcl_cmd *table)
     table++;
   }
 }
-#endif /* USE_TCL_BYTE_ARRAYS */
 
 void rem_tcl_commands(tcl_cmds *table)
 {
@@ -437,7 +336,6 @@ void rem_cd_tcl_cmds(cd_tcl_cmd *table)
   }
 }
 
-#ifdef USE_TCL_OBJ
 void add_tcl_objcommands(tcl_cmds *table)
 {
   int i;
@@ -446,17 +344,12 @@ void add_tcl_objcommands(tcl_cmds *table)
     Tcl_CreateObjCommand(interp, table[i].name, table[i].func, (ClientData) 0,
                          NULL);
 }
-#endif
 
 /* Get the current tcl result string. */
 const char *tcl_resultstring()
 {
   const char *result;
-#ifdef USE_TCL_OBJ
   result = Tcl_GetStringResult(interp);
-#else
-  result = interp->result;
-#endif
   return result;
 }
 
@@ -470,11 +363,7 @@ int tcl_resultempty() {
 int tcl_resultint()
 {
   int result;
-#ifdef USE_TCL_OBJ
   if (Tcl_GetIntFromObj(NULL, Tcl_GetObjResult(interp), &result) != TCL_OK)
-#else
-  if (Tcl_GetInt(NULL, interp->result, &result) != TCL_OK)
-#endif
     result = 0;
   return result;
 }
@@ -485,15 +374,24 @@ static tcl_strings def_tcl_strings[] = {
   {"motd",            motdfile,       120,           STR_PROTECT},
   {"admin",           admin,          120,                     0},
   {"help-path",       helpdir,        120, STR_DIR | STR_PROTECT},
-  {"temp-path",       tempdir,        120, STR_DIR | STR_PROTECT},
   {"text-path",       textdir,        120, STR_DIR | STR_PROTECT},
+#ifdef TLS
+  {"ssl-capath",      tls_capath,     120, STR_DIR | STR_PROTECT},
+  {"ssl-cafile",      tls_cafile,     120,           STR_PROTECT},
+  {"ssl-ciphers",     tls_ciphers,    120,           STR_PROTECT},
+  {"ssl-privatekey",  tls_keyfile,    120,           STR_PROTECT},
+  {"ssl-certificate", tls_certfile,   120,           STR_PROTECT},
+#endif
 #ifndef STATIC
   {"mod-path",        moddir,         120, STR_DIR | STR_PROTECT},
 #endif
   {"notify-newusers", notify_new,     120,                     0},
   {"owner",           owner,          120,           STR_PROTECT},
-  {"my-ip",           myip,           120,                     0},
-  {"my-hostname",     hostname,       120,                     0},
+  {"vhost4",          vhost,          120,                     0},
+#ifdef IPV6
+  {"vhost6",          vhost6,         120,                     0},
+#endif
+  {"listen-addr",     listen_ip,      120,                     0},
   {"network",         network,        40,                      0},
   {"whois-fields",    whois_fields,   1024,                    0},
   {"nat-ip",          natip,          120,                     0},
@@ -505,12 +403,20 @@ static tcl_strings def_tcl_strings[] = {
   {"logfile-suffix",  logfile_suffix, 20,                      0},
   {"timestamp-format",log_ts,         32,                      0},
   {"pidfile",         pid_file,       120,           STR_PROTECT},
+  {"configureargs",   EGG_AC_ARGS,    0,             STR_PROTECT},
   {NULL,              NULL,           0,                       0}
 };
 
 static tcl_ints def_tcl_ints[] = {
   {"ignore-time",           &ignore_time,          0},
   {"handlen",               &handlen,              2},
+#ifdef TLS
+  {"ssl-chain-depth",       &tls_maxdepth,         0},
+  {"ssl-verify-dcc",        &tls_vfydcc,           0},
+  {"ssl-verify-clients",    &tls_vfyclients,       0},
+  {"ssl-verify-bots",       &tls_vfybots,          0},
+  {"ssl-cert-auth",         &tls_auth,             0},
+#endif
   {"dcc-flood-thr",         &dcc_flood_thr,        0},
   {"hourly-updates",        &notify_users_at,      0},
   {"switch-logfiles-at",    &switch_logfiles_at,   0},
@@ -525,8 +431,6 @@ static tcl_ints def_tcl_ints[] = {
   {"console",               &conmask,              0},
   {"default-flags",         &default_flags,        0},
   {"numversion",            &egg_numver,           2},
-  {"die-on-sighup",         &die_on_sighup,        1},
-  {"die-on-sigterm",        &die_on_sigterm,       1},
   {"remote-boots",          &remote_boots,         1},
   {"max-socks",             &max_socks,            0},
   {"max-logs",              &max_logs,             0},
@@ -535,7 +439,6 @@ static tcl_ints def_tcl_ints[] = {
   {"raw-log",               &raw_log,              1},
   {"protect-telnet",        &protect_telnet,       0},
   {"dcc-sanitycheck",       &dcc_sanitycheck,      0},
-  {"sort-users",            &sort_users,           0},
   {"ident-timeout",         &identtimeout,         0},
   {"share-unlinks",         &share_unlinks,        0},
   {"log-time",              &shtime,               0},
@@ -553,10 +456,9 @@ static tcl_ints def_tcl_ints[] = {
   {"copy-to-tmp",           &copy_to_tmp,          0},
   {"quiet-reject",          &quiet_reject,         0},
   {"cidr-support",          &cidr_support,         0},
-  {"strict-servernames",    &strict_servernames,   0}, /* compat */
-  {"enable-simul",          &enable_simul,         0}, /* compat */
-  {"debug-output",          &debug_output,         0}, /* compat */
-  {"use-console-r",         &use_console_r,        0}, /* compat */
+#ifdef IPV6
+  {"prefer-ipv6",           &pref_af,              0},
+#endif
   {NULL,                    NULL,                  0}
 };
 
@@ -574,60 +476,26 @@ static void init_traces()
   add_tcl_coups(def_tcl_coups);
   add_tcl_strings(def_tcl_strings);
   add_tcl_ints(def_tcl_ints);
+  Tcl_TraceVar(interp, "my-ip", TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS, traced_myiphostname, NULL);
+  Tcl_TraceVar(interp, "my-hostname", TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS, traced_myiphostname, NULL);
 }
 
-/* tcleventcallback that's called after executing Tcl code,
- * used by do_tcl(). Just logs Tcl errors.
- */
-void bgtclcallback(char *context, char *script, int code,
-                   const char *result, int dofree) {
-#ifdef USE_TCL_ENCODING
-  Tcl_DString dstr;
-#endif
-
-  if (code == TCL_ERROR) {
-#ifdef USE_TCL_ENCODING
-    /* properly convert string to system encoding. */
-    Tcl_DStringInit(&dstr);
-    Tcl_UtfToExternalDString(NULL, result, -1, &dstr);
-    result = Tcl_DStringValue(&dstr);
-#endif
-    putlog(LOG_MISC, "*", "Tcl error in script for '%s':", context);
-    putlog(LOG_MISC, "*", "%s", result);
-#ifdef USE_TCL_ENCODING
-  Tcl_DStringFree(&dstr);
-#endif
-  }
-  if (dofree) {
-    nfree(context);
-    nfree(script);
-  }
-}
-
-/* Generic function (exported to modules) to execute Tcl code.
- * context is used to display Tcl errors (Error in '..')
- *
- * Schedules execution if defined(REPLACE_NOTIFIER) to bottom of mainloop,
- * where it's safe to recurse eventually (vwait/update).
- */
-void do_tcl(char *context, char *script) {
-  do_tcl_async(context, script, bgtclcallback);
-}
-
-/* Evaluates Tcl code.
- *
- * Should only be called from reentrant functions (can recurse),
- * use do_tcl/do_tcl_async instead.
- */
-void do_tcl_sync(char *context, char *script, tcleventcallback cb, int dofree) {
-  int x;
-
-  x = Tcl_Eval(interp, script);
-  cb(context, script, x, tcl_resultstring(), dofree);
+void kill_tcl()
+{
+  rem_tcl_coups(def_tcl_coups);
+  rem_tcl_strings(def_tcl_strings);
+  rem_tcl_ints(def_tcl_ints);
+  Tcl_UntraceVar(interp, "my-ip", TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS, traced_myiphostname, NULL);
+  Tcl_UntraceVar(interp, "my-hostname", TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS, traced_myiphostname, NULL);
+  kill_bind();
+  Tcl_DeleteInterp(interp);
 }
 
 extern tcl_cmds tcluser_cmds[], tcldcc_cmds[], tclmisc_cmds[],
        tclmisc_objcmds[], tcldns_cmds[];
+#ifdef TLS
+extern tcl_cmds tcltls_cmds[];
+#endif
 
 #ifdef REPLACE_NOTIFIER
 /* The tickle_*() functions replace the Tcl Notifier
@@ -694,27 +562,6 @@ struct threaddata *threaddata()
   return td;
 }
 
-/* Add a script snippet to the queue of Tcl events to execute.
- * See tclegg.h for the definition of the callback.
- */
-void do_tcl_async(char *context, char *script, tcleventcallback callback) {
-  tclevent_t *ev, *e = tclevents;
-
-  ev = nmalloc(sizeof(tclevent_t));
-  ev->script = nstrdup(script);
-  ev->context = nstrdup(context);
-  ev->callback = callback;
-  ev->next = NULL;
-
-  if (!e)
-    tclevents = ev;
-  else {
-    while (e->next)
-      e = e->next;
-    e->next = ev;
-  }
-}
-
 #else /* REPLACE_NOTIFIER */
 
 int tclthreadmainloop() { return 0; }
@@ -725,11 +572,6 @@ struct threaddata *threaddata()
   return &tsd;
 }
 
-/* Without the replacement notifier, this just executes the script. */
-void do_tcl_async(char *context, char *script, tcleventcallback callback) {
-  do_tcl_sync(context, script, callback, 0);
-}
-
 #endif /* REPLACE_NOTIFIER */
 
 int init_threaddata(int mainthread)
@@ -760,15 +602,9 @@ void init_tcl(int argc, char **argv)
   Tcl_NotifierProcs notifierprocs;
 #endif /* REPLACE_NOTIFIER */
 
-#ifdef USE_TCL_ENCODING
   const char *encoding;
-  int i;
-  char *langEnv;
-#endif /* USE_TCL_ENCODING */
-#ifdef USE_TCL_PACKAGE
-  int j;
-  char pver[1024] = "";
-#endif /* USE_TCL_PACKAGE */
+  int i, j;
+  char *langEnv, pver[1024] = "";
 
 #ifdef REPLACE_NOTIFIER
   egg_bzero(&notifierprocs, sizeof(notifierprocs));
@@ -785,13 +621,11 @@ void init_tcl(int argc, char **argv)
 /* This must be done *BEFORE* Tcl_SetSystemEncoding(),
  * or Tcl_SetSystemEncoding() will cause a segfault.
  */
-#ifdef USE_TCL_FINDEXEC
   /* This is used for 'info nameofexecutable'.
    * The filename in argv[0] must exist in a directory listed in
    * the environment variable PATH for it to register anything.
    */
   Tcl_FindExecutable(argv[0]);
-#endif /* USE_TCL_FINDEXEC */
 
   /* Initialize the interpreter */
   interp = Tcl_CreateInterp();
@@ -809,7 +643,6 @@ void init_tcl(int argc, char **argv)
   Tcl_SetServiceMode(TCL_SERVICE_ALL);
 
 /* Code based on Tcl's TclpSetInitialEncodings() */
-#ifdef USE_TCL_ENCODING
   /* Determine the current encoding from the LC_* or LANG environment
    * variables.
    */
@@ -880,9 +713,7 @@ resetPath:
   /* Keep the iso8859-1 encoding preloaded.  The IO package uses it for
    * gets on a binary channel. */
   Tcl_GetEncoding(NULL, "iso8859-1");
-#endif /* USE_TCL_ENCODING */
 
-#ifdef USE_TCL_PACKAGE
   /* Add eggdrop to Tcl's package list */
   for (j = 0; j <= strlen(egg_version); j++) {
     if ((egg_version[j] == ' ') || (egg_version[j] == '+'))
@@ -890,7 +721,6 @@ resetPath:
     pver[strlen(pver)] = egg_version[j];
   }
   Tcl_PkgProvide(interp, "eggdrop", pver);
-#endif /* USE_TCL_PACKAGE */
 
   /* Initialize binds and traces */
   init_bind();
@@ -900,31 +730,34 @@ resetPath:
   add_tcl_commands(tcluser_cmds);
   add_tcl_commands(tcldcc_cmds);
   add_tcl_commands(tclmisc_cmds);
-#ifdef USE_TCL_OBJ
   add_tcl_objcommands(tclmisc_objcmds);
-#endif
   add_tcl_commands(tcldns_cmds);
-}
-
-void kill_tcl()
-{
-#ifdef REPLACE_NOTIFIER
-  tclevent_t *ne, *e = tclevents;
-
-  while (e) {
-    ne = e->next;
-    e->callback(e->context, e->script, -1, "", 1);
-    nfree(e);
-    e = ne;
-  }
+#ifdef TLS
+  add_tcl_commands(tcltls_cmds);
 #endif
-  rem_tcl_coups(def_tcl_coups);
-  rem_tcl_strings(def_tcl_strings);
-  rem_tcl_ints(def_tcl_ints);
-  kill_bind();
-  Tcl_DeleteInterp(interp);
 }
 
+void do_tcl(char *whatzit, char *script)
+{
+  int code;
+  char *result;
+  Tcl_DString dstr;
+
+  code = Tcl_Eval(interp, script);
+
+  /* properly convert string to system encoding. */
+  Tcl_DStringInit(&dstr);
+  Tcl_UtfToExternalDString(NULL, tcl_resultstring(), -1, &dstr);
+  result = Tcl_DStringValue(&dstr);
+
+  if (code != TCL_OK) {
+    putlog(LOG_MISC, "*", "Tcl error in script for '%s':", whatzit);
+    putlog(LOG_MISC, "*", "%s", result);
+    Tcl_BackgroundError(interp);
+  }
+
+  Tcl_DStringFree(&dstr);
+}
 
 /* Interpret tcl file fname.
  *
@@ -934,9 +767,7 @@ int readtclprog(char *fname)
 {
   int code;
   EGG_CONST char *result;
-#ifdef USE_TCL_ENCODING
   Tcl_DString dstr;
-#endif
 
   if (!file_readable(fname))
     return 0;
@@ -944,25 +775,22 @@ int readtclprog(char *fname)
   code = Tcl_EvalFile(interp, fname);
   result = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
 
-#ifdef USE_TCL_ENCODING
   /* properly convert string to system encoding. */
   Tcl_DStringInit(&dstr);
   Tcl_UtfToExternalDString(NULL, result, -1, &dstr);
   result = Tcl_DStringValue(&dstr);
-#endif
 
   if (code != TCL_OK) {
     putlog(LOG_MISC, "*", "Tcl error in file '%s':", fname);
     putlog(LOG_MISC, "*", "%s", result);
+    Tcl_BackgroundError(interp);
     code = 0; /* JJM: refactored to remove premature return */
   } else {
     /* Refresh internal variables */
     code = 1;
   }
 
-#ifdef USE_TCL_ENCODING
   Tcl_DStringFree(&dstr);
-#endif
 
   return code;
 }
@@ -1088,10 +916,8 @@ void rem_tcl_coups(tcl_coups *list)
 */
 int tcl_threaded()
 {
-#ifdef HAVE_TCL_GETCURRENTTHREAD
   if (Tcl_GetCurrentThread() != (Tcl_ThreadId)0)
     return 1;
-#endif
 
   return 0;
 }
