@@ -1,12 +1,10 @@
 /*
  * tcldcc.c -- handles:
  *   Tcl stubs for the dcc commands
- *
- * $Id: tcldcc.c,v 1.70 2011/02/13 14:19:33 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,12 +24,17 @@
 #include "main.h"
 #include "tandem.h"
 #include "modules.h"
+#include <errno.h>
 
 extern Tcl_Interp *interp;
 extern tcl_timer_t *timer, *utimer;
 extern struct dcc_t *dcc;
 extern char botnetnick[];
-extern int dcc_total, backgrd, parties, make_userfile, do_restart, remote_boots, max_dcc;
+extern int dcc_total, backgrd, parties, make_userfile, do_restart, remote_boots, max_dcc, conmask;
+#ifdef TLS
+extern int tls_vfydcc;
+extern sock_list *socklist;
+#endif
 extern party_t *party;
 extern tand_t *tandbot;
 extern time_t now;
@@ -277,13 +280,12 @@ static int tcl_dccputchan STDVAR
   return TCL_OK;
 }
 
-static int tcl_console STDVAR
+static int tcl_do_console(Tcl_Interp *irp, ClientData cd, int argc,
+    char **argv, int reset)
 {
   int i, j, pls, arg;
   module_entry *me;
 
-  BADARGS(2, 4, " idx ?channel? ?console-modes?");
-
   i = findidx(atoi(argv[1]));
   if (i < 0 || dcc[i].type != &DCC_CHAT) {
     Tcl_AppendResult(irp, "invalid idx", NULL);
@@ -292,8 +294,8 @@ static int tcl_console STDVAR
   pls = 1;
 
   for (arg = 2; arg < argc; arg++) {
-    if (argv[arg][0] && ((strchr(CHANMETA, argv[arg][0]) != NULL) ||
-        (argv[arg][0] == '*'))) {
+    if (argv[arg][0] && !reset && ((strchr(CHANMETA, argv[arg][0]) 
+        != NULL) || (argv[arg][0] == '*'))) {
       if ((argv[arg][0] != '*') && (!findchan_by_dname(argv[arg]))) {
         /* If we dont find the channel, and it starts with a +, assume it
          * should be the console flags to set. */
@@ -304,23 +306,28 @@ static int tcl_console STDVAR
       }
       strncpyz(dcc[i].u.chat->con_chan, argv[arg], 81);
     } else {
-      if ((argv[arg][0] != '+') && (argv[arg][0] != '-'))
+      if (!reset && (argv[arg][0] != '+') && (argv[arg][0] != '-'))
         dcc[i].u.chat->con_flags = 0;
     do_console_flags:
-      for (j = 0; j < strlen(argv[arg]); j++) {
-        if (argv[arg][j] == '+')
-          pls = 1;
-        else if (argv[arg][j] == '-')
-          pls = -1;
-        else {
-          char s[2];
+      if (reset) {
+        dcc[i].u.chat->con_flags =
+            (dcc[i].user && (dcc[i].user->flags & USER_MASTER) ? conmask : 0);
+      } else {
+        for (j = 0; j < strlen(argv[arg]); j++) {
+          if (argv[arg][j] == '+')
+            pls = 1;
+          else if (argv[arg][j] == '-')
+            pls = -1;
+          else {
+            char s[2];
 
-          s[0] = argv[arg][j];
-          s[1] = 0;
-          if (pls == 1)
-            dcc[i].u.chat->con_flags |= logmodes(s);
-          else
-            dcc[i].u.chat->con_flags &= ~logmodes(s);
+            s[0] = argv[arg][j];
+            s[1] = 0;
+            if (pls == 1)
+              dcc[i].u.chat->con_flags |= logmodes(s);
+            else
+              dcc[i].u.chat->con_flags &= ~logmodes(s);
+          }
         }
       }
     }
@@ -336,6 +343,19 @@ static int tcl_console STDVAR
   return TCL_OK;
 }
 
+static int tcl_console STDVAR
+{
+  BADARGS(2, 4, " idx ?channel? ?console-modes?");
+  return tcl_do_console(irp, cd, argc, argv, 0);
+}
+
+static int tcl_resetconsole STDVAR
+{
+  BADARGS(2, 2, " idx");
+  return tcl_do_console(irp, cd, argc, argv, 1);
+}
+
+
 static int tcl_strip STDVAR
 {
   int i, j, pls, arg;
@@ -854,7 +874,7 @@ static int tcl_unlink STDVAR
 
 static int tcl_connect STDVAR
 {
-  int i, z, sock;
+  int i, sock;
   char s[81];
 
   BADARGS(3, 3, " hostname port");
@@ -863,25 +883,39 @@ static int tcl_connect STDVAR
     Tcl_AppendResult(irp, "out of dcc table space", NULL);
     return TCL_ERROR;
   }
-  sock = getsock(0);
 
-  if (sock < 0) {
-    Tcl_AppendResult(irp, MISC_NOFREESOCK, NULL);
-    return TCL_ERROR;
-  }
-  z = open_telnet_raw(sock, argv[1], atoi(argv[2]));
-  if (z < 0) {
-    killsock(sock);
-    if (z == -2)
-      strncpyz(s, "DNS lookup failed", sizeof s);
-    else
-      neterror(s);
-    Tcl_AppendResult(irp, s, NULL);
-    return TCL_ERROR;
-  }
   i = new_dcc(&DCC_SOCKET, 0);
-  dcc[i].sock = sock;
-  dcc[i].port = atoi(argv[2]);
+  if (i < 0) {
+    Tcl_AppendResult(irp, "Could not allocate socket.", NULL);
+    return TCL_ERROR;
+  }
+  sock = open_telnet(i, argv[1], atoi(argv[2]));
+  if (sock < 0) {
+    switch (sock) {
+      case -3:
+        Tcl_AppendResult(irp, MISC_NOFREESOCK, NULL);
+        break;
+      case -2:
+        Tcl_AppendResult(irp, "DNS lookup failed", NULL);
+        break;
+      default:
+        Tcl_AppendResult(irp, strerror(errno), NULL);
+    }
+    lostdcc(i);
+    return TCL_ERROR;
+  }
+#ifdef TLS
+  if (*argv[2] == '+') {
+    if (ssl_handshake(sock, TLS_CONNECT, 0, LOG_MISC, NULL, NULL)) {
+      killsock(sock);
+      lostdcc(i);
+      strncpyz(s, "Failed to establish a TLS session", sizeof s);
+      Tcl_AppendResult(irp, s, NULL);
+      return TCL_ERROR;
+    } else
+      dcc[i].ssl = 1;
+  }
+#endif
   strcpy(dcc[i].nick, "*");
   strncpyz(dcc[i].host, argv[1], UHOSTMAX);
   egg_snprintf(s, sizeof s, "%d", sock);
@@ -937,31 +971,41 @@ static int tcl_listen STDVAR
     }
     /* Try to grab port */
     j = port + 20;
-    i = -1;
+    i = -2;
     while (port < j && i < 0) {
       i = open_listen(&port);
       if (i == -1)
-        port++;
-      else if (i == -2)
         break;
+      else if (i == -2)
+        port++;
     }
+
     if (i == -1) {
-      egg_snprintf(msg, sizeof msg, "Couldn't listen on port '%d' on the "
-                   "given address. Please make sure 'my-ip' is set correctly, "
-                   "or try a different port.", realport);
+      egg_snprintf(msg, sizeof msg, "Couldn't listen on port '%d' on the given "
+                   "address: %s", realport, strerror(errno));
       Tcl_AppendResult(irp, msg, NULL);
       return TCL_ERROR;
     } else if (i == -2) {
-      Tcl_AppendResult(irp, "Couldn't assign the requested IP. Please make "
-                       "sure 'my-ip' is set properly.", NULL);
+      egg_snprintf(msg, sizeof msg, "Couldn't listen on port '%d' on the given "
+                   "address. Please make sure 'listen-addr' is set properly"
+                   " or try choosing a different port.", realport);
+      Tcl_AppendResult(irp, msg, NULL);
       return TCL_ERROR;
     }
     idx = new_dcc(&DCC_TELNET, 0);
-    dcc[idx].addr = iptolong(getmyip());
+    dcc[idx].sockname.addrlen = sizeof(dcc[idx].sockname.addr);
+    getsockname(i, &dcc[idx].sockname.addr.sa, &dcc[idx].sockname.addrlen);
+    dcc[idx].sockname.family = dcc[idx].sockname.addr.sa.sa_family;
     dcc[idx].port = port;
     dcc[idx].sock = i;
     dcc[idx].timeval = now;
   }
+#ifdef TLS
+  if (*argv[1] == '+')
+    dcc[idx].ssl = 1;
+  else
+    dcc[idx].ssl = 0;
+#endif
   /* script? */
   if (!strcmp(argv[2], "script")) {
     strcpy(dcc[idx].nick, "(script)");
@@ -1013,7 +1057,9 @@ static int tcl_listen STDVAR
   }
   pmap->realport = realport;
   pmap->mappedto = port;
-  putlog(LOG_MISC, "*", "Listening at telnet port %d (%s).", port, argv[2]);
+
+  putlog(LOG_MISC, "*", "Listening for telnet connections on %s:%d (%s).", iptostr(&dcc[idx].sockname.addr.sa), port, argv[2]);
+
   return TCL_OK;
 }
 
@@ -1093,31 +1139,31 @@ static int tcl_traffic STDVAR
   unsigned long in_total_today, in_total;
 
   /* IRC traffic */
-  sprintf(buf, "irc %ld %ld %ld %ld", itraffic_irc_today, itraffic_irc +
+  sprintf(buf, "irc %lu %lu %lu %lu", itraffic_irc_today, itraffic_irc +
           itraffic_irc_today, otraffic_irc_today,
           otraffic_irc + otraffic_irc_today);
   Tcl_AppendElement(irp, buf);
 
   /* Botnet traffic */
-  sprintf(buf, "botnet %ld %ld %ld %ld", itraffic_bn_today, itraffic_bn +
+  sprintf(buf, "botnet %lu %lu %lu %lu", itraffic_bn_today, itraffic_bn +
           itraffic_bn_today, otraffic_bn_today,
           otraffic_bn + otraffic_bn_today);
   Tcl_AppendElement(irp, buf);
 
   /* Partyline */
-  sprintf(buf, "partyline %ld %ld %ld %ld", itraffic_dcc_today, itraffic_dcc +
+  sprintf(buf, "partyline %lu %lu %lu %lu", itraffic_dcc_today, itraffic_dcc +
           itraffic_dcc_today, otraffic_dcc_today,
           otraffic_dcc + otraffic_dcc_today);
   Tcl_AppendElement(irp, buf);
 
   /* Transfer */
-  sprintf(buf, "transfer %ld %ld %ld %ld", itraffic_trans_today,
+  sprintf(buf, "transfer %lu %lu %lu %lu", itraffic_trans_today,
           itraffic_trans + itraffic_trans_today, otraffic_trans_today,
           otraffic_trans + otraffic_trans_today);
   Tcl_AppendElement(irp, buf);
 
   /* Misc traffic */
-  sprintf(buf, "misc %ld %ld %ld %ld", itraffic_unknown_today,
+  sprintf(buf, "misc %lu %lu %lu %lu", itraffic_unknown_today,
           itraffic_unknown + itraffic_unknown_today, otraffic_unknown_today,
           otraffic_unknown + otraffic_unknown_today);
   Tcl_AppendElement(irp, buf);
@@ -1133,7 +1179,7 @@ static int tcl_traffic STDVAR
                     otraffic_unknown_today;
   out_total = out_total_today + otraffic_irc + otraffic_bn + otraffic_dcc +
               otraffic_trans + otraffic_unknown;
-  sprintf(buf, "total %ld %ld %ld %ld", in_total_today, in_total,
+  sprintf(buf, "total %lu %lu %lu %lu", in_total_today, in_total,
           out_total_today, out_total);
   Tcl_AppendElement(irp, buf);
   return TCL_OK;
@@ -1150,6 +1196,7 @@ tcl_cmds tcldcc_cmds[] = {
   {"setchan",           tcl_setchan},
   {"dccputchan",     tcl_dccputchan},
   {"console",           tcl_console},
+  {"resetconsole", tcl_resetconsole},
   {"strip",               tcl_strip},
   {"echo",                 tcl_echo},
   {"page",                 tcl_page},
