@@ -6,12 +6,10 @@
  *   adding/removing new binding tables
  *   (non-Tcl) procedure lookups for msg/dcc/file commands
  *   (Tcl) binding internal procedures to msg/dcc/file commands
- *
- * $Id: tclhash.c,v 1.72 2011/07/09 15:07:48 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -41,7 +39,11 @@ extern time_t now;
 p_tcl_bind_list bind_table_list;
 p_tcl_bind_list H_chat, H_act, H_bcst, H_chon, H_chof, H_load, H_unld, H_link,
                 H_disc, H_dcc, H_chjn, H_chpt, H_bot, H_time, H_nkch, H_away,
-                H_note, H_filt, H_event, H_cron, H_log = NULL;
+                H_note, H_filt, H_event, H_die, H_cron, H_log = NULL;
+#ifdef TLS
+p_tcl_bind_list H_tls = NULL;
+static int builtin_idx();
+#endif
 
 static int builtin_2char();
 static int builtin_3char();
@@ -109,7 +111,7 @@ static inline void tcl_bind_list_delete(tcl_bind_list_t *tl)
   nfree(tl);
 }
 
-inline void garbage_collect_tclhash(void)
+void garbage_collect_tclhash(void)
 {
   tcl_bind_list_t *tl, *tl_next, *tl_prev;
   tcl_bind_mask_t *tm, *tm_next, *tm_prev;
@@ -237,7 +239,11 @@ void init_bind(void)
   H_away = add_bind_table("away", HT_STACKABLE, builtin_chat);
   H_act = add_bind_table("act", HT_STACKABLE, builtin_chat);
   H_event = add_bind_table("evnt", HT_STACKABLE, builtin_char);
+  H_die = add_bind_table("die", HT_STACKABLE, builtin_char);
   H_log = add_bind_table("log", HT_STACKABLE, builtin_log);
+#ifdef TLS
+  H_tls = add_bind_table("tls", HT_STACKABLE, builtin_idx);
+#endif
   add_builtins(H_dcc, C_dcc);
   Context;
 }
@@ -687,6 +693,19 @@ static int builtin_log STDVAR
   return TCL_OK;
 }
 
+#ifdef TLS
+static int builtin_idx STDVAR
+{
+  Function F = (Function) cd;
+
+  BADARGS(2, 2, " idx");
+
+  CHECKVALIDITY(builtin_idx);
+  F(atoi(argv[1]));
+  return TCL_OK;
+}
+#endif
+
 /* Trigger (execute) a Tcl proc
  *
  * Note: This is INLINE code for check_tcl_bind().
@@ -723,6 +742,7 @@ static inline int trigger_bind(const char *proc, const char *param,
   if (x == TCL_ERROR) {
     /* FIXME: we really should be able to log longer errors */
     putlog(LOG_MISC, "*", "Tcl error [%s]: %.*s", proc, 400, tcl_resultstring());
+    Tcl_BackgroundError(interp);
 
     return BIND_EXECUTED;
   }
@@ -850,13 +870,9 @@ int check_tcl_bind(tcl_bind_list_t *tl, const char *match,
            */
 
           tc->hits++;
-
           x = trigger_bind(tc->func_name, param, tm->mask);
 
-          if ((tl->flags & HT_DELETED) || (tm->flags & TBM_DELETED)) {
-            /* This bind or bind type was deleted within trigger_bind() */
-            return x;
-          } else if (match_type & BIND_ALTER_ARGS) {
+          if (match_type & BIND_ALTER_ARGS) {
             if (tcl_resultempty())
               return x;
           } else if ((match_type & BIND_STACKRET) && x == BIND_EXEC_LOG) {
@@ -867,10 +883,9 @@ int check_tcl_bind(tcl_bind_list_t *tl, const char *match,
             if (!result)
               result = x;
             continue;
-          } else if ((match_type & BIND_WANTRET) && x == BIND_EXEC_LOG) {
+          } else if ((match_type & BIND_WANTRET) && x == BIND_EXEC_LOG)
             /* Return immediately if any commands return 1 */
             return x;
-          }
         }
       }
     }
@@ -1167,22 +1182,53 @@ void check_tcl_cron(struct tm *tm)
 
 void check_tcl_event(const char *event)
 {
-  Tcl_SetVar(interp, "_event1", (char *) event, 0);
-  check_tcl_bind(H_event, event, 0, " $_event1", MATCH_EXACT | BIND_STACKABLE);
+  Tcl_SetVar(interp, "_event1", (char *) event, TCL_GLOBAL_ONLY);
+  check_tcl_bind(H_event, event, 0, " $::_event1",
+                 MATCH_EXACT | BIND_STACKABLE);
+}
+
+int check_tcl_signal(const char *event)
+{
+  int x;
+
+  Tcl_SetVar(interp, "_event1", (char *) event, TCL_GLOBAL_ONLY);
+  x = check_tcl_bind(H_event, event, 0, " $::_event1",
+                 MATCH_EXACT | BIND_STACKABLE | BIND_STACKRET);
+  return (x == BIND_EXEC_LOG);
+}
+
+void check_tcl_die(char *reason)
+{
+  Tcl_SetVar(interp, "_die1", reason, 0);
+  check_tcl_bind(H_die, reason, 0, " $_die1", MATCH_MASK | BIND_STACKABLE);
 }
 
 void check_tcl_log(int lv, char *chan, char *msg)
 {
   char mask[512];
 
-  snprintf(mask, sizeof mask, "%s %s", chan, msg);
-  Tcl_SetVar(interp, "_log1", masktype(lv), 0);
-  Tcl_SetVar(interp, "_log2", chan, 0);
-  Tcl_SetVar(interp, "_log3", msg, 0);
-  check_tcl_bind(H_log, mask, 0, " $_log1 $_log2 $_log3",
+  egg_snprintf(mask, sizeof mask, "%s %s", chan, msg);
+  Tcl_SetVar(interp, "_log1", masktype(lv), TCL_GLOBAL_ONLY);
+  Tcl_SetVar(interp, "_log2", chan, TCL_GLOBAL_ONLY);
+  Tcl_SetVar(interp, "_log3", msg, TCL_GLOBAL_ONLY);
+  check_tcl_bind(H_log, mask, 0, " $::_log1 $::_log2 $::_log3",
                  MATCH_MASK | BIND_STACKABLE);
 }
 
+#ifdef TLS
+int check_tcl_tls(int sock)
+{
+  int x;
+  char s[11];
+
+  egg_snprintf(s, sizeof s, "%d", sock);
+  Tcl_SetVar(interp, "_tls", s, 0);
+  x = check_tcl_bind(H_tls, s, 0, " $_tls", MATCH_MASK | BIND_STACKABLE |
+                     BIND_WANTRET);
+  return (x == BIND_EXEC_LOG);
+}
+#endif
+
 void tell_binds(int idx, char *par)
 {
   tcl_bind_list_t *tl, *tl_kind;
