@@ -1,12 +1,10 @@
 /*
  * tclmisc.c -- handles:
  *   Tcl stubs for everything else
- *
- * $Id: tclmisc.c,v 1.71 2011/02/13 14:19:33 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -52,7 +50,7 @@ extern char botnetnick[], quit_msg[];
 extern struct userrec *userlist;
 extern time_t now;
 extern module_entry *module_list;
-extern int max_logs;
+extern int max_logs, cache_hit, cache_miss;
 extern log_t *logs;
 extern Tcl_Interp *interp;
 
@@ -86,11 +84,11 @@ static int tcl_logfile STDVAR
     /* They just want a list of the logfiles and modes */
     for (i = 0; i < max_logs; i++)
       if (logs[i].filename != NULL) {
-        snprintf(s, sizeof s, "%s %s %s", masktype(logs[i].mask),
+        egg_snprintf(s, sizeof s, "%s %s %s", masktype(logs[i].mask),
                  logs[i].chname, logs[i].filename);
         Tcl_AppendElement(interp, s);
       }
-      return TCL_OK;
+    return TCL_OK;
   }
 
   BADARGS(4, 4, " ?logModes channel logFile?");
@@ -256,14 +254,19 @@ static int tcl_timer STDVAR
   unsigned long x;
   char s[16];
 
-  BADARGS(3, 3, " minutes command");
+  BADARGS(3, 4, " minutes command ?count?");
 
   if (atoi(argv[1]) < 0) {
     Tcl_AppendResult(irp, "time value must be positive", NULL);
     return TCL_ERROR;
   }
+  if (argc == 4 && atoi(argv[3]) < 0) {
+    Tcl_AppendResult(irp, "count value must be >= 0", NULL);
+    return TCL_ERROR;
+  }
   if (argv[2][0] != '#') {
-    x = add_timer(&timer, atoi(argv[1]), argv[2], 0L);
+    x = add_timer(&timer, atoi(argv[1]), (argc == 4 ? atoi(argv[3]) : 1),
+                  argv[2], 0L);
     egg_snprintf(s, sizeof s, "timer%lu", x);
     Tcl_AppendResult(irp, s, NULL);
   }
@@ -275,14 +278,19 @@ static int tcl_utimer STDVAR
   unsigned long x;
   char s[16];
 
-  BADARGS(3, 3, " seconds command");
+  BADARGS(3, 4, " seconds command ?count?");
 
   if (atoi(argv[1]) < 0) {
     Tcl_AppendResult(irp, "time value must be positive", NULL);
     return TCL_ERROR;
   }
+  if (argc == 4 && atoi(argv[3]) < 0) {
+    Tcl_AppendResult(irp, "count value must be >= 0", NULL);
+    return TCL_ERROR;
+  }
   if (argv[2][0] != '#') {
-    x = add_timer(&utimer, atoi(argv[1]), argv[2], 0L);
+    x = add_timer(&utimer, atoi(argv[1]), (argc == 4 ? atoi(argv[3]) : 1),
+                  argv[2], 0L);
     egg_snprintf(s, sizeof s, "timer%lu", x);
     Tcl_AppendResult(irp, s, NULL);
   }
@@ -430,11 +438,16 @@ static int tcl_strftime STDVAR
 
 static int tcl_myip STDVAR
 {
-  char s[16];
+#ifdef IPV6
+  char s[INET6_ADDRSTRLEN];
+#else
+  char s[INET_ADDRSTRLEN];
+#endif
+
 
   BADARGS(1, 1, "");
 
-  egg_snprintf(s, sizeof s, "%lu", iptolong(getmyip()));
+  getdccaddr(NULL, s, sizeof s);
   Tcl_AppendResult(irp, s, NULL);
   return TCL_OK;
 }
@@ -653,66 +666,61 @@ static int tcl_stripcodes STDVAR
 
   for (p = argv[1]; *p; p++)
     switch (*p) {
-    case 'a':
-      flags |= STRIP_ANSI;
+    case 'c':
+      flags |= STRIP_COLOR;
       break;
     case 'b':
       flags |= STRIP_BOLD;
       break;
-    case 'c':
-      flags |= STRIP_COLOR;
+    case 'r':
+      flags |= STRIP_REVERSE;
+      break;
+    case 'u':
+      flags |= STRIP_UNDERLINE;
+      break;
+    case 'a':
+      flags |= STRIP_ANSI;
       break;
     case 'g':
       flags |= STRIP_BELLS;
       break;
-    case 'r':
-      flags |= STRIP_REV;
+    case 'o':
+      flags |= STRIP_ORDINARY;
       break;
-    case 'u':
-      flags |= STRIP_UNDER;
+    case 'i':
+      flags |= STRIP_ITALICS;
+      break;
+    case '*':
+      flags |= STRIP_ALL;
       break;
     default:
       Tcl_AppendResult(irp, "Invalid strip-flags: ", argv[1], NULL);
       return TCL_ERROR;
     }
 
-  strip_mirc_codes(flags, argv[2]);
-  Tcl_AppendResult(irp, argv[2], NULL);
+  p = Tcl_Alloc(strlen(argv[2]) + 1);
+  strcpy(p, argv[2]);
+  strip_mirc_codes(flags, p);
+  Tcl_SetResult(irp, p, TCL_DYNAMIC);
   return TCL_OK;
 }
 
-#ifdef USE_TCL_OBJ
 static int tcl_md5(cd, irp, objc, objv)
 ClientData cd;
 Tcl_Interp *irp;
 int objc;
 Tcl_Obj *CONST objv[];
 {
-#else
-static int tcl_md5 STDVAR
-{
-#endif /* USE_TCL_OBJ */
   MD5_CTX md5context;
   char digest_string[33], *string;
   unsigned char digest[16];
   int i, len;
 
-#ifdef USE_TCL_OBJ
   if (objc != 2) {
     Tcl_WrongNumArgs(irp, 1, objv, "string");
     return TCL_ERROR;
   }
-#  ifdef USE_TCL_BYTE_ARRAYS
-  string = (char *)Tcl_GetByteArrayFromObj(objv[1], &len);
-#  else
   string = Tcl_GetStringFromObj(objv[1], &len);
-#  endif /* USE_TCL_BYTE_ARRAYS */
-#else /* USE_TCL_OBJ */
-  BADARGS(2, 2, " string");
-
-  string = argv[1];
-  len = strlen(argv[1]);
-#endif /* USE_TCL_OBJ */
 
   MD5_Init(&md5context);
   MD5_Update(&md5context, (unsigned char *) string, len);
@@ -757,12 +765,61 @@ static int tcl_matchstr STDVAR
 }
 
 tcl_cmds tclmisc_objcmds[] = {
-#ifdef USE_TCL_OBJ
   {"md5", tcl_md5},
-#endif /* USE_TCL_OBJ */
   {NULL,     NULL}
 };
 
+static int tcl_status STDVAR
+{
+  char s[15];
+
+  BADARGS(1, 2, " ?type?");
+
+  if ((argc < 2) || !strcmp(argv[1], "cpu")) {
+    Tcl_AppendElement(irp, "cputime");
+    egg_snprintf(s, sizeof s, "%f", getcputime());
+    Tcl_AppendElement(irp, s);
+  }
+  if ((argc < 2) || !strcmp(argv[1], "mem")) {
+    Tcl_AppendElement(irp, "expmem");
+    egg_snprintf(s, sizeof s, "%d", expected_memory());
+    Tcl_AppendElement(irp, s);
+  }
+  if ((argc < 2) || !strcmp(argv[1], "ipv6")) {
+    Tcl_AppendElement(irp, "ipv6");
+#ifdef IPV6
+    Tcl_AppendElement(irp, "enabled");
+#else
+    Tcl_AppendElement(irp, "disabled");
+#endif
+  }
+  if ((argc < 2) || !strcmp(argv[1], "tls")) {
+    Tcl_AppendElement(irp, "tls");
+#ifdef TLS
+    Tcl_AppendElement(irp, SSLeay_version(SSLEAY_VERSION));
+#else
+    Tcl_AppendElement(irp, "disabled");
+#endif
+  }
+  if ((argc < 2) || !strcmp(argv[1], "cache")) {
+    Tcl_AppendElement(irp, "usercache");
+    egg_snprintf(s, sizeof s, "%4.1f", 100.0 *
+             ((float) cache_hit) / ((float) (cache_hit + cache_miss)));
+    Tcl_AppendElement(irp, s);
+  }
+
+  return TCL_OK;
+}
+
+static int tcl_rfcequal STDVAR
+{
+  BADARGS(3, 3, " string1 string2");
+
+  Tcl_AppendResult(irp, !rfc_casecmp(argv[1], argv[2]) ? "1" : "0", NULL);
+
+  return TCL_OK;
+}
+
 tcl_cmds tclmisc_cmds[] = {
   {"logfile",           tcl_logfile},
   {"putlog",             tcl_putlog},
@@ -795,14 +852,13 @@ tcl_cmds tclmisc_cmds[] = {
   {"unloadhelp",     tcl_unloadhelp},
   {"reloadhelp",     tcl_reloadhelp},
   {"duration",         tcl_duration},
-#ifndef USE_TCL_OBJ
-  {"md5",                   tcl_md5},
-#endif /* USE_TCL_OBJ */
   {"binds",               tcl_binds},
   {"callevent",       tcl_callevent},
   {"stripcodes",     tcl_stripcodes},
   {"matchaddr",       tcl_matchaddr},
   {"matchcidr",       tcl_matchcidr},
   {"matchstr",         tcl_matchstr},
+  {"status",             tcl_status},
+  {"rfcequal",         tcl_rfcequal},
   {NULL,                       NULL}
 };
