@@ -1,12 +1,10 @@
 /*
  * userent.c -- handles:
  *   user-entry handling, new style more versatile.
- *
- * $Id: userent.c,v 1.41 2011/02/13 14:19:33 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,6 +24,7 @@
 #include "main.h"
 #include "users.h"
 
+
 extern int noshare;
 extern struct userrec *userlist;
 extern struct dcc_t *dcc;
@@ -34,7 +33,7 @@ extern char whois_fields[];
 
 
 int share_greet = 0;            /* Share greeting info                  */
-static struct user_entry_type *entry_type_list;
+struct user_entry_type *entry_type_list;
 
 
 void init_userent()
@@ -48,6 +47,9 @@ void init_userent()
   add_entry_type(&USERENTRY_PASS);
   add_entry_type(&USERENTRY_HOSTS);
   add_entry_type(&USERENTRY_BOTFL);
+#ifdef TLS
+  add_entry_type(&USERENTRY_FPRINT);
+#endif
 }
 
 void list_type_kill(struct list_type *t)
@@ -126,10 +128,6 @@ int def_set(struct userrec *u, struct user_entry *e, void *buf)
     int l = strlen(string);
     char *i;
 
-    if (l > 160)
-      l = 160;
-
-
     e->u.string = user_realloc(e->u.string, l + 1);
 
     strncpyz(e->u.string, string, l + 1);
@@ -166,6 +164,13 @@ int def_tcl_get(Tcl_Interp * interp, struct userrec *u,
   return TCL_OK;
 }
 
+int def_tcl_append(Tcl_Interp * interp, struct userrec *u,
+                   struct user_entry *e)
+{
+  Tcl_AppendElement(interp, e->u.string);
+  return TCL_OK;
+}
+
 int def_tcl_set(Tcl_Interp * irp, struct userrec *u,
                 struct user_entry *e, int argc, char **argv)
 {
@@ -210,7 +215,8 @@ struct user_entry_type USERENTRY_COMMENT = {
   def_tcl_set,
   def_expmem,
   comment_display,
-  "COMMENT"
+  "COMMENT",
+  def_tcl_append
 };
 
 struct user_entry_type USERENTRY_INFO = {
@@ -227,7 +233,8 @@ struct user_entry_type USERENTRY_INFO = {
   def_tcl_set,
   def_expmem,
   def_display,
-  "INFO"
+  "INFO",
+  def_tcl_append
 };
 
 int pass_set(struct userrec *u, struct user_entry *e, void *buf)
@@ -284,7 +291,8 @@ struct user_entry_type USERENTRY_PASS = {
   pass_tcl_set,
   def_expmem,
   0,
-  "PASS"
+  "PASS",
+  def_tcl_append
 };
 
 static int laston_unpack(struct userrec *u, struct user_entry *e)
@@ -373,11 +381,12 @@ static int laston_tcl_get(Tcl_Interp * irp, struct userrec *u,
   BADARGS(3, 4, " handle LASTON ?channel?");
 
   if (argc == 4) {
-    for (cr = u->chanrec; cr; cr = cr->next)
+    for (cr = u->chanrec; cr; cr = cr->next) {
       if (!rfc_casecmp(cr->channel, argv[3])) {
         Tcl_AppendResult(irp, int_to_base10(cr->laston), NULL);
         break;
       }
+    }
     if (!cr)
       Tcl_AppendResult(irp, "0", NULL);
   } else {
@@ -388,6 +397,22 @@ static int laston_tcl_get(Tcl_Interp * irp, struct userrec *u,
   return TCL_OK;
 }
 
+static int laston_tcl_append(Tcl_Interp *irp, struct userrec *u,
+                             struct user_entry *e)
+{
+  Tcl_DString ds;
+  struct chanuserrec *cr;
+
+  Tcl_DStringInit(&ds);
+  for (cr = u->chanrec; cr; cr = cr->next) {
+    Tcl_DStringAppendElement(&ds, cr->channel);
+    Tcl_DStringAppendElement(&ds, int_to_base10(cr->laston));
+  }
+  Tcl_AppendElement(irp, Tcl_DStringValue(&ds));
+  Tcl_DStringFree(&ds);
+  return TCL_OK;
+}
+
 static int laston_tcl_set(Tcl_Interp * irp, struct userrec *u,
                           struct user_entry *e, int argc, char **argv)
 {
@@ -455,7 +480,8 @@ struct user_entry_type USERENTRY_LASTON = {
   laston_tcl_set,
   laston_expmem,
   0,
-  "LASTON"
+  "LASTON",
+  laston_tcl_append
 };
 
 static int botaddr_unpack(struct userrec *u, struct user_entry *e)
@@ -473,10 +499,27 @@ static int botaddr_unpack(struct userrec *u, struct user_entry *e)
     strncpy(bi->address, p, q - p);
     bi->address[q - p] = 0;
     q++;
+#ifdef TLS
+    if (*q == '+')
+      bi->ssl |= TLS_BOT;
+#endif
     bi->telnet_port = atoi(q);
     if ((q = strchr(q, '/')))
+#ifdef TLS
+    {
+      if (q[1] == '+')
+        bi->ssl |= TLS_RELAY;
       bi->relay_port = atoi(q + 1);
+    }
+#else
+      bi->relay_port = atoi(q + 1);
+#endif
   }
+#ifdef IPV6
+  for (p = bi->address; *p; p++)
+    if (*p == ';')
+      *p = ':';
+#endif
   if (!bi->telnet_port)
     bi->telnet_port = 3333;
   if (!bi->relay_port)
@@ -488,13 +531,23 @@ static int botaddr_unpack(struct userrec *u, struct user_entry *e)
 
 static int botaddr_pack(struct userrec *u, struct user_entry *e)
 {
-  char work[1024];
+  char work[1024], *p, *q = work;
   struct bot_addr *bi;
   int l;
 
   bi = (struct bot_addr *) e->u.extra;
-  l = simple_sprintf(work, "%s:%u/%u", bi->address, bi->telnet_port,
+  for (p = bi->address; *p; p++)
+    if (*p == ':')
+      *q++ = ';';
+    else
+      *q++ = *p;
+#ifdef TLS
+  l = simple_sprintf(q, ":%s%u/%s%u", (bi->ssl & TLS_BOT) ? "+" : "",
+                     bi->telnet_port, (bi->ssl & TLS_RELAY) ? "+" : "",
                      bi->relay_port);
+#else
+  l = simple_sprintf(q, ":%u/%u", bi->telnet_port, bi->relay_port);
+#endif
   e->u.list = user_malloc(sizeof(struct list_type));
   e->u.list->next = NULL;
   e->u.list->extra = user_malloc(l + 1);
@@ -515,12 +568,29 @@ static int botaddr_kill(struct user_entry *e)
 static int botaddr_write_userfile(FILE *f, struct userrec *u,
                                   struct user_entry *e)
 {
+  int ret = 1;
+  char *p, *q, *addr;
   register struct bot_addr *bi = (struct bot_addr *) e->u.extra;
 
-  if (fprintf(f, "--%s %s:%u/%u\n", e->type->name, bi->address,
-              bi->telnet_port, bi->relay_port) == EOF)
-    return 0;
-  return 1;
+  p = bi->address;
+  addr = user_malloc(strlen(bi->address) + 1);
+  for (q = addr; *p; p++)
+    if (*p == ':')
+      *q++ = ';';
+    else
+      *q++ = *p;
+  *q = 0;
+#ifdef TLS
+  if (fprintf(f, "--%s %s:%s%u/%s%u\n", e->type->name, addr,
+      (bi->ssl & TLS_BOT) ? "+" : "", bi->telnet_port, (bi->ssl & TLS_RELAY) ?
+      "+" : "", bi->relay_port) == EOF)
+#else
+  if (fprintf(f, "--%s %s:%u/%u\n", e->type->name, addr,
+      bi->telnet_port, bi->relay_port) == EOF)
+#endif
+    ret = 0;
+  nfree(addr);
+  return ret;
 }
 
 static int botaddr_set(struct userrec *u, struct user_entry *e, void *buf)
@@ -537,22 +607,60 @@ static int botaddr_set(struct userrec *u, struct user_entry *e, void *buf)
     bi = e->u.extra = buf;
   }
   if (bi && !noshare && !(u->flags & USER_UNSHARED)) {
+#ifdef TLS
+    shareout(NULL, "c BOTADDR %s %s %s%d %s%d\n", u->handle, bi->address,
+             (bi->ssl & TLS_BOT) ? "+" : "", bi->telnet_port,
+             (bi->ssl & TLS_RELAY) ? "+" : "", bi->relay_port);
+#else
     shareout(NULL, "c BOTADDR %s %s %d %d\n", u->handle,
              bi->address, bi->telnet_port, bi->relay_port);
+#endif
   }
   return 1;
 }
 
+static int botaddr_tcl_dstring(Tcl_DString *ds, struct user_entry *e)
+{
+  struct bot_addr *bi = (struct bot_addr *)e->u.extra;
+
+  Tcl_DStringAppendElement(ds, bi->address);
+  /* This is safe because there are no special characters in the port */
+  Tcl_DStringAppend(ds, " ", 1);
+#ifdef TLS
+  if (bi->ssl & TLS_BOT)
+    Tcl_DStringAppend(ds, "+", 1);
+#endif
+  Tcl_DStringAppend(ds, int_to_base10(bi->telnet_port), -1);
+  Tcl_DStringAppend(ds, " ", 1);
+#ifdef TLS
+  if (bi->ssl & TLS_RELAY)
+    Tcl_DStringAppend(ds, "+", 1);
+#endif
+  Tcl_DStringAppend(ds, int_to_base10(bi->relay_port), -1);
+  return TCL_OK;
+}
+
 static int botaddr_tcl_get(Tcl_Interp * interp, struct userrec *u,
                            struct user_entry *e, int argc, char **argv)
 {
-  register struct bot_addr *bi = (struct bot_addr *) e->u.extra;
-  char number[20];
+  Tcl_DString ds;
+  Tcl_DStringInit(&ds);
+  botaddr_tcl_dstring(&ds, e);
 
-  sprintf(number, " %d", bi->telnet_port);
-  Tcl_AppendResult(interp, bi->address, number, NULL);
-  sprintf(number, " %d", bi->relay_port);
-  Tcl_AppendResult(interp, number, NULL);
+  Tcl_AppendResult(interp, Tcl_DStringValue(&ds), NULL);
+  Tcl_DStringFree(&ds);
+  return TCL_OK;
+}
+
+static int botaddr_tcl_append(Tcl_Interp * interp, struct userrec *u,
+                           struct user_entry *e)
+{
+  Tcl_DString ds;
+  Tcl_DStringInit(&ds);
+  botaddr_tcl_dstring(&ds, e);
+
+  Tcl_AppendElement(interp, Tcl_DStringValue(&ds));
+  Tcl_DStringFree(&ds);
   return TCL_OK;
 }
 
@@ -572,10 +680,36 @@ static int botaddr_tcl_set(Tcl_Interp * irp, struct userrec *u,
       nfree(bi->address);
     bi->address = user_malloc(strlen(argv[3]) + 1);
     strcpy(bi->address, argv[3]);
-    if (argc > 4)
+    if (argc > 4) {
+#ifdef TLS
+      /* If no user port set, use bot port for both entries */
+      if (*argv[4] == '+') {
+        bi->ssl |= TLS_BOT;
+        if (argc == 5) {
+          bi->ssl |= TLS_RELAY;
+        } 
+      } else {
+        bi->ssl &= ~TLS_BOT;
+        if (argc == 5) {
+          bi->ssl &= ~TLS_RELAY;
+        }
+      }
+#endif
       bi->telnet_port = atoi(argv[4]);
-    if (argc > 5)
+      if (argc == 5) {
+        bi->relay_port = atoi(argv[4]);
+      }
+    }
+    if (argc > 5) {
+#ifdef TLS
+      if (*argv[5] == '+') {
+        bi->ssl |= TLS_RELAY;
+      } else {
+        bi->ssl &= ~TLS_RELAY;
+      }
+#endif
       bi->relay_port = atoi(argv[5]);
+    }
     if (!bi->telnet_port)
       bi->telnet_port = 3333;
     if (!bi->relay_port)
@@ -597,7 +731,12 @@ static void botaddr_display(int idx, struct user_entry *e)
   register struct bot_addr *bi = (struct bot_addr *) e->u.extra;
 
   dprintf(idx, "  ADDRESS: %.70s\n", bi->address);
+#ifdef TLS
+  dprintf(idx, "     users: %s%d, bots: %s%d\n", (bi->ssl & TLS_RELAY) ? "+" : "",
+          bi->relay_port, (bi->ssl & TLS_BOT) ? "+" : "", bi->telnet_port);
+#else
   dprintf(idx, "     users: %d, bots: %d\n", bi->relay_port, bi->telnet_port);
+#endif
 }
 
 static int botaddr_gotshare(struct userrec *u, struct user_entry *e,
@@ -611,6 +750,12 @@ static int botaddr_gotshare(struct userrec *u, struct user_entry *e,
   bi->address = user_malloc(strlen(arg) + 1);
   strcpy(bi->address, arg);
   arg = newsplit(&buf);
+#ifdef TLS
+  if (*arg == '+')
+    bi->ssl |= TLS_BOT;
+  if (*buf == '+')
+    bi->ssl |= TLS_RELAY;
+#endif
   bi->telnet_port = atoi(arg);
   bi->relay_port = atoi(buf);
   if (!bi->telnet_port)
@@ -633,6 +778,9 @@ static int botaddr_dupuser(struct userrec *new, struct userrec *old,
 
       bi2->telnet_port = bi->telnet_port;
       bi2->relay_port = bi->relay_port;
+#ifdef TLS
+      bi2->ssl = bi->ssl;
+#endif
       bi2->address = user_malloc(strlen(bi->address) + 1);
       strcpy(bi2->address, bi->address);
       return set_user(&USERENTRY_BOTADDR, new, bi2);
@@ -655,7 +803,8 @@ struct user_entry_type USERENTRY_BOTADDR = {
   botaddr_tcl_set,
   botaddr_expmem,
   botaddr_display,
-  "BOTADDR"
+  "BOTADDR",
+  botaddr_tcl_append
 };
 
 int xtra_set(struct userrec *u, struct user_entry *e, void *buf)
@@ -687,16 +836,21 @@ int xtra_set(struct userrec *u, struct user_entry *e, void *buf)
     egg_list_delete(&e->u.list, (struct list_type *) old);
     nfree(old->key);
     nfree(old->data);
+    if (old == e->u.extra)
+      e->u.extra = NULL;
     nfree(old);
+    old = NULL;
   }
-  if (old != new && new->data) {
-    if (new->data[0])
+  /* don't do anything when old == new */
+  if (old != new) {
+    if (new->data && new->data[0])
       list_insert((&e->u.extra), new)  /* do not add a ';' here */
-  } else {
-    if (new->data)
-      nfree(new->data);
-    nfree(new->key);
-    nfree(new);
+    else {
+      if (new->data)
+        nfree(new->data);
+      nfree(new->key);
+      nfree(new);
+    }
   }
   return TCL_OK;
 }
@@ -869,6 +1023,21 @@ int xtra_kill(struct user_entry *e)
   return 1;
 }
 
+static int xtra_tcl_dstring(Tcl_DString *ds, int sublist, struct user_entry *e)
+{
+  struct xtra_key *x;
+
+  for (x = e->u.extra; x; x = x->next) {
+    if (sublist)
+      Tcl_DStringStartSublist(ds);
+    Tcl_DStringAppendElement(ds, x->key);
+    Tcl_DStringAppendElement(ds, x->data);
+    if (sublist)
+      Tcl_DStringEndSublist(ds);
+  }
+  return TCL_OK;
+}
+
 static int xtra_tcl_get(Tcl_Interp * irp, struct userrec *u,
                         struct user_entry *e, int argc, char **argv)
 {
@@ -876,24 +1045,30 @@ static int xtra_tcl_get(Tcl_Interp * irp, struct userrec *u,
 
   BADARGS(3, 4, " handle XTRA ?key?");
 
-  if (argc == 4) {
-    for (x = e->u.extra; x; x = x->next)
-      if (!egg_strcasecmp(argv[3], x->key)) {
-        Tcl_AppendResult(irp, x->data, NULL);
-        return TCL_OK;
-      }
+  if (argc == 3) {
+    Tcl_DString ds;
+    Tcl_DStringInit(&ds);
+    xtra_tcl_dstring(&ds, 1, e);
+    Tcl_AppendResult(irp, Tcl_DStringValue(&ds), NULL);
+    Tcl_DStringFree(&ds);
     return TCL_OK;
   }
-  for (x = e->u.extra; x; x = x->next) {
-    char *p;
-    EGG_CONST char *list[2];
+  for (x = e->u.extra; x; x = x->next)
+    if (!egg_strcasecmp(argv[3], x->key)) {
+      Tcl_AppendResult(irp, x->data, NULL);
+      return TCL_OK;
+    }
+  return TCL_OK;
+}
 
-    list[0] = x->key;
-    list[1] = x->data;
-    p = Tcl_Merge(2, list);
-    Tcl_AppendElement(irp, p);
-    Tcl_Free((char *) p);
-  }
+static int xtra_tcl_append(Tcl_Interp *irp, struct userrec *u,
+                           struct user_entry *e)
+{
+  Tcl_DString ds;
+  Tcl_DStringInit(&ds);
+  xtra_tcl_dstring(&ds, 0, e);
+  Tcl_AppendElement(irp, Tcl_DStringValue(&ds));
+  Tcl_DStringFree(&ds);
   return TCL_OK;
 }
 
@@ -925,7 +1100,8 @@ struct user_entry_type USERENTRY_XTRA = {
   xtra_tcl_set,
   xtra_expmem,
   xtra_display,
-  "XTRA"
+  "XTRA",
+  xtra_tcl_append
 };
 
 static int hosts_dupuser(struct userrec *new, struct userrec *old,
@@ -975,7 +1151,7 @@ static void hosts_display(int idx, struct user_entry *e)
   strcpy(s, "  HOSTS: ");
   for (q = e->u.list; q; q = q->next) {
     if (s[0] && !s[9])
-      strcat(s, q->extra);
+      strncat(s, q->extra, (sizeof s - strlen(s) -1));
     else if (!s[0])
       sprintf(s, "         %s", q->extra);
     else {
@@ -1044,6 +1220,20 @@ static int hosts_tcl_get(Tcl_Interp * irp, struct userrec *u,
   return TCL_OK;
 }
 
+static int hosts_tcl_append(Tcl_Interp *irp, struct userrec *u,
+                            struct user_entry *e)
+{
+  Tcl_DString ds;
+  struct list_type *x;
+
+  Tcl_DStringInit(&ds);
+  for (x = e->u.list; x; x = x->next)
+    Tcl_DStringAppendElement(&ds, x->extra);
+  Tcl_AppendElement(irp, Tcl_DStringValue(&ds));
+  Tcl_DStringFree(&ds);
+  return TCL_OK;
+}
+
 static int hosts_tcl_set(Tcl_Interp * irp, struct userrec *u,
                          struct user_entry *e, int argc, char **argv)
 {
@@ -1077,9 +1267,72 @@ struct user_entry_type USERENTRY_HOSTS = {
   hosts_tcl_set,
   hosts_expmem,
   hosts_display,
-  "HOSTS"
+  "HOSTS",
+  hosts_tcl_append
 };
 
+#ifdef TLS
+int fprint_unpack(struct userrec *u, struct user_entry *e)
+{
+  char *tmp;
+
+  tmp = ssl_fpconv(e->u.list->extra, NULL);
+  nfree(e->u.list->extra);
+  e->u.list->extra = NULL;
+  list_type_kill(e->u.list);
+  e->u.string = tmp;
+  return 1;
+}
+
+int fprint_set(struct userrec *u, struct user_entry *e, void *buf)
+{
+  char *fp = buf;
+
+  if (!fp || !fp[0] || (fp[0] == '-')) {
+    if (e->u.string) {
+      nfree(e->u.string);
+      e->u.string = NULL;
+    }
+  } else {
+    fp = ssl_fpconv(buf, e->u.string);
+    if (fp)
+      e->u.string = fp;
+    else
+      return 0;
+  }
+  if (!noshare && !(u->flags & (USER_BOT | USER_UNSHARED)))
+    shareout(NULL, "c FPRINT %s %s\n", u->handle, e->u.string ? e->u.string : "");
+  return 1;
+}
+
+static int fprint_tcl_set(Tcl_Interp * irp, struct userrec *u,
+                        struct user_entry *e, int argc, char **argv)
+{
+  BADARGS(3, 4, " handle FPRINT ?new-fingerprint?");
+
+  fprint_set(u, e, argc == 3 ? NULL : argv[3]);
+  return TCL_OK;
+}
+
+struct user_entry_type USERENTRY_FPRINT = {
+  0,
+  def_gotshare,
+  0,
+  fprint_unpack,
+  def_pack,
+  def_write_userfile,
+  def_kill,
+  def_get,
+  fprint_set,
+  def_tcl_get,
+  fprint_tcl_set,
+  def_expmem,
+  0,
+  "FPRINT",
+  def_tcl_append
+};
+#endif /* TLS */
+
 int egg_list_append(struct list_type **h, struct list_type *i)
 {
   for (; *h; h = &((*h)->next));
@@ -1116,6 +1369,7 @@ int add_entry_type(struct user_entry_type *type)
 
     if (e && e->name) {
       e->type = type;
+
       e->type->unpack(u, e);
       nfree(e->name);
       e->name = NULL;
