@@ -3,12 +3,10 @@
  *   add_q() del_q() str2flags() flags2str() str2chflags() chflags2str()
  *   a bunch of functions to find and change user records
  *   change and check user (and channel-specific) flags
- *
- * $Id: userrec.c,v 1.65 2011/02/13 14:19:33 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -39,7 +37,6 @@ extern char userfile[], ver[], botnetnick[];
 extern time_t now;
 
 int noshare = 1;                   /* don't send out to sharebots   */
-int sort_users = 0;                /* sort the userlist when saving */
 struct userrec *userlist = NULL;   /* user records are stored here  */
 struct userrec *lastuser = NULL;   /* last accessed user record     */
 maskrec *global_bans = NULL, *global_exempts = NULL, *global_invites = NULL;
@@ -77,7 +74,7 @@ void *_user_realloc(void *ptr, int size, const char *file, int line)
 #endif
 }
 
-inline int expmem_mask(struct maskrec *m)
+int expmem_mask(struct maskrec *m)
 {
   int result = 0;
 
@@ -358,7 +355,12 @@ struct userrec *get_user_by_equal_host(char *host)
 }
 
 /* Try: pass_match_by_host("-",host)
- * will return 1 if no password is set for that host
+ * If a '-' is sent as the password, it denotes the intent
+ *   to merely check if a password is set for that user.
+ * Returns 0 if password is set and does not match
+ * Returns 1 if password matches, or if we are
+ *   checking if a password is set and it is not
+ *   (via the '-' char).
  */
 int u_pass_match(struct userrec *u, char *pass)
 {
@@ -367,8 +369,11 @@ int u_pass_match(struct userrec *u, char *pass)
   if (!u)
     return 0;
   cmp = get_user(&USERENTRY_PASS, u);
-  if (!cmp && (!pass[0] || (pass[0] == '-')))
+  if (!cmp && (pass[0] == '-'))
     return 1;
+/* If password is not set in userrecord, or password
+ * is not sent, or '-' is sent
+ */
   if (!cmp || !pass || !pass[0] || (pass[0] == '-'))
     return 0;
   if (u->flags & USER_BOT) {
@@ -561,14 +566,15 @@ void write_userfile(int idx)
   }
   if (!quiet_save)
     putlog(LOG_MISC, "*", USERF_WRITING);
-  if (sort_users)
-    sort_userlist();
+
+  sort_userlist();
   tt = now;
-  strcpy(s1, ctime(&tt));
+  strncpyz(s1, ctime(&tt), sizeof s1);
   fprintf(f, "#4v: %s -- %s -- written %s", ver, botnetnick, s1);
   ok = 1;
+  /* Add all users except the -tn user */
   for (u = userlist; u && ok; u = u->next)
-    if (!write_user(u, f, idx))
+    if (egg_strcasecmp(u->handle, EGG_BG_HANDLE) && !write_user(u, f, idx))
       ok = 0;
   if (!ok || !write_ignores(f, -1) || fflush(f)) {
     putlog(LOG_MISC, "*", "%s (%s)", USERF_ERRWRITE, strerror(ferror(f)));
@@ -589,6 +595,9 @@ int change_handle(struct userrec *u, char *newh)
 
   if (!u)
     return 0;
+  /* Don't allow the -tn handle to be changed */
+  if (!egg_strcasecmp(u->handle, EGG_BG_HANDLE))
+    return 0;
   /* Nothing that will confuse the userfile */
   if (!newh[1] && strchr(BADHANDCHARS, newh[0]))
     return 0;
@@ -914,7 +923,7 @@ void user_del_chan(char *dname)
 int check_conflags(struct flag_record *fr, int md)
 {
   if (!glob_owner(*fr))
-    md &= ~(LOG_RAW | LOG_SRVOUT | LOG_BOTNET | LOG_BOTSHARE);
+    md &= ~(LOG_RAW | LOG_SRVOUT | LOG_BOTNETIN | LOG_BOTNETOUT | LOG_BOTSHRIN | LOG_BOTSHROUT);
   if (!glob_master(*fr)) {
     md &= ~(LOG_FILES | LOG_LEV1 | LOG_LEV2 | LOG_LEV3 | LOG_LEV4 |
             LOG_LEV5 | LOG_LEV6 | LOG_LEV7 | LOG_LEV8 | LOG_DEBUG |
@@ -923,6 +932,6 @@ int check_conflags(struct flag_record *fr, int md)
       md &= ~(LOG_MISC | LOG_CMDS);
   }
   if (!glob_botmast(*fr))
-    md &= ~LOG_BOTS;
+    md &= ~(LOG_BOTS | LOG_BOTMSG);
   return md;
 }
