@@ -9,12 +9,10 @@
  *   reading the user file
  *
  * dprintf'ized, 9nov1995
- *
- * $Id: users.c,v 1.60 2011/02/13 14:19:33 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -36,7 +34,6 @@
 #include "chan.h"
 #include "modules.h"
 #include "tandem.h"
-char natip[121] = "";
 
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -695,16 +692,17 @@ int readuserfile(char *file, struct userrec **ret)
   noshare = noxtra = 1;
   /* read opening comment */
   s = buf;
-  fgets(s, 180, f);
-  if (s[1] < '4') {
-    fatal(USERF_OLDFMT, 0);
+  if (fgets(s, 180, f) != NULL) {
+    if (s[0] == '#' && s[1] < '4' && s[1] >= '0' && s[2] == 'v') {
+      fatal(USERF_OLDFMT, 0);
+    }
+    if (s[0] != '#' || s[1] > '4' || s[2] != 'v')
+      fatal(USERF_INVALID, 0);
   }
-  if (s[1] > '4')
-    fatal(USERF_INVALID, 0);
-  while (!feof(f)) {
-    s = buf;
-    fgets(s, 511, f);
-    if (!feof(f)) {
+  /* don't check for feof after fgets, skips last line if it has no \n (ie on windows) */
+  while (!feof(f) && fgets(buf, sizeof buf, f) != NULL) {
+      /* for the sake of git blame/history, I'm not re-indenting the next 255 lines*/
+      s = buf;
       if (s[0] != '#' && s[0] != ';' && s[0]) {
         code = newsplit(&s);
         rmspace(s);
@@ -833,13 +831,15 @@ int readuserfile(char *file, struct userrec **ret)
             /* NOTE only remove exempts for when getting a userfile
              * from another bot & that channel is shared */
             cst = findchan_by_dname(lasthand);
-            if ((*ret == userlist) || channel_shared(cst)) {
-              clear_masks(cst->exempts);
-              cst->exempts = NULL;
-            } else {
-              /* otherwise ignore any exempts for this channel */
-              cst = NULL;
-              lasthand[0] = 0;
+            if (cst) {
+              if ((*ret == userlist) || channel_shared(cst)) {
+                clear_masks(cst->exempts);
+                cst->exempts = NULL;
+              } else {
+                /* otherwise ignore any exempts for this channel */
+                cst = NULL;
+                lasthand[0] = 0;
+              }
             }
           }
         } else if (!strncmp(code, "$$", 2)) {
@@ -891,7 +891,7 @@ int readuserfile(char *file, struct userrec **ret)
             if (!ok) {
               ue = user_malloc(sizeof(struct user_entry));
 
-              ue->name = user_malloc(strlen(code + 1));
+              ue->name = user_malloc(strlen(code) - 1);
               ue->type = NULL;
               strcpy(ue->name, code + 2);
               ue->u.list = user_malloc(sizeof(struct list_type));
@@ -936,7 +936,7 @@ int readuserfile(char *file, struct userrec **ret)
             } else {
               fr.match = FR_GLOBAL;
               break_down_flags(attr, &fr, 0);
-              strcpy(lasthand, code);
+              strncpyz(lasthand, code, sizeof lasthand);
               cst = NULL;
               if (strlen(code) > HANDLEN)
                 code[HANDLEN] = 0;
@@ -957,7 +957,6 @@ int readuserfile(char *file, struct userrec **ret)
           }
         }
       }
-    }
   }
   fclose(f);
   (*ret) = bu;
@@ -1072,8 +1071,7 @@ void autolink_cycle(char *start)
             char *p = MISC_REJECTED;
 
             /* we're directly connected to the offending bot?! (shudder!) */
-            putlog(LOG_BOTS, "*", "%s %s", BOT_REJECTING, dcc[i].nick);
-            chatout("*** %s bot %s\n", p, dcc[i].nick);
+            putlog(LOG_BOTS, "*", "%s bot %s.", p, dcc[i].nick);
             botnet_send_unlinked(i, dcc[i].nick, p);
             dprintf(i, "bye %s\n", BOT_REJECTING);
             killsock(dcc[i].sock);
