@@ -1,12 +1,10 @@
 /*
  * users.h
  *   structures and definitions used by users.c and userrec.c
- *
- * $Id: users.h,v 1.21 2011/02/13 14:19:33 simple Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2011 Eggheads Development Team
+ * Copyright (C) 1999 - 2018 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -62,6 +60,7 @@ struct user_entry_type {
   int (*expmem) (struct user_entry *);
   void (*display) (int idx, struct user_entry *);
   char *name;
+  int (*tcl_append) (Tcl_Interp *, struct userrec *, struct user_entry *);
 };
 
 
@@ -69,6 +68,9 @@ struct user_entry_type {
 extern struct user_entry_type USERENTRY_COMMENT, USERENTRY_LASTON,
   USERENTRY_XTRA, USERENTRY_INFO, USERENTRY_BOTADDR, USERENTRY_HOSTS,
   USERENTRY_PASS, USERENTRY_BOTFL;
+#ifdef TLS
+extern struct user_entry_type USERENTRY_FPRINT;
+#endif
 #endif
 
 
@@ -81,6 +83,11 @@ struct bot_addr {
   int telnet_port;
   int relay_port;
   char *address;
+#ifdef TLS
+#  define TLS_BOT 1
+#  define TLS_RELAY 2
+  int ssl;
+#endif
 };
 
 struct user_entry {
